Package: DataQualityDashboard
Type: Package
Title: Execute and View Data Quality Checks on OMOP CDM Database
Version: 2.8.9
Date: 2026-05-17
Authors@R: c(
  person("Katy", "Sadowski", email = "sadowski@ohdsi.org", role = c("aut", "cre")),
  person("Clair", "Blacketer", role = c("aut")),
  person("Maxim", "Moinat", role = c("aut")),
  person("Ajit", "Londhe", role = c("aut")),
  person("Anthony", "Sena", role = c("aut")),
  person("Anthony", "Molinaro", role = c("aut")),
  person("Frank", "DeFalco", role = c("aut")),
  person("Pavel", "Grafkin", role = c("aut"))
  )
Author: Katy Sadowski [aut, cre],
  Clair Blacketer [aut],
  Maxim Moinat [aut],
  Ajit Londhe [aut],
  Anthony Sena [aut],
  Anthony Molinaro [aut],
  Frank DeFalco [aut],
  Pavel Grafkin [aut]
Maintainer: Katy Sadowski <sadowski@ohdsi.org>
Description: Assesses data quality in Observational Medical Outcomes Partnership Common Data Model (OMOP CDM) databases. Executes data quality checks and provides an R 'shiny' application to view the results.
License: Apache License 2.0
Config/build/clean-inst-doc: FALSE
VignetteBuilder: knitr
Config/testthat/edition: 3
URL: https://github.com/OHDSI/DataQualityDashboard
BugReports: https://github.com/OHDSI/DataQualityDashboard/issues
Depends: R (>= 3.2.2), DatabaseConnector (>= 2.0.2)
Imports: magrittr, ParallelLogger, dplyr, jsonlite, rJava, SqlRender
        (>= 1.10.1), plyr, stringr, rlang, tidyselect, readr
Suggests: testthat, knitr, rmarkdown, markdown, shiny, ggplot2, Eunomia
        (>= 2.0.0), R.utils, devtools
RoxygenNote: 7.3.2
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2026-05-18 00:58:09 UTC; katysadowski
Repository: CRAN
Date/Publication: 2026-05-18 07:00:23 UTC
Built: R 4.5.2; ; 2026-05-18 08:56:50 UTC; unix

# Generated by roxygen2: do not edit by hand

export(convertJsonResultsFileCase)
export(executeDqChecks)
export(listDqChecks)
export(reEvaluateThresholds)
export(viewDqDashboard)
export(writeDBResultsToJson)
export(writeJsonResultsToCsv)
export(writeJsonResultsToTable)
import(DatabaseConnector)
import(magrittr)
importFrom(SqlRender,camelCaseToSnakeCase)
importFrom(SqlRender,snakeCaseToCamelCase)
importFrom(dplyr,case_when)
importFrom(dplyr,mutate)
importFrom(dplyr,rename_with)
importFrom(jsonlite,fromJSON)
importFrom(jsonlite,parse_json)
importFrom(jsonlite,toJSON)
importFrom(magrittr,"%>%")
importFrom(readr,local_edition)
importFrom(readr,read_csv)
importFrom(rlang,.data)
importFrom(stats,na.omit)
importFrom(stats,setNames)
importFrom(stringr,regex)
importFrom(stringr,str_detect)
importFrom(tidyselect,all_of)
importFrom(tools,file_path_sans_ext)
importFrom(utils,packageVersion)
importFrom(utils,write.table)

DataQualityDashboard
====================

<!-- badges: start -->
[![codecov](https://codecov.io/gh/OHDSI/DataQualityDashboard/branch/main/graph/badge.svg)](https://app.codecov.io/gh/OHDSI/DataQualityDashboard)
[![Build Status](https://github.com/OHDSI/DataQualityDashboard/workflows/R-CMD-check/badge.svg)](https://github.com/OHDSI/DataQualityDashboard/actions?query=workflow%3AR-CMD-check)
<!-- badges: end -->

DataQualityDashboard is part of [HADES](https://ohdsi.github.io/Hades/).

The goal of the Data Quality Dashboard (DQD) project is to design and develop an open-source tool to expose and evaluate observational data quality. 

Introduction
============

This package will run a series of data quality checks against an OMOP CDM instance (currently supports v5.4, v5.3 and v5.2). It systematically runs the checks, evaluates the checks against some pre-specified threshold, and then communicates what was done in a transparent and easily understandable way. 

Overview
========

The quality checks were organized according to the Kahn Framework<sup id="kahn">[1](#f1)</sup> which uses a system of categories and contexts that represent strategies for assessing data quality. For an introduction to the kahn framework please click [here](https://pmc.ncbi.nlm.nih.gov/articles/PMC5051581/). 

Using this framework, the Data Quality Dashboard takes a systematic-based approach to running data quality checks. Instead of writing thousands of individual checks, we use “data quality check types”. These “check types” are more general, parameterized data quality checks into which OMOP tables, fields, and concepts can be substituted to represent a singular data quality idea. For example, one check type might be written as 

*The number and percent of records with a value in the **cdmFieldName** field of the **cdmTableName** table less than **plausibleValueLow**.*

This would be considered an atemporal plausibility verification check because we are looking for implausibly low values in some field based on internal knowledge. We can use this check type to substitute in values for **cdmFieldName**, **cdmTableName**, and **plausibleValueLow** to create a unique data quality check. If we apply it to PERSON.YEAR_OF_BIRTH here is how that might look: 

*The number and percent of records with a value in the **year_of_birth** field of the **PERSON** table less than **1850**.* 

And, since it is parameterized, we can similarly apply it to DRUG_EXPOSURE.days_supply: 

*The number and percent of records with a value in the **days_supply** field of the **DRUG_EXPOSURE** table less than **0**.* 

Version 1 of the tool includes 24 different check types organized into Kahn contexts and categories. Additionally, each data quality check type is considered either a table check, field check, or concept-level check. Table-level checks are those evaluating the table at a high-level without reference to individual fields, or those that span multiple event tables. These include checks making sure required tables are present or that at least some of the people in the PERSON table have records in the event tables. Field-level checks are those related to specific fields in a table. The majority of the check types in version 1 are field-level checks. These include checks evaluating primary key relationship and those investigating if the concepts in a field conform to the specified domain. Concept-level checks are related to individual concepts. These include checks looking for gender-specific concepts in persons of the wrong gender and plausible values for measurement-unit pairs. For a detailed description and definition of each check type please click [here](https://ohdsi.github.io/DataQualityDashboard/articles/CheckTypeDescriptions). 

After systematically applying the 24 check types to an OMOP CDM version approximately 4,000 individual data quality checks are resolved, run against the database, and evaluated based on a pre-specified threshold. The R package then creates a json object that is read into an RShiny application to view the results.


<img src="man/figures/dqDashboardScreenshot.png"/>


Features
========
- Utilizes configurable data check thresholds
- Analyzes data in the OMOP Common Data Model format for all data checks
- Produces a set of data check results with supplemental investigation assets.

Data Requirements
=================

Prior to execution the DataQualityDashboard package requires that the CDM_SOURCE table is properly populated.  The following table is a guide to the expected contents of the CDM_SOURCE table.

cdmFieldName | userGuidance | etlConventions
-- | -- | --
cdm_source_name | The name of the CDM instance. | Descriptive name for the source   data.
cdm_source_abbreviation | The abbreviation of the CDM   instance. | The abbreviation should   consistent for different release from the same source.
cdm_holder | The holder of the CDM instance. | The institution that controls   access to the data.  If possible   include contact information for who to contact to request access to the data.
source_description | The description of the CDM   instance. | Add notes, caveats, special   characteristics about the source data that would not be assumed from the   general descriptive name.  This   description intended to help analysts determine if the data is suitable for   the problem they are studying.
source_documentation_reference | Reference to where one can find   documentation about the source data. | Can include URLs, file name,   source data experts contact information (if they agree to it)
cdm_etl_reference | Reference to where one can find   documentation about the source to ETL into OMOP CDM. | Assuming there is a document or   files (such as Rabbit in the Hat) describing the ETL.  May be the location of the ETL source and   documentation repository.
source_release_date | The release date of the source   data. | When the source data was made   available for ETL'ing.  For sites doing   incremental updates, the date the last increment made available.  This implies that for sites doing incremental   updates the CDM Source table should be updated to reflect that changes were   made to the CDM.
cdm_release_date | The release date of the CDM   instance. | When the source data was made   available for general use.  For sites   doing incremental updates, this implies that the CDM Source table should be   updated to reflect that changes were made to the CDM.
cdm_version | Identifies the CDM version | Enter the numeric portion of the   version, e.g. 5.4
cdm_version_concept_id | The Concept Id representing the   version of the CDM. | SELECT concept_id WHERE domain =   Metadata and vocabulary_id = CDM and concept_code like %[numeric portion of   the version]%
vocabulary_version | The vocabulary version used in   the ETL | Obtained by SELECT   vocabulary_version FROM vocabulary WHERE vocabulary_id = 'None'

Technology
==========
DataQualityDashboard is an R package 

System Requirements
===================
Requires R (version 3.2.2 or higher). Requires [DatabaseConnector](https://github.com/OHDSI/DatabaseConnector) (version 2.0.2 or higher).

A variety of database platforms are supported, as documented [here](https://ohdsi.github.io/Hades/supportedPlatforms.html). 

Note that while data quality check threshold files are provided for OMOP CDM versions 5.2, 5.3, and 5.4, the package is currently only tested against versions 5.3 and 5.4.

Installation
=============
1. See the instructions [here](https://ohdsi.github.io/Hades/rSetup.html) for configuring your R environment, including RTools and Java.

2. In R, install DataQualityDashboard from CRAN:

  ```r
  install.packages("DataQualityDashboard")
  ```

   To install from GitHub instead:

  ```r
  install.packages("remotes")
  remotes::install_github("OHDSI/DataQualityDashboard")
  ```

User Documentation
==================
Documentation can be found on the [package website](https://ohdsi.github.io/DataQualityDashboard/index.html).

PDF versions of the documentation are also available:

* Vignette: [Add a New Data Quality Check](https://github.com/OHDSI/DataQualityDashboard/raw/main/inst/doc/AddNewCheck.pdf)
* Vignette: [Check Status Descriptions](https://github.com/OHDSI/DataQualityDashboard/raw/main/inst/doc/CheckStatusDefinitions.pdf)
* Vignette: [Running the DQD on a Cohort](https://github.com/OHDSI/DataQualityDashboard/raw/main/inst/doc/DqdForCohorts.pdf)
* Vignette: [Failure Thresholds and How to Change Them](https://github.com/OHDSI/DataQualityDashboard/raw/main/inst/doc/Thresholds.pdf)
* Vignette: [SqlOnly Mode](https://github.com/OHDSI/DataQualityDashboard/raw/main/inst/doc/SqlOnly.pdf)
* Package manual: [DataQualityDashboard manual](https://github.com/OHDSI/DataQualityDashboard/raw/main/inst/doc/DataQualityDashboard.pdf) 

Support
=======

* Developer questions/comments/feedback: <a href="http://forums.ohdsi.org/c/developers">OHDSI Forum</a>
* We use the <a href="https://github.com/OHDSI/DataQualityDashboard/issues">GitHub issue tracker</a> for all bugs/issues/enhancements 
 
License
=======
DataQualityDashboard is licensed under Apache License 2.0

Development
===========
DataQualityDashboard is being developed in R Studio.

### Development status

DataQualityDashboard latest release (representing code in the `main` branch) is ready for use. 

# Acknowledgements
- This project is supported in part through the National Science Foundation grant IIS 1251151.

<b id="f1">1</b> Kahn, M.G., et al., A Harmonized Data Quality Assessment Terminology and Framework for the Secondary Use of Electronic Health Record Data. EGEMS (Wash DC), 2016. 4(1): p. 1244. [↩](#kahn)

library(shiny)
server <- function(input, output, session) {
  observe({
    jsonPath <- Sys.getenv("jsonPath")
    results <- DataQualityDashboard::convertJsonResultsFileCase(jsonPath, writeToFile = FALSE, targetCase = "camel")
    results <- jsonlite::parse_json(jsonlite::toJSON(results))
    session$sendCustomMessage("results", results)
  })
}

ui <- fluidPage(
  suppressDependencies("bootstrap"),
  shiny::htmlTemplate(filename = "www/index.html"),
  tags$head(
    tags$script(src = "js/loadResults.js"),
    tags$script("Shiny.addCustomMessageHandler('results', loadResults);")
  )
)

shiny::shinyApp(ui = ui, server = server)

# Copyright 2026 Observational Health Data Sciences and Informatics
#
# This file is part of DataQualityDashboard
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Internal function to read threshold files
#'
#' @param checkThresholdLoc The location of the threshold file
#' @param defaultLoc The default location of the threshold file
#'
#' @return A dataframe containing the threshold data
#'
#' @keywords internal
.readThresholdFile <- function(checkThresholdLoc, defaultLoc) {
  thresholdFile <- checkThresholdLoc

  if (checkThresholdLoc == "default") {
    thresholdFile <- system.file(
      "csv",
      defaultLoc,
      package = "DataQualityDashboard"
    )
  }

  colspec <- readr::spec_csv(thresholdFile)

  # plausibleUnitConceptIds is a comma-separated list of concept ids, but it is being interpreted as col_double()
  if ("plausibleUnitConceptIds" %in% names(colspec$cols)) {
    colspec$cols$plausibleUnitConceptIds <- readr::col_character()
  }

  result <- read_csv(
    file = thresholdFile,
    col_types = colspec,
    na = c(" ", "")
  )
  result <- as.data.frame(result)
  return(result)
}

# Copyright 2026 Observational Health Data Sciences and Informatics
#
# This file is part of DataQualityDashboard
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Write DQD results to json
#'
#' @param result           A DQD results object (list)
#' @param outputFolder     The output folder
#' @param outputFile       The output filename
#'
#' @return NULL (writes results to JSON file)
#'
#' @keywords internal

.writeResultsToJson <- function(result,
                                outputFolder,
                                outputFile) {
  resultJson <- jsonlite::toJSON(result)

  resultFilename <- file.path(outputFolder, outputFile)
  result$outputFile <- outputFile

  ParallelLogger::logInfo(sprintf("Writing results to file: %s", resultFilename))
  write(resultJson, file = resultFilename)
}

#' Internal function to write the check results to a table in the database. Requires write access to the database
#'
#' @param connectionDetails         A connectionDetails object for connecting to the CDM database
#' @param resultsDatabaseSchema     The fully qualified database name of the results schema
#' @param checkResults              A dataframe containing the fully summarized data quality check results
#' @param writeTableName            The name of the table to be written to the database. Default is "dqdashboard_results".
#' @param cohortDefinitionId        (OPTIONAL) The cohort definition id for the cohort you wish to run the DQD on. The package assumes a standard OHDSI cohort table called 'Cohort'
#'                                  with the fields cohort_definition_id and subject_id.
#'
#' @return NULL (writes results to database table)
#'
#' @keywords internal

.writeResultsToTable <- function(connectionDetails,
                                 resultsDatabaseSchema,
                                 checkResults,
                                 writeTableName,
                                 cohortDefinitionId) {
  connection <- DatabaseConnector::connect(connectionDetails = connectionDetails)
  on.exit(DatabaseConnector::disconnect(connection = connection))

  if (length(cohortDefinitionId > 0)) {
    tableName <- sprintf("%s.%s_%s", resultsDatabaseSchema, writeTableName, cohortDefinitionId)
  } else {
    tableName <- sprintf("%s.%s", resultsDatabaseSchema, writeTableName)
  }

  ParallelLogger::logInfo(sprintf("Writing results to table %s", tableName))

  ddl <- SqlRender::loadRenderTranslateSql(
    sqlFilename = "result_dataframe_ddl.sql",
    packageName = "DataQualityDashboard",
    tableName = tableName,
    dbms = connectionDetails$dbms
  )

  DatabaseConnector::executeSql(
    connection = connection,
    sql = ddl,
    progressBar = TRUE
  )

  # convert column names to snake case, omitting the checkId column,
  # which has no underscore in the results table DDL
  for (i in seq_len(ncol(checkResults))) {
    if (colnames(checkResults)[i] == "checkId") {
      colnames(checkResults)[i] <- tolower(colnames(checkResults)[i])
    } else {
      colnames(checkResults)[i] <- SqlRender::camelCaseToSnakeCase(colnames(checkResults)[i])
    }
  }

  tryCatch(
    expr = {
      DatabaseConnector::insertTable(
        connection = connection, tableName = tableName, data = checkResults,
        dropTableIfExists = FALSE, createTable = FALSE, tempTable = FALSE
      )
      ParallelLogger::logInfo("Finished writing table")
    },
    error = function(e) {
      warning(sprintf("Writing table failed: %s", e$message))
      ParallelLogger::logError(sprintf("Writing table failed: %s", e$message))
    }
  )
}

#' Internal function to write the check results to a csv file.
#'
#' @param checkResults              A dataframe containing the fully summarized data quality check results
#' @param csvPath                   The path where the csv file should be written
#' @param columns                   The columns to be included in the csv file. Default is all columns in the checkResults dataframe.
#' @param delimiter                 The delimiter for the file. Default is comma.
#'
#' @return NULL (writes results to CSV file)
#'
#' @keywords internal

.writeResultsToCsv <- function(checkResults,
                               csvPath,
                               columns = c(
                                 "checkId", "failed", "passed",
                                 "isError", "notApplicable",
                                 "checkName", "checkDescription",
                                 "thresholdValue", "notesValue",
                                 "checkLevel", "category",
                                 "subcategory", "context",
                                 "checkLevel", "cdmTableName",
                                 "cdmFieldName", "conceptId",
                                 "unitConceptId", "numViolatedRows",
                                 "pctViolatedRows", "numDenominatorRows",
                                 "executionTime", "notApplicableReason",
                                 "error", "queryText"
                               ),
                               delimiter = ",") {
  tryCatch(
    expr = {
      ParallelLogger::logInfo(sprintf("Writing results to CSV file %s", csvPath))
      columns <- intersect(union(c("checkId", "failed", "passed", "isError", "notApplicable"), columns), colnames(checkResults))
      if (is.element("queryText", columns)) {
        checkResults$queryText <- stringr::str_replace_all(checkResults$queryText, "\n", " ")
        checkResults$queryText <- stringr::str_replace_all(checkResults$queryText, "\r", " ")
        checkResults$queryText <- stringr::str_replace_all(checkResults$queryText, "\t", " ")
      }
      if (is.element("error", columns)) {
        checkResults$error <- stringr::str_replace_all(checkResults$error, "\n", " ")
        checkResults$error <- stringr::str_replace_all(checkResults$error, "\r", " ")
        checkResults$error <- stringr::str_replace_all(checkResults$error, "\t", " ")
      }
      write.table(dplyr::select(checkResults, all_of(columns)), file = csvPath, sep = delimiter, row.names = FALSE, na = "")
      ParallelLogger::logInfo("Finished writing to CSV file")
    },
    error = function(e) {
      ParallelLogger::logError(sprintf("Writing to CSV file failed: %s", e$message))
    }
  )
}

library(testthat)

test_that("evaluateThresholds handles missing table/field errors correctly for cdmField/cdmTable", {
  # Create empty objects so thresholdFieldExists will be FALSE
  tableChecks <- data.frame()
  fieldChecks <- data.frame()
  conceptChecks <- data.frame()

  # Simulate a check result for cdmField with a missing table error
  checkResults <- data.frame(
    checkName = "cdmField",
    cdmTableName = "FOO",
    cdmFieldName = "bar",
    checkLevel = "FIELD",
    error = "relation \"foo\" does not exist",
    failed = 0,
    isError = 0,
    numViolatedRows = 0,
    pctViolatedRows = 0,
    stringsAsFactors = FALSE
  )

  # Call the evaluateThresholds function
  result <- DataQualityDashboard:::.evaluateThresholds(checkResults, tableChecks, fieldChecks, conceptChecks)
  expect_equal(result$failed, 1)
  expect_equal(result$isError, 0)

  # Simulate a check result for cdmTable with a missing table error
  checkResults <- data.frame(
    checkName = "cdmTable",
    cdmTableName = "FOO",
    cdmFieldName = NA,
    checkLevel = "TABLE",
    error = "relation \"foo\" does not exist",
    failed = 0,
    isError = 0,
    numViolatedRows = 0,
    pctViolatedRows = 0,
    stringsAsFactors = FALSE
  )

  result <- DataQualityDashboard:::.evaluateThresholds(checkResults, tableChecks, fieldChecks, conceptChecks)
  expect_equal(result$failed, 1)
  expect_equal(result$isError, 0)
})

test_that("evaluateThresholds handles missing table/field errors correctly for other checks", {
  # Create empty objects so thresholdFieldExists will be FALSE
  tableChecks <- data.frame()
  fieldChecks <- data.frame()
  conceptChecks <- data.frame()

  # Simulate a check result for measurePersonCompleteness with a missing table error
  checkResults <- data.frame(
    checkName = "measurePersonCompleteness",
    cdmTableName = "FOO",
    cdmFieldName = NA,
    checkLevel = "TABLE",
    error = "relation \"foo\" does not exist",
    failed = 0,
    isError = 0,
    numViolatedRows = 0,
    pctViolatedRows = 0,
    stringsAsFactors = FALSE
  )

  result <- DataQualityDashboard:::.evaluateThresholds(checkResults, tableChecks, fieldChecks, conceptChecks)
  expect_equal(result$isError, 1)
  expect_equal(result$failed, 0)

  # Simulate a check result for measureValueCompleteness with a missing field error
  checkResults <- data.frame(
    checkName = "measureValueCompleteness",
    cdmTableName = "OBSERVATION",
    cdmFieldName = "bar",
    checkLevel = "FIELD",
    error = "column \"bar\" does not exist",
    failed = 0,
    isError = 0,
    numViolatedRows = 0,
    pctViolatedRows = 0,
    stringsAsFactors = FALSE
  )

  result <- DataQualityDashboard:::.evaluateThresholds(checkResults, tableChecks, fieldChecks, conceptChecks)
  expect_equal(result$isError, 1)
  expect_equal(result$failed, 0)
})

test_that("evaluateThresholds handles non-missing table/field errors correctly", {
  # Create empty objects so thresholdFieldExists will be FALSE
  tableChecks <- data.frame()
  fieldChecks <- data.frame()
  conceptChecks <- data.frame()

  # Simulate a check result with a general SQL error (not missing table/field)
  checkResults <- data.frame(
    checkName = "cdmTable",
    cdmTableName = "PERSON",
    cdmFieldName = NA,
    checkLevel = "TABLE",
    error = "syntax error at or near \"SELECT\"",
    failed = 0,
    isError = 0,
    numViolatedRows = 0,
    pctViolatedRows = 0,
    stringsAsFactors = FALSE
  )

  result <- DataQualityDashboard:::.evaluateThresholds(checkResults, tableChecks, fieldChecks, conceptChecks)
  expect_equal(result$isError, 1)
  expect_equal(result$failed, 0)
})

# Copyright 2026 Observational Health Data Sciences and Informatics
#
# This file is part of DataQualityDashboard
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


#' View DQ Dashboard
#'
#' @param jsonPath       The fully-qualified path to the JSON file produced by  \code{\link{executeDqChecks}}
#' @param launch.browser Passed on to \code{shiny::runApp}
#' @param display.mode   Passed on to \code{shiny::runApp}
#' @param ...            Extra parameters for shiny::runApp() like "port" or "host"
#'
#' @return NULL (launches Shiny application)
#'
#' @importFrom jsonlite toJSON parse_json
#'
#' @export
viewDqDashboard <- function(jsonPath, launch.browser = NULL, display.mode = NULL, ...) {
  if (!requireNamespace("shiny", quietly = TRUE)) {
    stop("The 'shiny' package must be installed to use this function. Please install it with: install.packages(\"shiny\")", call. = FALSE)
  }

  Sys.setenv(jsonPath = jsonPath)
  appDir <- system.file("shinyApps", package = "DataQualityDashboard")

  if (is.null(display.mode)) {
    display.mode <- "normal"
  }

  if (is.null(launch.browser)) {
    launch.browser <- TRUE
  }

  shiny::runApp(appDir = appDir, launch.browser = launch.browser, display.mode = display.mode, ...)
}

# Copyright 2026 Observational Health Data Sciences and Informatics
#
# This file is part of DataQualityDashboard
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Determines if all checks are present expected to calculate the 'Not Applicable' status
#'
#' @param checkResults A dataframe containing the results of the data quality checks
#'
#' @return A logical value indicating whether all required checks are present
#'
#' @keywords internal
.hasNAchecks <- function(checkResults) {
  checkNames <- unique(checkResults$checkName)
  return(.containsNAchecks(checkNames))
}

#' Determines if all checks required for 'Not Applicable' status are in the checkNames
#'
#' @param checkNames A character vector of check names
#'
#' @return A logical value indicating whether all required checks are present
#'
#' @keywords internal
.containsNAchecks <- function(checkNames) {
  naCheckNames <- c("cdmTable", "cdmField", "measureValueCompleteness")
  missingNAChecks <- !(naCheckNames %in% checkNames)
  if (any(missingNAChecks)) {
    return(FALSE)
  }
  return(TRUE)
}

#' Applies the 'Not Applicable' status to a single check
#'
#' @param x Results from a single check
#'
#' @return A numeric value (0 or 1) indicating whether the check is not applicable
#'
#' @keywords internal
.applyNotApplicable <- function(x) {
  # Special rule for measurePersonCompleteness
  if (x$checkName == "measurePersonCompleteness") {
    if (x$tableIsMissing) {
      return(1)
    } else {
      return(0)
    }
  }

  # Special case: cdmTable should never be marked as NA, no matter what
  if (x$checkName == "cdmTable") {
    return(0)
  }

  # Special case: cdmField should only be NA if table is missing, otherwise never NA
  if (x$checkName == "cdmField") {
    if (x$tableIsMissing) {
      return(1)
    } else {
      return(0)
    }
  }

  # Not applicable if table or field is missing (for regular checks)
  if (x$tableIsMissing || x$fieldIsMissing) {
    return(1)
  }

  # Errors not related to a missing table or field should not be marked NA
  if (x$isError == 1) {
    return(0)
  }

  if (x$tableIsEmpty) {
    return(1)
  }

  # No NA status for measureValueCompleteness if field is empty
  if (x$checkName == "measureValueCompleteness") {
    return(0)
  }

  if (any(x$fieldIsEmpty, x$conceptIsMissing, x$conceptAndUnitAreMissing, na.rm = TRUE)) {
    return(1)
  }

  return(0)
}

#' Determines if check should be notApplicable and the notApplicableReason
#'
#' @param checkResults A dataframe containing the results of the data quality checks
#'
#' @return A dataframe with updated check results including notApplicable status and reasons
#'
#' @keywords internal
.calculateNotApplicableStatus <- function(checkResults) {
  # Look up missing tables and add variable tableIsMissing to checkResults
  missingTables <- checkResults %>%
    dplyr::filter(
      .data$checkName == "cdmTable"
    ) %>%
    dplyr::mutate(
      .data$cdmTableName,
      tableIsMissing = .data$failed == 1,
      .keep = "none"
    )

  # Look up missing fields and add variable fieldIsMissing to checkResults
  missingFields <- checkResults %>%
    dplyr::filter(
      .data$checkName == "cdmField"
    ) %>%
    dplyr::mutate(
      .data$cdmTableName,
      .data$cdmFieldName,
      fieldIsMissing = .data$failed == 1,
      .keep = "none"
    )

  # Look up empty tables and add variable tableIsEmpty to checkResults
  # One row per table: table is empty if any measureValueCompleteness check has numDenominatorRows == 0
  emptyTables <- checkResults %>%
    dplyr::filter(
      .data$checkName == "measureValueCompleteness"
    ) %>%
    dplyr::mutate(
      .data$cdmTableName,
      tableIsEmpty = .data$numDenominatorRows == 0,
      .keep = "none"
    ) %>%
    dplyr::distinct()

  # Each table should appear at most once; duplicates indicate a deeper bug
  duplicateMissing <- missingTables %>%
    dplyr::count(.data$cdmTableName) %>%
    dplyr::filter(.data$n > 1)
  if (nrow(duplicateMissing) > 0) {
    stop("Duplicate table names in missingTables indicates an issue with check execution - cdmTable ran more than once for the same table: ", paste(duplicateMissing$cdmTableName, collapse = ", "))
  }
  duplicateEmpty <- emptyTables %>%
    dplyr::count(.data$cdmTableName) %>%
    dplyr::filter(.data$n > 1)
  if (nrow(duplicateEmpty) > 0) {
    stop("Duplicate table names in emptyTables indicates an issue with check execution - measureValueCompleteness had different denominator counts for the same table: ", paste(duplicateEmpty$cdmTableName, collapse = ", "))
  }

  # Look up empty fields and add variable fieldIsEmpty to checkResults
  emptyFields <- checkResults %>%
    dplyr::filter(
      .data$checkName == "measureValueCompleteness"
    ) %>%
    dplyr::mutate(
      .data$cdmTableName,
      .data$cdmFieldName,
      fieldIsEmpty = .data$numDenominatorRows == .data$numViolatedRows,
      .keep = "none"
    )

  # Assign notApplicable status
  checkResults <- checkResults %>%
    dplyr::left_join(
      missingTables,
      by = "cdmTableName"
    ) %>%
    dplyr::left_join(
      missingFields,
      by = c("cdmTableName", "cdmFieldName")
    ) %>%
    dplyr::left_join(
      emptyTables,
      by = "cdmTableName"
    ) %>%
    dplyr::left_join(
      emptyFields,
      by = c("cdmTableName", "cdmFieldName")
    ) %>%
    dplyr::mutate(
      tableIsMissing = dplyr::coalesce(.data$tableIsMissing, FALSE),
      tableIsEmpty = dplyr::coalesce(.data$tableIsEmpty, FALSE),
      conceptIsMissing = dplyr::coalesce(.data$checkLevel == "CONCEPT" & is.na(.data$unitConceptId) & .data$numDenominatorRows == 0, FALSE),
      conceptAndUnitAreMissing = dplyr::coalesce(.data$checkLevel == "CONCEPT" & !is.na(.data$unitConceptId) & .data$numDenominatorRows == 0, FALSE),
      fieldIsMissing = dplyr::coalesce(.data$fieldIsMissing, FALSE),
      fieldIsEmpty = dplyr::coalesce(.data$fieldIsEmpty, FALSE)
    )

  checkResults$notApplicable <- NA
  checkResults$notApplicableReason <- NA

  conditionOccurrenceIsMissing <- missingTables %>%
    dplyr::filter(.data$cdmTableName == "CONDITION_OCCURRENCE") %>%
    dplyr::mutate(conditionOccurrenceIsMissing = dplyr::coalesce(.data$tableIsMissing, FALSE)) %>%
    dplyr::pull(.data$conditionOccurrenceIsMissing)

  conditionOccurrenceIsEmpty <- emptyTables %>%
    dplyr::filter(.data$cdmTableName == "CONDITION_OCCURRENCE") %>%
    dplyr::mutate(conditionOccurrenceIsEmpty = dplyr::coalesce(.data$tableIsEmpty, FALSE)) %>%
    dplyr::pull(.data$conditionOccurrenceIsEmpty)

  for (i in seq_len(nrow(checkResults))) {
    # Special rule for measureConditionEraCompleteness, which should be notApplicable if CONDITION_OCCURRENCE is empty
    if (checkResults[i, "checkName"] == "measureConditionEraCompleteness") {
      if (conditionOccurrenceIsMissing || conditionOccurrenceIsEmpty) {
        checkResults$notApplicable[i] <- 1
        checkResults$notApplicableReason[i] <- "Table CONDITION_OCCURRENCE is empty."
      } else {
        checkResults$notApplicable[i] <- 0
      }
    } else {
      checkResults$notApplicable[i] <- .applyNotApplicable(checkResults[i, ])
    }
  }

  checkResults <- checkResults %>%
    dplyr::mutate(
      notApplicableReason = ifelse(
        .data$notApplicable == 1,
        dplyr::case_when(
          !is.na(.data$notApplicableReason) ~ .data$notApplicableReason,
          .data$tableIsMissing ~ sprintf("Table %s does not exist.", .data$cdmTableName),
          .data$fieldIsMissing ~ sprintf("Field %s.%s does not exist.", .data$cdmTableName, .data$cdmFieldName),
          .data$tableIsEmpty ~ sprintf("Table %s is empty.", .data$cdmTableName),
          .data$fieldIsEmpty ~ sprintf("Field %s.%s is not populated.", .data$cdmTableName, .data$cdmFieldName),
          .data$conceptIsMissing ~ sprintf("%s=%s is missing from the %s table.", .data$cdmFieldName, .data$conceptId, .data$cdmTableName),
          .data$conceptAndUnitAreMissing ~ sprintf("Combination of %s=%s, unitConceptId=%s and VALUE_AS_NUMBER IS NOT NULL is missing from the %s table.", .data$cdmFieldName, .data$conceptId, .data$unitConceptId, .data$cdmTableName) # nolint
        ),
        NA
      ),
      failed = ifelse(.data$notApplicable == 1, 0, .data$failed),
      passed = ifelse(.data$failed == 0 & .data$isError == 0 & .data$notApplicable == 0, 1, 0)
    ) %>%
    dplyr::select(-c("tableIsMissing", "fieldIsMissing", "tableIsEmpty", "fieldIsEmpty", "conceptIsMissing", "conceptAndUnitAreMissing"))

  return(checkResults)
}

library(testthat)

test_that("measurePersonCompleteness should not be marked as not applicable when table is empty", {
  # Create a mock check result for measurePersonCompleteness with tableIsEmpty = TRUE
  mockCheckResult <- data.frame(
    checkName = "measurePersonCompleteness",
    cdmTableName = "DEVICE_EXPOSURE",
    isError = 0,
    tableIsMissing = FALSE,
    fieldIsMissing = FALSE,
    tableIsEmpty = TRUE,
    fieldIsEmpty = FALSE,
    conceptIsMissing = FALSE,
    conceptAndUnitAreMissing = FALSE
  )

  # Test that .applyNotApplicable returns 0 (not applicable = FALSE) for measurePersonCompleteness
  # when tableIsEmpty is TRUE but tableIsMissing is FALSE
  result <- DataQualityDashboard:::.applyNotApplicable(mockCheckResult)
  expect_equal(result, 0)
})

test_that("measurePersonCompleteness should be marked as not applicable when table is missing", {
  # Create a mock check result for measurePersonCompleteness with tableIsMissing = TRUE
  mockCheckResult <- data.frame(
    checkName = "measurePersonCompleteness",
    cdmTableName = "DEVICE_EXPOSURE",
    isError = 0,
    tableIsMissing = TRUE,
    fieldIsMissing = FALSE,
    tableIsEmpty = FALSE,
    fieldIsEmpty = FALSE,
    conceptIsMissing = FALSE,
    conceptAndUnitAreMissing = FALSE
  )

  # Test that .applyNotApplicable returns 1 (not applicable = TRUE) for measurePersonCompleteness
  # when tableIsMissing is TRUE
  result <- DataQualityDashboard:::.applyNotApplicable(mockCheckResult)
  expect_equal(result, 1)
})

test_that("Not Applicable status Table Empty", {
  testthat::skip_if_not_installed("Eunomia")
  outputFolder <- tempfile("dqd_")
  on.exit(unlink(outputFolder, recursive = TRUE))

  # Make sure the device exposure table is empty
  connection <- DatabaseConnector::connect(connectionDetailsEunomiaNaChecks)
  on.exit(DatabaseConnector::disconnect(connection), add = TRUE)
  DatabaseConnector::executeSql(connection, "DELETE FROM DEVICE_EXPOSURE;")

  results <- withCallingHandlers(
    executeDqChecks(
      connectionDetails = connectionDetailsEunomiaNaChecks,
      cdmDatabaseSchema = cdmDatabaseSchemaEunomia,
      resultsDatabaseSchema = resultsDatabaseSchemaEunomia,
      cdmSourceName = "Eunomia",
      checkNames = c("cdmTable", "cdmField", "measureValueCompleteness"),
      # Eunomia COST table has misspelled 'REVEUE_CODE_SOURCE_VALUE'
      tablesToExclude = c("COST", "CONCEPT", "VOCABULARY", "CONCEPT_ANCESTOR", "CONCEPT_RELATIONSHIP", "CONCEPT_CLASS", "CONCEPT_SYNONYM", "RELATIONSHIP", "DOMAIN"),
      outputFolder = outputFolder,
      writeToTable = FALSE
    ),
    warning = function(w) {
      if (grepl("^Missing check names", w$message)) {
        invokeRestart("muffleWarning")
      }
    }
  )

  r <- results$CheckResults[results$CheckResults$checkName == "measureValueCompleteness" &
    results$CheckResults$tableName == "device_exposure", ]
  expect_true(all(r$notApplicable == 1))
})

test_that("measureConditionEraCompleteness Not Applicable if condition_occurrence empty", {
  testthat::skip_if_not_installed("Eunomia")
  outputFolder <- tempfile("dqd_")
  on.exit(unlink(outputFolder, recursive = TRUE))

  # Remove records from Condition Occurrence
  connection <- DatabaseConnector::connect(connectionDetailsEunomiaNaChecks)
  on.exit(DatabaseConnector::disconnect(connection), add = TRUE)
  DatabaseConnector::executeSql(connection, "CREATE TABLE CONDITION_OCCURRENCE_BACK AS SELECT * FROM CONDITION_OCCURRENCE;")
  DatabaseConnector::executeSql(connection, "DELETE FROM CONDITION_OCCURRENCE;")

  results <- withCallingHandlers(
    executeDqChecks(
      connectionDetails = connectionDetailsEunomiaNaChecks,
      cdmDatabaseSchema = cdmDatabaseSchemaEunomia,
      resultsDatabaseSchema = resultsDatabaseSchemaEunomia,
      cdmSourceName = "Eunomia",
      checkNames = c("cdmTable", "cdmField", "measureValueCompleteness", "measureConditionEraCompleteness"),
      # Eunomia COST table has misspelled 'REVEUE_CODE_SOURCE_VALUE'
      tablesToExclude = c("COST", "CONCEPT", "VOCABULARY", "CONCEPT_ANCESTOR", "CONCEPT_RELATIONSHIP", "CONCEPT_CLASS", "CONCEPT_SYNONYM", "RELATIONSHIP", "DOMAIN"),
      outputFolder = outputFolder,
      writeToTable = FALSE
    ),
    warning = function(w) {
      if (grepl("^Missing check names", w$message)) {
        invokeRestart("muffleWarning")
      }
    }
  )

  # Reinstate Condition Occurrence
  DatabaseConnector::executeSql(connection, "INSERT INTO CONDITION_OCCURRENCE SELECT * FROM CONDITION_OCCURRENCE_BACK;")
  DatabaseConnector::executeSql(connection, "DROP TABLE CONDITION_OCCURRENCE_BACK;")

  r <- results$CheckResults[results$CheckResults$checkName == "measureConditionEraCompleteness", ]
  expect_true(r$notApplicable == 1)
})

test_that("measureConditionEraCompleteness Fails if condition_era empty", {
  testthat::skip_if_not_installed("Eunomia")
  outputFolder <- tempfile("dqd_")
  on.exit(unlink(outputFolder, recursive = TRUE))

  # Remove records from Condition Era
  connection <- DatabaseConnector::connect(connectionDetailsEunomiaNaChecks)
  on.exit(DatabaseConnector::disconnect(connection), add = TRUE)
  DatabaseConnector::executeSql(connection, "CREATE TABLE CONDITION_ERA_BACK AS SELECT * FROM CONDITION_ERA;")
  DatabaseConnector::executeSql(connection, "DELETE FROM CONDITION_ERA;")

  results <- withCallingHandlers(
    executeDqChecks(
      connectionDetails = connectionDetailsEunomiaNaChecks,
      cdmDatabaseSchema = cdmDatabaseSchemaEunomia,
      resultsDatabaseSchema = resultsDatabaseSchemaEunomia,
      cdmSourceName = "Eunomia",
      checkNames = c("cdmTable", "cdmField", "measureValueCompleteness", "measureConditionEraCompleteness"),
      # Eunomia COST table has misspelled 'REVEUE_CODE_SOURCE_VALUE'
      tablesToExclude = c("COST", "CONCEPT", "VOCABULARY", "CONCEPT_ANCESTOR", "CONCEPT_RELATIONSHIP", "CONCEPT_CLASS", "CONCEPT_SYNONYM", "RELATIONSHIP", "DOMAIN"),
      outputFolder = outputFolder,
      writeToTable = FALSE
    ),
    warning = function(w) {
      if (grepl("^Missing check names", w$message)) {
        invokeRestart("muffleWarning")
      }
    }
  )

  # Reinstate the Condition Era
  DatabaseConnector::executeSql(connection, "INSERT INTO CONDITION_ERA SELECT * FROM CONDITION_ERA_BACK;")
  DatabaseConnector::executeSql(connection, "DROP TABLE CONDITION_ERA_BACK;")

  r <- results$CheckResults[results$CheckResults$checkName == "measureConditionEraCompleteness", ]
  expect_true(r$failed == 1)
})

test_that("measurePersonCompleteness NOT marked as Not Applicable when table is empty", {
  testthat::skip_if_not_installed("Eunomia")
  outputFolder <- tempfile("dqd_")
  on.exit(unlink(outputFolder, recursive = TRUE))

  # Remove records from Device Exposure to make it empty
  connection <- DatabaseConnector::connect(connectionDetailsEunomiaNaChecks)
  on.exit(DatabaseConnector::disconnect(connection), add = TRUE)
  DatabaseConnector::executeSql(connection, "CREATE TABLE OBSERVATION_PERIOD_BACK AS SELECT * FROM OBSERVATION_PERIOD;")
  DatabaseConnector::executeSql(connection, "DELETE FROM OBSERVATION_PERIOD;")

  results <- withCallingHandlers(
    executeDqChecks(
      connectionDetails = connectionDetailsEunomiaNaChecks,
      cdmDatabaseSchema = cdmDatabaseSchemaEunomia,
      resultsDatabaseSchema = resultsDatabaseSchemaEunomia,
      cdmSourceName = "Eunomia",
      checkNames = c("cdmTable", "cdmField", "measureValueCompleteness", "measurePersonCompleteness"),
      # Eunomia COST table has misspelled 'REVEUE_CODE_SOURCE_VALUE'
      tablesToExclude = c("COST", "CONCEPT", "VOCABULARY", "CONCEPT_ANCESTOR", "CONCEPT_RELATIONSHIP", "CONCEPT_CLASS", "CONCEPT_SYNONYM", "RELATIONSHIP", "DOMAIN"),
      outputFolder = outputFolder,
      writeToTable = FALSE
    ),
    warning = function(w) {
      if (grepl("^Missing check names", w$message)) {
        invokeRestart("muffleWarning")
      }
    }
  )

  # Reinstate Device Exposure
  DatabaseConnector::executeSql(connection, "INSERT INTO OBSERVATION_PERIOD SELECT * FROM OBSERVATION_PERIOD_BACK;")
  DatabaseConnector::executeSql(connection, "DROP TABLE OBSERVATION_PERIOD_BACK;")

  # measurePersonCompleteness should NOT be marked as not applicable when table is empty
  r <- results$CheckResults[results$CheckResults$checkName == "measurePersonCompleteness" &
    results$CheckResults$cdmTableName == "OBSERVATION_PERIOD", ]
  expect_true(r$notApplicable == 0)

  # It should fail because the threshold is 100% and all persons have 0 records in empty table
  expect_true(r$failed == 1)
})

test_that("NA applied correctly when table or field is missing", {
  # measurePersonCompleteness with isError=1 and tableIsMissing=TRUE should be NA
  mockCheckResult <- data.frame(
    checkName = "measurePersonCompleteness",
    cdmTableName = "FOO",
    cdmFieldName = NA,
    isError = 1,
    tableIsMissing = TRUE,
    fieldIsMissing = FALSE,
    tableIsEmpty = FALSE,
    fieldIsEmpty = FALSE,
    conceptIsMissing = FALSE,
    conceptAndUnitAreMissing = FALSE
  )
  result <- DataQualityDashboard:::.applyNotApplicable(mockCheckResult)
  expect_equal(result, 1)

  # measureValueCompleteness with isError=1 and fieldIsMissing=TRUE should be NA
  mockCheckResult <- data.frame(
    checkName = "measureValueCompleteness",
    cdmTableName = "OBSERVATION",
    cdmFieldName = "bar",
    isError = 1,
    tableIsMissing = FALSE,
    fieldIsMissing = TRUE,
    tableIsEmpty = FALSE,
    fieldIsEmpty = FALSE,
    conceptIsMissing = FALSE,
    conceptAndUnitAreMissing = FALSE
  )
  result <- DataQualityDashboard:::.applyNotApplicable(mockCheckResult)
  expect_equal(result, 1)
})

test_that(".applyNotApplicable handles cdmTable and cdmField correctly", {
  # cdmTable should NEVER be NA, no matter what
  # Test with missing table
  mockCheckResult <- data.frame(
    checkName = "cdmTable",
    cdmTableName = "FOO",
    cdmFieldName = NA,
    isError = 0,
    tableIsMissing = TRUE,
    fieldIsMissing = FALSE,
    tableIsEmpty = FALSE,
    fieldIsEmpty = FALSE,
    conceptIsMissing = FALSE,
    conceptAndUnitAreMissing = FALSE
  )
  result <- DataQualityDashboard:::.applyNotApplicable(mockCheckResult)
  expect_equal(result, 0)

  # Test with empty table
  mockCheckResult <- data.frame(
    checkName = "cdmTable",
    cdmTableName = "FOO",
    cdmFieldName = NA,
    isError = 0,
    tableIsMissing = FALSE,
    fieldIsMissing = FALSE,
    tableIsEmpty = TRUE,
    fieldIsEmpty = FALSE,
    conceptIsMissing = FALSE,
    conceptAndUnitAreMissing = FALSE
  )
  result <- DataQualityDashboard:::.applyNotApplicable(mockCheckResult)
  expect_equal(result, 0)

  # Test with error
  mockCheckResult <- data.frame(
    checkName = "cdmTable",
    cdmTableName = "FOO",
    cdmFieldName = NA,
    isError = 1,
    tableIsMissing = FALSE,
    fieldIsMissing = FALSE,
    tableIsEmpty = FALSE,
    fieldIsEmpty = FALSE,
    conceptIsMissing = FALSE,
    conceptAndUnitAreMissing = FALSE
  )
  result <- DataQualityDashboard:::.applyNotApplicable(mockCheckResult)
  expect_equal(result, 0)

  # cdmField should only be NA if table is missing, otherwise never NA
  # Test with missing table (should BE NA)
  mockCheckResult <- data.frame(
    checkName = "cdmField",
    cdmTableName = "OBSERVATION",
    cdmFieldName = "bar",
    isError = 0,
    tableIsMissing = TRUE,
    fieldIsMissing = FALSE,
    tableIsEmpty = FALSE,
    fieldIsEmpty = FALSE,
    conceptIsMissing = FALSE,
    conceptAndUnitAreMissing = FALSE
  )
  result <- DataQualityDashboard:::.applyNotApplicable(mockCheckResult)
  expect_equal(result, 1)

  # Test with missing field but table exists (should NOT be NA)
  mockCheckResult <- data.frame(
    checkName = "cdmField",
    cdmTableName = "OBSERVATION",
    cdmFieldName = "bar",
    isError = 0,
    tableIsMissing = FALSE,
    fieldIsMissing = TRUE,
    tableIsEmpty = FALSE,
    fieldIsEmpty = FALSE,
    conceptIsMissing = FALSE,
    conceptAndUnitAreMissing = FALSE
  )
  result <- DataQualityDashboard:::.applyNotApplicable(mockCheckResult)
  expect_equal(result, 0)

  # Test with empty table (should NOT be NA)
  mockCheckResult <- data.frame(
    checkName = "cdmField",
    cdmTableName = "OBSERVATION",
    cdmFieldName = "bar",
    isError = 0,
    tableIsMissing = FALSE,
    fieldIsMissing = FALSE,
    tableIsEmpty = TRUE,
    fieldIsEmpty = FALSE,
    conceptIsMissing = FALSE,
    conceptAndUnitAreMissing = FALSE
  )
  result <- DataQualityDashboard:::.applyNotApplicable(mockCheckResult)
  expect_equal(result, 0)
})

# Copyright 2026 Observational Health Data Sciences and Informatics
#
# This file is part of DataQualityDashboard
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Internal function to evaluate the data quality checks against given thresholds.
#'
#' @param checkResults              A dataframe containing the results of the data quality checks
#' @param tableChecks               A dataframe containing the table checks
#' @param fieldChecks               A dataframe containing the field checks
#' @param conceptChecks             A dataframe containing the concept checks
#'
#' @return A dataframe with updated check results including pass/fail status and threshold values
#'
#' @keywords internal

.evaluateThresholds <- function(checkResults,
                                tableChecks,
                                fieldChecks,
                                conceptChecks) {
  checkResults$failed <- 0
  checkResults$passed <- 0
  checkResults$isError <- 0
  checkResults$notApplicable <- 0
  checkResults$notApplicableReason <- NA
  checkResults$thresholdValue <- NA
  checkResults$notesValue <- NA

  for (i in 1:nrow(checkResults)) {
    thresholdField <- sprintf("%sThreshold", checkResults[i, ]$checkName)
    notesField <- sprintf("%sNotes", checkResults[i, ]$checkName)

    # find if field exists -----------------------------------------------
    thresholdFieldExists <- eval(parse(
      text =
        sprintf(
          "'%s' %%in%% colnames(%sChecks)",
          thresholdField,
          tolower(checkResults[i, ]$checkLevel)
        )
    ))

    if (!thresholdFieldExists) {
      thresholdValue <- NA
      notesValue <- NA
    } else {
      if (checkResults[i, ]$checkLevel == "TABLE") {
        thresholdFilter <- sprintf(
          "tableChecks$%s[tableChecks$cdmTableName == '%s']",
          thresholdField, checkResults[i, ]$cdmTableName
        )
        notesFilter <- sprintf(
          "tableChecks$%s[tableChecks$cdmTableName == '%s']",
          notesField, checkResults[i, ]$cdmTableName
        )
      } else if (checkResults[i, ]$checkLevel == "FIELD") {
        thresholdFilter <- sprintf(
          "fieldChecks$%s[fieldChecks$cdmTableName == '%s' &
                                fieldChecks$cdmFieldName == '%s']",
          thresholdField,
          checkResults[i, ]$cdmTableName,
          checkResults[i, ]$cdmFieldName
        )
        notesFilter <- sprintf(
          "fieldChecks$%s[fieldChecks$cdmTableName == '%s' &
                                fieldChecks$cdmFieldName == '%s']",
          notesField,
          checkResults[i, ]$cdmTableName,
          checkResults[i, ]$cdmFieldName
        )
      } else if (checkResults[i, ]$checkLevel == "CONCEPT") {
        if (is.na(checkResults[i, ]$unitConceptId) &&
          grepl(",", checkResults[i, ]$conceptId)) {
          thresholdFilter <- sprintf(
            "conceptChecks$%s[conceptChecks$cdmTableName == '%s' &
                                  conceptChecks$cdmFieldName == '%s' &
                                  conceptChecks$conceptId == '%s']",
            thresholdField,
            checkResults[i, ]$cdmTableName,
            checkResults[i, ]$cdmFieldName,
            checkResults[i, ]$conceptId
          )
          notesFilter <- sprintf(
            "conceptChecks$%s[conceptChecks$cdmTableName == '%s' &
                                  conceptChecks$cdmFieldName == '%s' &
                                  conceptChecks$conceptId == '%s']",
            notesField,
            checkResults[i, ]$cdmTableName,
            checkResults[i, ]$cdmFieldName,
            checkResults[i, ]$conceptId
          )
        } else if (is.na(checkResults[i, ]$unitConceptId) &&
          !grepl(",", checkResults[i, ]$conceptId)) {
          thresholdFilter <- sprintf(
            "conceptChecks$%s[conceptChecks$cdmTableName == '%s' &
                                  conceptChecks$cdmFieldName == '%s' &
                                  conceptChecks$conceptId == %s]",
            thresholdField,
            checkResults[i, ]$cdmTableName,
            checkResults[i, ]$cdmFieldName,
            checkResults[i, ]$conceptId
          )
          notesFilter <- sprintf(
            "conceptChecks$%s[conceptChecks$cdmTableName == '%s' &
                                  conceptChecks$cdmFieldName == '%s' &
                                  conceptChecks$conceptId == %s]",
            notesField,
            checkResults[i, ]$cdmTableName,
            checkResults[i, ]$cdmFieldName,
            checkResults[i, ]$conceptId
          )
        } else {
          thresholdFilter <- sprintf(
            "conceptChecks$%s[conceptChecks$cdmTableName == '%s' &
                                  conceptChecks$cdmFieldName == '%s' &
                                  conceptChecks$conceptId == %s &
                                  conceptChecks$unitConceptId == '%s']",
            thresholdField,
            checkResults[i, ]$cdmTableName,
            checkResults[i, ]$cdmFieldName,
            checkResults[i, ]$conceptId,
            as.integer(checkResults[i, ]$unitConceptId)
          )
          notesFilter <- sprintf(
            "conceptChecks$%s[conceptChecks$cdmTableName == '%s' &
                                  conceptChecks$cdmFieldName == '%s' &
                                  conceptChecks$conceptId == %s &
                                  conceptChecks$unitConceptId == '%s']",
            notesField,
            checkResults[i, ]$cdmTableName,
            checkResults[i, ]$cdmFieldName,
            checkResults[i, ]$conceptId,
            as.integer(checkResults[i, ]$unitConceptId)
          )
        }
      }

      thresholdValue <- eval(parse(text = thresholdFilter))
      notesValue <- eval(parse(text = notesFilter))

      checkResults[i, ]$thresholdValue <- thresholdValue
      checkResults[i, ]$notesValue <- notesValue
    }

    if (!is.na(checkResults[i, ]$error)) {
      # Parse error message for missing table/field
      errorMsg <- checkResults[i, ]$error
      if (grepl("does not exist|missing|not found|invalid object name|invalid column name|ORA-00942|no such table|no such column", errorMsg, ignore.case = TRUE) &&
        (checkResults[i, ]$checkName == "cdmField" || checkResults[i, ]$checkName == "cdmTable")) {
        checkResults[i, ]$failed <- 1
        checkResults[i, ]$isError <- 0
      } else {
        checkResults[i, ]$isError <- 1
        checkResults[i, ]$failed <- 0
      }
    } else if (is.na(thresholdValue) | thresholdValue == 0) {
      # If no threshold, or threshold is 0%, then any violating records will cause this check to fail
      if (!is.na(checkResults[i, ]$numViolatedRows) & checkResults[i, ]$numViolatedRows > 0) {
        checkResults[i, ]$failed <- 1
      }
    } else if (checkResults[i, ]$pctViolatedRows * 100 > thresholdValue) {
      checkResults[i, ]$failed <- 1
    }
  }

  if (.hasNAchecks(checkResults)) {
    checkResults <- .calculateNotApplicableStatus(checkResults)
  } else {
    checkResults$passed <- as.integer(checkResults$failed == 0 & checkResults$isError == 0)
  }

  checkResults
}

# Copyright 2026 Observational Health Data Sciences and Informatics
#
# This file is part of DataQualityDashboard
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Internal function to summarize the results of the DQD run.
#'
#' @param checkResults              A dataframe containing the results of the checks after running against the database
#'
#' @return A list containing summary statistics of the check results
#'
#' @keywords internal
#'

.summarizeResults <- function(checkResults) {
  countTotal <- nrow(checkResults)

  countThresholdFailed <- nrow(checkResults[checkResults$failed == 1 &
    is.na(checkResults$error), ])

  countErrorFailed <- nrow(checkResults[!is.na(checkResults$error), ])

  countOverallFailed <- nrow(checkResults[checkResults$failed == 1, ])

  countPassed <- countTotal - countOverallFailed

  countTotalPlausibility <- nrow(checkResults[checkResults$category == "Plausibility", ])

  countTotalConformance <- nrow(checkResults[checkResults$category == "Conformance", ])

  countTotalCompleteness <- nrow(checkResults[checkResults$category == "Completeness", ])

  countFailedPlausibility <- nrow(checkResults[checkResults$category == "Plausibility" &
    checkResults$failed == 1, ])

  countFailedConformance <- nrow(checkResults[checkResults$category == "Conformance" &
    checkResults$failed == 1, ])

  countFailedCompleteness <- nrow(checkResults[checkResults$category == "Completeness" &
    checkResults$failed == 1, ])

  countPassedPlausibility <- nrow(checkResults[checkResults$category == "Plausibility" &
    checkResults$passed == 1, ])

  countPassedConformance <- nrow(checkResults[checkResults$category == "Conformance" &
    checkResults$passed == 1, ])

  countPassedCompleteness <- nrow(checkResults[checkResults$category == "Completeness" &
    checkResults$passed == 1, ])

  list(
    countTotal = countTotal,
    countPassed = countPassed,
    countErrorFailed = countErrorFailed,
    countThresholdFailed = countThresholdFailed,
    countOverallFailed = countOverallFailed,
    percentPassed = round(countPassed / (countPassed + countOverallFailed) * 100, 2),
    percentFailed = round(countOverallFailed / (countPassed + countOverallFailed) * 100, 2),
    countTotalPlausibility = countTotalPlausibility,
    countTotalConformance = countTotalConformance,
    countTotalCompleteness = countTotalCompleteness,
    countFailedPlausibility = countFailedPlausibility,
    countFailedConformance = countFailedConformance,
    countFailedCompleteness = countFailedCompleteness,
    countPassedPlausibility = countPassedPlausibility,
    countPassedConformance = countPassedConformance,
    countPassedCompleteness = countPassedCompleteness
  )
}

# Copyright 2026 Observational Health Data Sciences and Informatics
#
# This file is part of DataQualityDashboard
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' @title Execute DQ checks
#'
#' @description This function will connect to the database, generate the sql scripts, and run the data quality checks against the database. By default, results will be written to a json file as well as a database table.
#'
#' @param connectionDetails         A connectionDetails object for connecting to the CDM database
#' @param cdmDatabaseSchema         The fully qualified database name of the CDM schema
#' @param resultsDatabaseSchema     The fully qualified database name of the results schema
#' @param vocabDatabaseSchema       The fully qualified database name of the vocabulary schema (default is to set it as the cdmDatabaseSchema)
#' @param numThreads                The number of concurrent threads to use to execute the queries
#' @param cdmSourceName             The name of the CDM data source
#' @param sqlOnly                   Should the SQLs be executed (FALSE) or just returned (TRUE)?
#' @param sqlOnlyUnionCount         (OPTIONAL) In sqlOnlyIncrementalInsert mode, how many SQL commands to union in each query to insert check results into results table (can speed processing when queries done in parallel). Default is 1.
#' @param sqlOnlyIncrementalInsert  (OPTIONAL) In sqlOnly mode, boolean to determine whether to generate SQL queries that insert check results and associated metadata into results table.  Default is FALSE (for backwards compatibility to <= v2.2.0)
#' @param outputFolder              The folder to output logs, SQL files, and JSON results file to
#' @param outputFile                (OPTIONAL) File to write results JSON object
#' @param verboseMode               Boolean to determine if the console will show all execution steps. Default is FALSE
#' @param writeToTable              Boolean to indicate if the check results will be written to the dqdashboard_results table in the resultsDatabaseSchema. Default is TRUE
#' @param writeTableName            The name of the results table. Defaults to `dqdashboard_results`.  Used when sqlOnly or writeToTable is True.
#' @param writeToCsv                Boolean to indicate if the check results will be written to a csv file. Default is FALSE
#' @param csvFile                   (OPTIONAL) CSV file to write results
#' @param checkLevels               Choose which DQ check levels to execute. Default is all 3 (TABLE, FIELD, CONCEPT)
#' @param checkSeverity             Choose which DQ check severity levels to execute. Default is all 3 (fatal, convention, characterization)
#' @param checkNames                (OPTIONAL) Choose which check names to execute. Names can be found in inst/csv/OMOP_CDM_v[cdmVersion]_Check_Descriptions.csv. Note that "cdmTable", "cdmField" and "measureValueCompleteness" are always executed.
#' @param cohortDefinitionId        The cohort definition id for the cohort you wish to run the DQD on. The package assumes a standard OHDSI cohort table
#'                                  with the fields cohort_definition_id and subject_id.
#' @param cohortDatabaseSchema      The schema where the cohort table is located.
#' @param cohortTableName           The name of the cohort table. Defaults to `cohort`.
#' @param tablesToExclude           (OPTIONAL) Choose which CDM tables to exclude from the execution.
#' @param cdmVersion                The CDM version to target for the data source. Options are "5.2", "5.3", or "5.4". By default, "5.3" is used.
#' @param tableCheckThresholdLoc    The location of the threshold file for evaluating the table checks. If not specified the default thresholds will be applied.
#' @param fieldCheckThresholdLoc    The location of the threshold file for evaluating the field checks. If not specified the default thresholds will be applied.
#' @param conceptCheckThresholdLoc  The location of the threshold file for evaluating the concept checks. If not specified the default thresholds will be applied.
#'
#' @return A list object of results
#'
#' @importFrom magrittr %>%
#' @import DatabaseConnector
#' @importFrom stringr str_detect regex
#' @importFrom utils packageVersion write.table
#' @importFrom rlang .data
#' @importFrom tidyselect all_of
#' @importFrom readr read_csv local_edition
#' @importFrom dplyr mutate case_when
#'
#' @export
#'
executeDqChecks <- function(connectionDetails,
                            cdmDatabaseSchema,
                            resultsDatabaseSchema,
                            vocabDatabaseSchema = cdmDatabaseSchema,
                            cdmSourceName,
                            numThreads = 1,
                            sqlOnly = FALSE,
                            sqlOnlyUnionCount = 1,
                            sqlOnlyIncrementalInsert = FALSE,
                            outputFolder,
                            outputFile = "",
                            verboseMode = FALSE,
                            writeToTable = TRUE,
                            writeTableName = "dqdashboard_results",
                            writeToCsv = FALSE,
                            csvFile = "",
                            checkLevels = c("TABLE", "FIELD", "CONCEPT"),
                            checkNames = c(),
                            checkSeverity = c("fatal", "convention", "characterization"),
                            cohortDefinitionId = c(),
                            cohortDatabaseSchema = resultsDatabaseSchema,
                            cohortTableName = "cohort",
                            tablesToExclude = c("CONCEPT", "VOCABULARY", "CONCEPT_ANCESTOR", "CONCEPT_RELATIONSHIP", "CONCEPT_CLASS", "CONCEPT_SYNONYM", "RELATIONSHIP", "DOMAIN"),
                            cdmVersion = "5.3",
                            tableCheckThresholdLoc = "default",
                            fieldCheckThresholdLoc = "default",
                            conceptCheckThresholdLoc = "default") {
  # Check input -------------------------------------------------------------------------------------------------------------------
  if (!any(class(connectionDetails) %in% c("connectionDetails", "ConnectionDetails"))) {
    stop("connectionDetails must be an object of class 'connectionDetails' or 'ConnectionDetails'.")
  }

  if (!str_detect(cdmVersion, regex(acceptedCdmRegex))) {
    stop("cdmVersion must contain a version of the form '5.X' where X is an integer between 2 and 4 inclusive.")
  }

  if (sqlOnlyIncrementalInsert == TRUE && sqlOnly == FALSE) {
    stop("Set `sqlOnly` to TRUE in order to use `sqlOnlyIncrementalInsert` mode.")
  }

  stopifnot(is.character(cdmDatabaseSchema), is.character(resultsDatabaseSchema), is.numeric(numThreads))
  stopifnot(is.character(cdmSourceName), is.logical(sqlOnly), is.character(outputFolder), is.logical(verboseMode))
  stopifnot(is.logical(writeToTable), is.character(checkLevels))
  stopifnot(is.numeric(sqlOnlyUnionCount) && sqlOnlyUnionCount > 0)
  stopifnot(is.logical(sqlOnlyIncrementalInsert))
  stopifnot(is.character(cohortDatabaseSchema), is.character(cohortTableName))

  if (!all(checkLevels %in% c("TABLE", "FIELD", "CONCEPT"))) {
    stop('checkLevels argument must be a subset of c("TABLE", "FIELD", "CONCEPT").
         You passed in ', paste(checkLevels, collapse = ", "))
  }

  if (!all(checkSeverity %in% c("fatal", "convention", "characterization"))) {
    stop('checkSeverity argument must be a subset of c("fatal", "convention", "characterization").
         You passed in ', paste(checkSeverity, collapse = ", "))
  }

  stopifnot(
    is.null(checkNames) | is.character(checkNames),
    is.character(checkSeverity),
    is.null(tablesToExclude) | is.character(tablesToExclude)
  )
  stopifnot(is.character(cdmVersion))


  # temporary patch to work around vroom 1.6.4 bug
  readr::local_edition(1)

  # capture metadata -----------------------------------------------------------------------
  if (!sqlOnly) {
    connection <- DatabaseConnector::connect(connectionDetails = connectionDetails)
    on.exit(DatabaseConnector::disconnect(connection = connection), add = TRUE)
    sql <- SqlRender::render(
      sql = "select * from @cdmDatabaseSchema.cdm_source;",
      cdmDatabaseSchema = cdmDatabaseSchema
    )
    sql <- SqlRender::translate(sql = sql, targetDialect = connectionDetails$dbms)
    metadata <- DatabaseConnector::querySql(connection = connection, sql = sql, snakeCaseToCamelCase = TRUE)
    if (nrow(metadata) < 1) {
      stop("Please populate the cdm_source table before executing data quality checks.")
    }
    if (nrow(metadata) > 1) {
      metadata <- metadata[1, ]
      warning("The cdm_source table has more than 1 row. A single row from this table has been selected to populate DQD metadata.")
    }
    metadata$dqdVersion <- as.character(packageVersion("DataQualityDashboard"))
  } else {
    connection <- NULL
    metadata <- data.frame(
      dqdVersion = as.character(packageVersion("DataQualityDashboard")),
      cdmSourceName = cdmSourceName
    )
  }

  # Setup output folder ------------------------------------------------------------------------------------------------------------
  if (!dir.exists(outputFolder)) {
    dir.create(path = outputFolder, recursive = TRUE)
  }

  if (dir.exists(file.path(outputFolder, "errors"))) {
    unlink(file.path(outputFolder, "errors"), recursive = TRUE)
  }

  dir.create(file.path(outputFolder, "errors"), recursive = TRUE)

  # Log execution -----------------------------------------------------------------------------------------------------------------
  logFileName <- sprintf("log_DqDashboard_%s.txt", cdmSourceName)
  unlink(file.path(outputFolder, logFileName))

  if (verboseMode) {
    appenders <- list(
      ParallelLogger::createConsoleAppender(layout = ParallelLogger::layoutTimestamp),
      ParallelLogger::createFileAppender(
        layout = ParallelLogger::layoutParallel,
        fileName = file.path(outputFolder, logFileName)
      )
    )
  } else {
    appenders <- list(ParallelLogger::createFileAppender(
      layout = ParallelLogger::layoutParallel,
      fileName = file.path(outputFolder, logFileName)
    ))
  }


  logger <- ParallelLogger::createLogger(
    name = "DqDashboard",
    threshold = "INFO",
    appenders = appenders
  )
  ParallelLogger::registerLogger(logger = logger)
  on.exit(ParallelLogger::unregisterLogger("DqDashboard", silent = TRUE), add = TRUE)

  # load Threshold CSVs ----------------------------------------------------------------------------------------

  startTime <- Sys.time()

  checkDescriptionsDf <- read_csv(
    file = system.file(
      "csv",
      sprintf("OMOP_CDMv%s_Check_Descriptions.csv", cdmVersion),
      package = "DataQualityDashboard"
    )
  )
  checkDescriptionsDf <- as.data.frame(checkDescriptionsDf)

  tableChecks <- .readThresholdFile(
    checkThresholdLoc = tableCheckThresholdLoc,
    defaultLoc = sprintf("OMOP_CDMv%s_Table_Level.csv", cdmVersion)
  )

  fieldChecks <- .readThresholdFile(
    checkThresholdLoc = fieldCheckThresholdLoc,
    defaultLoc = sprintf("OMOP_CDMv%s_Field_Level.csv", cdmVersion)
  )

  conceptChecks <- .readThresholdFile(
    checkThresholdLoc = conceptCheckThresholdLoc,
    defaultLoc = sprintf("OMOP_CDMv%s_Concept_Level.csv", cdmVersion)
  )
  # ensure we use only checks that are intended to be run -----------------------------------------

  if (length(tablesToExclude) > 0) {
    tablesToExclude <- toupper(tablesToExclude)
    ParallelLogger::logInfo(sprintf("CDM Tables skipped: %s", paste(tablesToExclude, collapse = ", ")))
    tableChecks <- tableChecks[!tableChecks$cdmTableName %in% tablesToExclude, ]
    fieldChecks <- fieldChecks[!fieldChecks$cdmTableName %in% tablesToExclude, ]
    conceptChecks <- conceptChecks[!conceptChecks$cdmTableName %in% tablesToExclude, ]
  }

  ## remove offset from being checked as it is a reserved word in some databases
  fieldChecks <- subset(fieldChecks, fieldChecks$cdmFieldName != "offset")

  tableChecks <- dplyr::mutate(tableChecks, schema = dplyr::case_when(
    schema == "CDM" ~ cdmDatabaseSchema,
    schema == "VOCAB" ~ vocabDatabaseSchema,
    schema == "COHORT" ~ cohortDatabaseSchema,
    TRUE ~ cdmDatabaseSchema
  ))

  fieldChecks <- merge(x = fieldChecks, y = tableChecks[, c("cdmTableName", "schema")], by = "cdmTableName", all.x = TRUE)

  checksToInclude <- checkDescriptionsDf$checkName[sapply(checkDescriptionsDf$checkName, function(check) {
    !is.null(eval(parse(text = sprintf("tableChecks$%s", check)))) |
      !is.null(eval(parse(text = sprintf("fieldChecks$%s", check)))) |
      !is.null(eval(parse(text = sprintf("conceptChecks$%s", check))))
  })]

  checkDescriptionsDf <- checkDescriptionsDf[checkDescriptionsDf$checkLevel %in% checkLevels &
    checkDescriptionsDf$severity %in% checkSeverity &
    checkDescriptionsDf$evaluationFilter != "" &
    checkDescriptionsDf$sqlFile != "" &
    checkDescriptionsDf$checkName %in% checksToInclude, ]

  if (length(checkNames) > 0) {
    checkDescriptionsDf <- checkDescriptionsDf[checkDescriptionsDf$checkName %in% checkNames, ]
  }

  if (nrow(checkDescriptionsDf) == 0) {
    stop("No checks are available based on excluded tables. Please review tablesToExclude.")
  }

  if (!.containsNAchecks(checkDescriptionsDf$checkName)) {
    warning("Missing check names to calculate the 'Not Applicable' status. Results will show pass/fail without Not Applicable status.")
  }

  if ("plausibleDuringLife" %in% checkDescriptionsDf$checkName) {
    warning("DEPRECATION WARNING - The plausibleDuringLife check has been reimplemented with the plausibleBeforeDeath check.")
  }

  if ("plausibleTemporalAfter" %in% checkDescriptionsDf$checkName) {
    warning("DEPRECATION WARNING - The plausibleTemporalAfter check has been reimplemented with the plausibleAfterBirth and plausibleStartBeforeEnd checks.")
  }

  if ("plausibleGender" %in% checkDescriptionsDf$checkName) {
    warning("DEPRECATION WARNING - The plausibleGender check has been reimplemented with the plausibleGenderUseDescendants check.")
  }

  checkDescriptions <- split(checkDescriptionsDf, seq_len(nrow(checkDescriptionsDf)))

  # set up connection for single vs. multi-threaded execution
  connection <- NULL
  if (numThreads == 1 && !sqlOnly) {
    connection <- DatabaseConnector::connect(connectionDetails = connectionDetails)
  }

  fieldChecks$cdmFieldName <- toupper(fieldChecks$cdmFieldName)
  conceptChecks$cdmFieldName <- toupper(conceptChecks$cdmFieldName)

  cluster <- ParallelLogger::makeCluster(numberOfThreads = numThreads, singleThreadToMain = TRUE)
  resultsList <- ParallelLogger::clusterApply(
    cluster = cluster, x = checkDescriptions,
    fun = .runCheck,
    tableChecks,
    fieldChecks,
    conceptChecks,
    connectionDetails,
    connection,
    cdmDatabaseSchema,
    vocabDatabaseSchema,
    resultsDatabaseSchema,
    writeTableName,
    cohortDatabaseSchema,
    cohortTableName,
    cohortDefinitionId,
    outputFolder,
    sqlOnlyUnionCount,
    sqlOnlyIncrementalInsert,
    sqlOnly,
    progressBar = TRUE
  )
  ParallelLogger::stopCluster(cluster = cluster)

  allResults <- NULL

  if (!sqlOnly) {
    checkResults <- do.call(rbind, resultsList)

    # evaluate thresholds-------------------------------------------------------------------
    checkResults <- .evaluateThresholds(
      checkResults = checkResults,
      tableChecks = tableChecks,
      fieldChecks = fieldChecks,
      conceptChecks = conceptChecks
    )

    # create overview
    overview <- .summarizeResults(checkResults = checkResults)

    endTime <- Sys.time()
    delta <- endTime - startTime

    # Create result
    allResults <- list(
      startTimestamp = startTime,
      endTimestamp = endTime,
      executionTime = sprintf("%.0f %s", delta, attr(delta, "units")),
      # new variable executionTimeSeconds added to store execution time in seconds
      executionTimeSeconds = as.numeric(delta),
      CheckResults = checkResults,
      Metadata = metadata,
      Overview = overview
    )

    # Write result
    if (nchar(outputFile) == 0) {
      endTimestamp <- format(endTime, "%Y%m%d%H%M%S")
      outputFile <- sprintf("%s-%s.json", tolower(metadata$cdmSourceAbbreviation), endTimestamp)
    }

    .writeResultsToJson(allResults, outputFolder, outputFile)

    ParallelLogger::logInfo("Execution Complete")
  } else {
    .writeDDL(resultsDatabaseSchema, writeTableName, connectionDetails$dbms, outputFolder)
  }

  # write to table ----------------------------------------------------------------------

  if (!sqlOnly && writeToTable) {
    .writeResultsToTable(
      connectionDetails = connectionDetails,
      resultsDatabaseSchema = resultsDatabaseSchema,
      checkResults = allResults$CheckResults,
      writeTableName = writeTableName,
      cohortDefinitionId = cohortDefinitionId
    )
  }

  # write to CSV ----------------------------------------------------------------------

  if (!sqlOnly && writeToCsv) {
    if (nchar(csvFile) == 0) {
      csvFile <- sprintf("%s.csv", sub(pattern = "(.*)\\..*$", replacement = "\\1", basename(allResults$outputFile)))
    }
    .writeResultsToCsv(
      checkResults = allResults$CheckResults,
      csvPath = file.path(outputFolder, csvFile)
    )
  }

  invisible(allResults)
}

#' Internal function to determine if the connection needs auto commit
#'
#' @param connectionDetails A connectionDetails object for connecting to the CDM database
#' @param connection A connection for connecting to the CDM database using the DatabaseConnector::connect(connectionDetails) function.
#'
#' @return A logical value indicating if the connection needs auto commit
#'
#' @keywords internal
#'
.needsAutoCommit <- function(connectionDetails, connection) {
  autoCommit <- FALSE
  if (!is.null(connection)) {
    if (inherits(connection, "DatabaseConnectorJdbcConnection")) {
      if (connectionDetails$dbms %in% c("postgresql", "redshift")) {
        autoCommit <- TRUE
      }
    }
  }
  return(autoCommit)
}

library(testthat)

test_that("plausibleAfterBirth allows events on same day as birth (issue #561)", {
  testthat::skip_if_not_installed("Eunomia")
  outputFolder <- tempfile("dqd_")
  on.exit(unlink(outputFolder, recursive = TRUE))

  connection <- DatabaseConnector::connect(connectionDetailsPlausibleAfterBirth)
  on.exit(DatabaseConnector::disconnect(connection), add = TRUE)

  # Set up test data: person with birth_datetime and visit on same day
  DatabaseConnector::executeSql(connection, "
    -- Update person 1 to have a specific birth_datetime
    UPDATE person
    SET birth_datetime = '1990-01-15 10:00:00'
    WHERE person_id = 1;
  ")

  # Truncate visit_occurrence and add only the test row
  DatabaseConnector::executeSql(connection, "
    DELETE FROM visit_occurrence;

    INSERT INTO visit_occurrence
    (visit_occurrence_id, person_id, visit_concept_id, visit_start_date, visit_start_datetime, visit_end_date, visit_end_datetime, visit_type_concept_id)
    VALUES
    (1, 1, 9201, '1990-01-15', '1990-01-15 10:00:00', '1990-01-15', '1990-01-15 10:00:00', 44818517);
  ")

  # Run the plausibleAfterBirth check
  results <- withCallingHandlers(
    executeDqChecks(
      connectionDetails = connectionDetailsPlausibleAfterBirth,
      cdmDatabaseSchema = cdmDatabaseSchemaEunomia,
      resultsDatabaseSchema = resultsDatabaseSchemaEunomia,
      cdmSourceName = "Eunomia",
      checkNames = "plausibleAfterBirth",
      tablesToExclude = c("COST", "CONCEPT", "VOCABULARY", "CONCEPT_ANCESTOR", "CONCEPT_RELATIONSHIP", "CONCEPT_CLASS", "CONCEPT_SYNONYM", "RELATIONSHIP", "DOMAIN"),
      outputFolder = outputFolder,
      writeToTable = FALSE
    ),
    warning = function(w) {
      if (grepl("^Missing check names", w$message)) {
        invokeRestart("muffleWarning")
      }
    }
  )

  # Get results for visit_occurrence.visit_start_date
  r <- results$CheckResults[results$CheckResults$checkName == "plausibleAfterBirth" &
    results$CheckResults$cdmTableName == "VISIT_OCCURRENCE" &
    results$CheckResults$cdmFieldName == "VISIT_START_DATE", ]

  # Debug: check what columns exist
  cat("Available columns in CheckResults:\n")
  print(colnames(results$CheckResults))

  # Debug: check all plausibleAfterBirth results
  cat("\nAll plausibleAfterBirth results:\n")
  print(results$CheckResults[results$CheckResults$checkName == "plausibleAfterBirth", ])

  # Debug: check all results for visit_occurrence
  cat("\nAll results for visit_occurrence:\n")
  visitResults <- results$CheckResults[grepl("visit", tolower(results$CheckResults$cdmTableName)), ]
  print(visitResults)

  # Should have 0 violations (event on same day as birth should be allowed)
  expect_equal(r$numViolatedRows, 0)
})

library(testthat)
testthat::local_edition(3)

test_that("Execute a single DQ check on Synthea/Eunomia", {
  testthat::skip_if_not_installed("Eunomia")
  outputFolder <- tempfile("dqd_")
  on.exit(unlink(outputFolder, recursive = TRUE))

  withCallingHandlers(
    results <- executeDqChecks(
      connectionDetails = connectionDetailsEunomia,
      cdmDatabaseSchema = cdmDatabaseSchemaEunomia,
      resultsDatabaseSchema = resultsDatabaseSchemaEunomia,
      cdmSourceName = "Eunomia",
      checkNames = "measurePersonCompleteness",
      outputFolder = outputFolder,
      writeToTable = FALSE
    ),
    warning = function(w) {
      if (grepl("^Missing check names", w$message)) {
        invokeRestart("muffleWarning")
      }
    }
  )

  expect_true(nrow(results$CheckResults) > 1)
})

test_that("Execute all TABLE checks on Synthea/Eunomia", {
  testthat::skip_if_not_installed("Eunomia")
  outputFolder <- tempfile("dqd_")
  on.exit(unlink(outputFolder, recursive = TRUE))

  results <- withCallingHandlers(
    executeDqChecks(
      connectionDetails = connectionDetailsEunomia,
      cdmDatabaseSchema = cdmDatabaseSchemaEunomia,
      resultsDatabaseSchema = resultsDatabaseSchemaEunomia,
      cdmSourceName = "Eunomia",
      checkLevels = "TABLE",
      outputFolder = outputFolder,
      writeToTable = FALSE
    ),
    warning = function(w) {
      if (grepl("^Missing check names", w$message)) {
        invokeRestart("muffleWarning")
      }
    }
  )

  expect_true(nrow(results$CheckResults) > 0)
})

test_that("Execute FIELD checks on Synthea/Eunomia", {
  testthat::skip_if_not_installed("Eunomia")
  outputFolder <- tempfile("dqd_")
  on.exit(unlink(outputFolder, recursive = TRUE))

  results <- withCallingHandlers(
    executeDqChecks(
      connectionDetails = connectionDetailsEunomia,
      cdmDatabaseSchema = cdmDatabaseSchemaEunomia,
      resultsDatabaseSchema = resultsDatabaseSchemaEunomia,
      cdmSourceName = "Eunomia",
      checkLevels = "FIELD",
      outputFolder = outputFolder,
      writeToTable = FALSE
    ),
    warning = function(w) {
      if (grepl("^Missing check names", w$message) || grepl("^DEPRECATION WARNING", w$message)) {
        invokeRestart("muffleWarning")
      }
    }
  )
  expect_true(nrow(results$CheckResults) > 0)
})

test_that("Execute CONCEPT checks on Synthea/Eunomia", {
  testthat::skip_if_not_installed("Eunomia")
  outputFolder <- tempfile("dqd_")
  on.exit(unlink(outputFolder, recursive = TRUE))
  results <- withCallingHandlers(
    executeDqChecks(
      connectionDetails = connectionDetailsEunomia,
      cdmDatabaseSchema = cdmDatabaseSchemaEunomia,
      resultsDatabaseSchema = resultsDatabaseSchemaEunomia,
      cdmSourceName = "Eunomia",
      checkLevels = "CONCEPT",
      conceptCheckThresholdLoc = system.file(
        "csv",
        "unittest_OMOP_CDMv5.3_Concept_Level.csv",
        package = "DataQualityDashboard"
      ),
      outputFolder = outputFolder,
      writeToTable = FALSE
    ),
    warning = function(w) {
      if (grepl("^Missing check names", w$message) || grepl("^DEPRECATION WARNING", w$message)) {
        invokeRestart("muffleWarning")
      }
    }
  )
  expect_true(nrow(results$CheckResults) > 0)
})

test_that("Execute observation period overlap check", {
  testthat::skip_if_not_installed("Eunomia")
  outputFolder <- tempfile("dqd_")
  on.exit(unlink(outputFolder, recursive = TRUE))

  # First, run the check on clean data (should pass)
  resultsClean <- withCallingHandlers(
    executeDqChecks(
      connectionDetails = connectionDetailsEunomiaOverlap,
      cdmDatabaseSchema = cdmDatabaseSchemaEunomia,
      resultsDatabaseSchema = resultsDatabaseSchemaEunomia,
      cdmSourceName = "Eunomia",
      checkNames = c("measureObservationPeriodOverlap"),
      outputFolder = outputFolder,
      writeToTable = FALSE
    ),
    warning = function(w) {
      if (grepl("^Missing check names", w$message)) {
        invokeRestart("muffleWarning")
      }
    }
  )

  expect_true(nrow(resultsClean$CheckResults) > 0)

  # Get the result for the observation period overlap check
  overlapResultClean <- resultsClean$CheckResults[
    resultsClean$CheckResults$checkName == "measureObservationPeriodOverlap",
  ]

  expect_true(nrow(overlapResultClean) == 1)

  # Now create overlapping observation periods to test the failure scenario
  connection <- DatabaseConnector::connect(connectionDetailsEunomiaOverlap)
  on.exit(DatabaseConnector::disconnect(connection), add = TRUE)

  # Insert overlapping observation periods for a test person
  # First, get an existing person_id
  personId <- DatabaseConnector::querySql(connection, "SELECT person_id FROM observation_period LIMIT 1;")[1, 1]

  # Insert overlapping observation periods
  DatabaseConnector::renderTranslateExecuteSql(connection,
    "INSERT INTO observation_period (observation_period_id, person_id, observation_period_start_date, observation_period_end_date, period_type_concept_id)
     VALUES
     (999999, @person_id, strftime('%s', '2020-01-01'), strftime('%s', '2020-06-30'), 44814724),
     (999998, @person_id, strftime('%s', '2020-04-01'), strftime('%s', '2020-12-31'), 44814724);",
    person_id = personId
  )

  # Run the check again with overlapping data (should fail)
  resultsOverlap <- withCallingHandlers(
    executeDqChecks(
      connectionDetails = connectionDetailsEunomiaOverlap,
      cdmDatabaseSchema = cdmDatabaseSchemaEunomia,
      resultsDatabaseSchema = resultsDatabaseSchemaEunomia,
      cdmSourceName = "Eunomia",
      checkNames = c("measureObservationPeriodOverlap"),
      outputFolder = outputFolder,
      writeToTable = FALSE
    ),
    warning = function(w) {
      if (grepl("^Missing check names", w$message)) {
        invokeRestart("muffleWarning")
      }
    }
  )

  expect_true(nrow(resultsOverlap$CheckResults) > 0)

  # Get the result for the observation period overlap check with overlapping data
  overlapResultOverlap <- resultsOverlap$CheckResults[
    resultsOverlap$CheckResults$checkName == "measureObservationPeriodOverlap",
  ]

  expect_true(nrow(overlapResultOverlap) == 1)

  # Verify that the check detected the overlap (should have violated rows)
  expect_true(overlapResultOverlap$numViolatedRows > 0)
  expect_true(overlapResultOverlap$pctViolatedRows > 0)

  # Verify that the clean data had no violations
  expect_true(overlapResultClean$numViolatedRows == 0)
  expect_true(overlapResultClean$pctViolatedRows == 0)

  # Clean up the overlapping data
  DatabaseConnector::renderTranslateExecuteSql(
    connection,
    "DELETE FROM observation_period WHERE observation_period_id IN (999999, 999998);"
  )

  # Now test back-to-back observation periods
  DatabaseConnector::renderTranslateExecuteSql(connection,
    "INSERT INTO observation_period (observation_period_id, person_id, observation_period_start_date, observation_period_end_date, period_type_concept_id)
     VALUES
     (999997, @person_id, strftime('%s', '2020-01-01'), strftime('%s', '2020-06-30'), 44814724),
     (999996, @person_id, strftime('%s', '2020-07-01'), strftime('%s', '2020-12-31'), 44814724);",
    person_id = personId
  )

  # Run the check again with back-to-back data (should fail)
  resultsBackToBack <- withCallingHandlers(
    executeDqChecks(
      connectionDetails = connectionDetailsEunomiaOverlap,
      cdmDatabaseSchema = cdmDatabaseSchemaEunomia,
      resultsDatabaseSchema = resultsDatabaseSchemaEunomia,
      cdmSourceName = "Eunomia",
      checkNames = c("measureObservationPeriodOverlap"),
      outputFolder = outputFolder,
      writeToTable = FALSE
    ),
    warning = function(w) {
      if (grepl("^Missing check names", w$message)) {
        invokeRestart("muffleWarning")
      }
    }
  )

  expect_true(nrow(resultsBackToBack$CheckResults) > 0)

  # Get the result for the observation period overlap check with back-to-back data
  overlapResultBackToBack <- resultsBackToBack$CheckResults[
    resultsBackToBack$CheckResults$checkName == "measureObservationPeriodOverlap",
  ]

  expect_true(nrow(overlapResultBackToBack) == 1)

  # Verify that the check detected the back-to-back periods (should have violated rows)
  expect_true(overlapResultBackToBack$numViolatedRows > 0)
  expect_true(overlapResultBackToBack$pctViolatedRows > 0)

  # Clean up the back-to-back data
  DatabaseConnector::renderTranslateExecuteSql(
    connection,
    "DELETE FROM observation_period WHERE observation_period_id IN (999997, 999996);"
  )
})

test_that("Execute a single DQ check on a cohort in Synthea/Eunomia", {
  testthat::skip_if_not_installed("Eunomia")
  # simulating cohort table entries using observation period data
  connection <- DatabaseConnector::connect(connectionDetailsEunomia)
  on.exit(DatabaseConnector::disconnect(connection), add = TRUE)
  fakeCohortId <- 123
  DatabaseConnector::renderTranslateExecuteSql(connection,
    "INSERT INTO @results_schema.cohort SELECT @cohort_id, person_id, observation_period_start_date, observation_period_end_date FROM @cdm_schema.observation_period LIMIT 10;",
    results_schema = resultsDatabaseSchemaEunomia,
    cohort_id = fakeCohortId,
    cdm_schema = cdmDatabaseSchemaEunomia
  )

  outputFolder <- tempfile("dqd_")
  on.exit(unlink(outputFolder, recursive = TRUE))

  withCallingHandlers(
    results <- executeDqChecks(
      connectionDetails = connectionDetailsEunomia,
      cdmDatabaseSchema = cdmDatabaseSchemaEunomia,
      resultsDatabaseSchema = resultsDatabaseSchemaEunomia,
      cdmSourceName = "Eunomia",
      checkNames = "measurePersonCompleteness",
      outputFolder = outputFolder,
      writeToTable = FALSE,
      cohortTableName = "cohort",
      cohortDefinitionId = fakeCohortId
    ),
    warning = function(w) {
      if (grepl("^Missing check names", w$message)) {
        invokeRestart("muffleWarning")
      }
    }
  )

  expect_true(nrow(results$CheckResults) > 1)
  DatabaseConnector::renderTranslateExecuteSql(connection,
    "DELETE FROM @results_schema.cohort WHERE cohort_definition_id = @cohort_id",
    results_schema = resultsDatabaseSchemaEunomia,
    cohort_id = fakeCohortId
  )
})

test_that("Execute a single DQ check on remote databases", {
  skip_on_cran()
  outputFolder <- tempfile("dqd_")
  on.exit(unlink(outputFolder, recursive = TRUE))

  dbTypes <- c(
    "oracle",
    "postgresql",
    "sql server",
    "redshift",
    "iris",
    "snowflake",
    "spark",
    "bigquery"
  )

  for (dbType in dbTypes) {
    print(sprintf("Processing database type: %s", dbType))

    if (dbType %in% c(
      "oracle",
      "postgresql",
      "sql server",
      "redshift",
      "spark"
    )) {
      cdmPattern <- "CDM5"
      if (dbType != "spark") {
        cdmSchemaPattern <- "CDM54"
      } else {
        cdmSchemaPattern <- "CDM"
      }
    } else if (dbType %in% c(
      "iris",
      "snowflake",
      "bigquery"
    )) {
      cdmPattern <- "CDM"
      if (dbType != "snowflake") {
        cdmSchemaPattern <- "CDM"
      } else {
        cdmSchemaPattern <- "CDM53"
      }
    }

    if (dbType == "bigquery") {
      bqKeyFile <- tempfile(fileext = ".json")
      writeLines(Sys.getenv("CDM_BIG_QUERY_KEY_FILE"), bqKeyFile)
      sysConnectionString <- gsub(
        "<keyfile path>",
        normalizePath(bqKeyFile, winslash = "/"),
        Sys.getenv("CDM_BIG_QUERY_CONNECTION_STRING")
      )
      sysUser <- Sys.getenv(sprintf("%s_BIG_QUERY_USER", cdmPattern))
      sysPassword <- Sys.getenv(sprintf("%s_BIG_QUERY_PASSWORD", cdmPattern))
      sysServer <- Sys.getenv(sprintf("%s_BIG_QUERY_SERVER", cdmPattern))
    } else {
      sysUser <- Sys.getenv(sprintf("%s_%s_USER", cdmPattern, toupper(gsub(" ", "_", dbType))))
      sysPassword <- Sys.getenv(sprintf("%s_%s_PASSWORD", cdmPattern, toupper(gsub(" ", "_", dbType))))
      sysServer <- Sys.getenv(sprintf("%s_%s_SERVER", cdmPattern, toupper(gsub(" ", "_", dbType))))
      if (sysServer == "") {
        sysConnectionString <- Sys.getenv(sprintf("%s_%s_CONNECTION_STRING", cdmPattern, toupper(gsub(" ", "_", dbType))))
      } else {
        sysConnectionString <- ""
      }
    }

    if ((sysUser != "" & sysPassword != "" & (sysServer != "" | sysConnectionString != "")) |
      (dbType == "bigquery" & sysConnectionString != "")) {
      print(sprintf("Connection details found for %s, proceeding...", dbType))

      if (dbType == "bigquery") {
        cdmDatabaseSchema <- Sys.getenv(sprintf("%s_BIG_QUERY_%s_SCHEMA", cdmPattern, cdmSchemaPattern))
        resultsDatabaseSchema <- Sys.getenv(sprintf("%s_BIG_QUERY_OHDSI_SCHEMA", cdmPattern))
      } else {
        cdmDatabaseSchema <- Sys.getenv(sprintf("%s_%s_%s_SCHEMA", cdmPattern, toupper(gsub(" ", "_", dbType)), cdmSchemaPattern))
        resultsDatabaseSchema <- Sys.getenv(sprintf("%s_%s_OHDSI_SCHEMA", cdmPattern, toupper(gsub(" ", "_", dbType))))
      }

      connectionDetails <- DatabaseConnector::createConnectionDetails(
        dbms = dbType,
        user = sysUser,
        password = sysPassword,
        server = sysServer,
        connectionString = sysConnectionString,
        pathToDriver = jdbcDriverFolder
      )

      # Verify connection before attempting to run checks
      connectionOk <- tryCatch(
        {
          verifyConnection(connectionDetails)
          TRUE
        },
        error = function(e) {
          warning(sprintf("Cannot connect to %s database: %s", dbType, e$message))
          FALSE
        }
      )

      if (!connectionOk) {
        print(sprintf("Skipping %s due to connection failure, continuing to next database...", dbType))
        next
      }

      print(sprintf("Connection verified for %s, running checks...", dbType))

      # Run the checks
      checkOk <- tryCatch(
        {
          results <- withCallingHandlers(
            executeDqChecks(
              connectionDetails = connectionDetails,
              cdmDatabaseSchema = cdmDatabaseSchema,
              resultsDatabaseSchema = resultsDatabaseSchema,
              cdmSourceName = "test",
              numThreads = 1,
              sqlOnly = FALSE,
              outputFolder = outputFolder,
              verboseMode = FALSE,
              writeToTable = FALSE,
              checkNames = "measurePersonCompleteness",
              cdmVersion = "5.4"
            ),
            warning = function(w) {
              if (grepl("^Missing check names", w$message)) {
                invokeRestart("muffleWarning")
              }
            }
          )
          expect_true(nrow(results$CheckResults) > 0)
          TRUE
        },
        error = function(e) {
          if (grepl("Connection reset|Communication link failure|SocketException", e$message, ignore.case = TRUE)) {
            warning(sprintf("Connection error on %s: %s", dbType, e$message))
            FALSE
          } else {
            # For non-connection errors, fail the test
            stop(e)
          }
        }
      )

      if (!checkOk) {
        print(sprintf("Skipping %s due to check execution failure, continuing to next database...", dbType))
        next
      }

      print(sprintf("Successfully completed checks for %s", dbType))
    } else {
      print(sprintf("No connection details found for %s, skipping...", dbType))
    }
  }
})

test_that("Check invalid cdm version", {
  testthat::skip_if_not_installed("Eunomia")
  outputFolder <- tempfile("dqd_")
  on.exit(unlink(outputFolder, recursive = TRUE))

  expect_error(
    executeDqChecks(
      connectionDetails = connectionDetailsEunomia,
      resultsDatabaseSchema = resultsDatabaseSchemaEunomia,
      cdmSourceName = "Eunomia",
      checkNames = "measurePersonCompleteness",
      outputFolder = outputFolder,
      writeToTable = FALSE,
      cdmVersion = "5.2.3.1"
    ),
    regexp = "^cdmVersion must contain a version of the form '5.X'"
  )
})

test_that("Execute DQ checks and write to table", {
  testthat::skip_if_not_installed("Eunomia")
  outputFolder <- tempfile("dqd_")
  on.exit(unlink(outputFolder, recursive = TRUE))

  withCallingHandlers(
    results <- executeDqChecks(
      connectionDetails = connectionDetailsEunomia,
      cdmDatabaseSchema = cdmDatabaseSchemaEunomia,
      resultsDatabaseSchema = resultsDatabaseSchemaEunomia,
      cdmSourceName = "Eunomia",
      checkNames = "measurePersonCompleteness",
      outputFolder = outputFolder,
      writeToTable = TRUE,
      writeTableName = "dqd_results"
    ),
    warning = function(w) {
      if (grepl("^Missing check names", w$message) || grepl("Column.*is of type.*logical.*but this is not supported", w$message)) {
        invokeRestart("muffleWarning")
      }
    }
  )
  connection <- DatabaseConnector::connect(connectionDetailsEunomia)
  on.exit(DatabaseConnector::disconnect(connection), add = TRUE)
  tableNames <- DatabaseConnector::getTableNames(connection = connection, databaseSchema = resultsDatabaseSchemaEunomia)
  expect_true("dqd_results" %in% tolower(tableNames))
  DatabaseConnector::renderTranslateExecuteSql(connection, "DROP TABLE @database_schema.dqd_results;", database_schema = resultsDatabaseSchemaEunomia)
})

test_that("Execute DQ checks using sqlOnly=TRUE and sqlOnlyUnionCount=4 and sqlOnlyIncrementalInsert=TRUE", {
  outputFolder <- tempfile("dqd_")
  on.exit(unlink(outputFolder, recursive = TRUE))
  sqlOnlyConnectionDetails <- DatabaseConnector::createConnectionDetails(dbms = "sql server", pathToDriver = "/")

  withCallingHandlers(
    results <- executeDqChecks(
      connectionDetails = sqlOnlyConnectionDetails,
      cdmDatabaseSchema = "@yourCdmSchema",
      resultsDatabaseSchema = "@yourResultsSchema",
      cdmSourceName = "Eunomia",
      checkNames = "measurePersonCompleteness",
      outputFolder = outputFolder,
      writeToTable = FALSE,
      sqlOnly = TRUE,
      sqlOnlyUnionCount = 4,
      sqlOnlyIncrementalInsert = TRUE,
      writeTableName = "dqdashboard_results"
    ),
    warning = function(w) {
      if (grepl("^Missing check names", w$message)) {
        invokeRestart("muffleWarning")
      }
    }
  )
  expect_true("ddlDqdResults.sql" %in% list.files(outputFolder))
  dqdSqlFile <- "TABLE_measurePersonCompleteness.sql"
  expect_true(dqdSqlFile %in% list.files(outputFolder))

  dqdSqlFilePath <- file.path(outputFolder, dqdSqlFile)
  expect_snapshot(cat(SqlRender::readSql(dqdSqlFilePath)))
})

test_that("Execute DQ checks using sqlOnly=TRUE and sqlOnlyUnionCount=1 and sqlOnlyIncrementalInsert=TRUE", {
  outputFolder <- tempfile("dqd_")
  on.exit(unlink(outputFolder, recursive = TRUE))
  sqlOnlyConnectionDetails <- DatabaseConnector::createConnectionDetails(dbms = "sql server", pathToDriver = "/")

  withCallingHandlers(
    results <- executeDqChecks(
      connectionDetails = sqlOnlyConnectionDetails,
      cdmDatabaseSchema = "@yourCdmSchema",
      resultsDatabaseSchema = "@yourResultsSchema",
      cdmSourceName = "Eunomia",
      checkNames = "measurePersonCompleteness",
      outputFolder = outputFolder,
      writeToTable = FALSE,
      sqlOnly = TRUE,
      sqlOnlyUnionCount = 1,
      sqlOnlyIncrementalInsert = TRUE,
      writeTableName = "dqdashboard_results"
    ),
    warning = function(w) {
      if (grepl("^Missing check names", w$message)) {
        invokeRestart("muffleWarning")
      }
    }
  )
  expect_true("ddlDqdResults.sql" %in% list.files(outputFolder))
  dqdSqlFile <- "TABLE_measurePersonCompleteness.sql"
  expect_true(dqdSqlFile %in% list.files(outputFolder))

  dqdSqlFilePath <- file.path(outputFolder, dqdSqlFile)
  expect_snapshot(cat(SqlRender::readSql(dqdSqlFilePath)))
})

test_that("Execute DQ checks using sqlOnly=TRUE and sqlOnlyUnionCount=1 and sqlOnlyIncrementalInsert=FALSE (the behavior in version <= 2.2.0)", {
  outputFolder <- tempfile("dqd_")
  on.exit(unlink(outputFolder, recursive = TRUE))
  sqlOnlyConnectionDetails <- DatabaseConnector::createConnectionDetails(dbms = "sql server", pathToDriver = "/")

  withCallingHandlers(
    results <- executeDqChecks(
      connectionDetails = sqlOnlyConnectionDetails,
      cdmDatabaseSchema = "@yourCdmSchema",
      resultsDatabaseSchema = "@yourResultsSchema",
      cdmSourceName = "Eunomia",
      checkNames = "measurePersonCompleteness",
      outputFolder = outputFolder,
      writeToTable = FALSE,
      sqlOnly = TRUE,
      sqlOnlyUnionCount = 1,
      sqlOnlyIncrementalInsert = FALSE,
      writeTableName = "dqdashboard_results"
    ),
    warning = function(w) {
      if (grepl("^Missing check names", w$message)) {
        invokeRestart("muffleWarning")
      }
    }
  )
  expect_true("ddlDqdResults.sql" %in% list.files(outputFolder))
  dqdSqlFile <- "measurePersonCompleteness.sql"
  expect_true(dqdSqlFile %in% list.files(outputFolder))

  dqdSqlFilePath <- file.path(outputFolder, dqdSqlFile)
  expect_snapshot(cat(SqlRender::readSql(dqdSqlFilePath)))
})

test_that("Incremental insert SQL is valid.", {
  testthat::skip_if_not_installed("Eunomia")
  outputFolder <- tempfile("dqd_")
  on.exit(unlink(outputFolder, recursive = TRUE))

  withCallingHandlers(
    results <- executeDqChecks(
      connectionDetails = connectionDetailsEunomia,
      cdmDatabaseSchema = cdmDatabaseSchemaEunomia,
      resultsDatabaseSchema = resultsDatabaseSchemaEunomia,
      cdmSourceName = "Eunomia",
      checkNames = "measurePersonCompleteness",
      outputFolder = outputFolder,
      writeToTable = FALSE,
      sqlOnly = TRUE,
      sqlOnlyUnionCount = 4,
      sqlOnlyIncrementalInsert = TRUE,
      writeTableName = "dqd_results"
    ),
    warning = function(w) {
      if (grepl("^Missing check names", w$message)) {
        invokeRestart("muffleWarning")
      }
    }
  )

  ddlSqlFile <- file.path(outputFolder, "ddlDqdResults.sql")
  ddlSql <- SqlRender::readSql(ddlSqlFile)
  checkSqlFile <- file.path(outputFolder, "TABLE_measurePersonCompleteness.sql")
  checkSql <- SqlRender::readSql(checkSqlFile)

  connection <- DatabaseConnector::connect(connectionDetailsEunomia)
  on.exit(DatabaseConnector::disconnect(connection), add = TRUE)
  DatabaseConnector::executeSql(connection = connection, sql = ddlSql)
  DatabaseConnector::executeSql(connection = connection, sql = checkSql)

  checkResults <- DatabaseConnector::renderTranslateQuerySql(connection, "SELECT * FROM @database_schema.dqd_results;", database_schema = resultsDatabaseSchemaEunomia)
  expect_equal(nrow(checkResults), 16)

  DatabaseConnector::renderTranslateExecuteSql(connection, "DROP TABLE @database_schema.dqd_results;", database_schema = resultsDatabaseSchemaEunomia)
})

test_that("Multiple cdm_source rows triggers warning.", {
  testthat::skip_if_not_installed("Eunomia")
  outputFolder <- tempfile("dqd_")
  on.exit(unlink(outputFolder, recursive = TRUE))

  connectionDetailsEunomiaCS <- Eunomia::getEunomiaConnectionDetails()
  connection <- DatabaseConnector::connect(connectionDetailsEunomiaCS)
  on.exit(DatabaseConnector::disconnect(connection), add = TRUE)
  DatabaseConnector::renderTranslateExecuteSql(connection, "INSERT INTO @database_schema.cdm_source VALUES ('foo',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);", database_schema = cdmDatabaseSchemaEunomia)

  w <- capture_warnings(
    results <- executeDqChecks(
      connectionDetails = connectionDetailsEunomiaCS,
      cdmDatabaseSchema = cdmDatabaseSchemaEunomia,
      resultsDatabaseSchema = resultsDatabaseSchemaEunomia,
      cdmSourceName = "Eunomia",
      checkNames = "measurePersonCompleteness",
      outputFolder = outputFolder,
      writeToTable = FALSE
    )
  )

  expect_match(w, "Missing check names", all = FALSE)
  expect_match(w, "The cdm_source table has", all = FALSE)

  expect_true(nrow(results$CheckResults) > 1)
})

test_that("Execute checks on Synthea/Eunomia to test new variable executionTimeSeconds", {
  testthat::skip_if_not_installed("Eunomia")
  outputFolder <- tempfile("dqd_")
  on.exit(unlink(outputFolder, recursive = TRUE))
  results <- withCallingHandlers(
    executeDqChecks(
      connectionDetails = connectionDetailsEunomia,
      cdmDatabaseSchema = cdmDatabaseSchemaEunomia,
      resultsDatabaseSchema = resultsDatabaseSchemaEunomia,
      cdmSourceName = "Eunomia",
      checkNames = "measurePersonCompleteness",
      conceptCheckThresholdLoc = system.file(
        "csv",
        "unittest_OMOP_CDMv5.3_Concept_Level.csv",
        package = "DataQualityDashboard"
      ),
      outputFolder = outputFolder,
      writeToTable = FALSE
    ),
    warning = function(w) {
      if (grepl("^Missing check names", w$message)) {
        invokeRestart("muffleWarning")
      }
    }
  )
  expect_true(is.numeric(results$executionTimeSeconds))
})


test_that("checkNames are filtered by checkSeverity", {
  testthat::skip_if_not_installed("Eunomia")
  outputFolder <- tempfile("dqd_")
  on.exit(unlink(outputFolder, recursive = TRUE))

  results <- withCallingHandlers(
    executeDqChecks(
      connectionDetails = connectionDetailsEunomia,
      cdmDatabaseSchema = cdmDatabaseSchemaEunomia,
      resultsDatabaseSchema = resultsDatabaseSchemaEunomia,
      cdmSourceName = "Eunomia",
      checkSeverity = "fatal",
      outputFolder = outputFolder,
      writeToTable = FALSE
    ),
    warning = function(w) {
      if (grepl("^Missing check names", w$message)) {
        invokeRestart("muffleWarning")
      }
    }
  )

  expectedCheckNames <- c(
    "cdmTable", "cdmField", "isRequired", "cdmDatatype",
    "isPrimaryKey", "isForeignKey", "measureObservationPeriodOverlap"
  )
  expect_true(all(results$CheckResults$checkName %in% expectedCheckNames))
})

test_that("Execute a single DQ check on DuckDB", {
  testthat::skip_on_cran()
  testthat::skip_on_ci()
  testthat::skip_if_not_installed("Eunomia")
  testthat::skip_if_not_installed("duckdb")
  outputFolder <- tempfile("dqd_")
  on.exit(unlink(outputFolder, recursive = TRUE))

  # Get Eunomia database file in DuckDB format
  eunomiaDbPath <- Eunomia::getDatabaseFile(
    datasetName = "GiBleed",
    dbms = "duckdb"
  )

  # Create DuckDB connection details using the Eunomia database file
  duckdbConnectionDetails <- DatabaseConnector::createConnectionDetails(
    dbms = "duckdb",
    server = eunomiaDbPath
  )

  withCallingHandlers(
    results <- executeDqChecks(
      connectionDetails = duckdbConnectionDetails,
      cdmDatabaseSchema = "main",
      resultsDatabaseSchema = "main",
      cdmSourceName = "DuckDB Test",
      checkNames = "measurePersonCompleteness",
      outputFolder = outputFolder,
      writeToTable = FALSE
    ),
    warning = function(w) {
      if (grepl("^Missing check names", w$message)) {
        invokeRestart("muffleWarning")
      }
    }
  )

  expect_true(nrow(results$CheckResults) > 0)
})

library(testthat)

test_that("Write DB results to json", {
  testthat::skip_if_not_installed("Eunomia")
  outputFolder <- tempfile("dqd_")
  on.exit(unlink(outputFolder, recursive = TRUE))
  connectionDetailsEunomia <- Eunomia::getEunomiaConnectionDetails()
  cdmDatabaseSchemaEunomia <- "main"
  resultsDatabaseSchemaEunomia <- "main"
  writeTableName <- "dqd_db_results"

  # Suppress both DatabaseConnector warnings and Missing check names warning
  results <- withCallingHandlers(
    DataQualityDashboard::executeDqChecks(
      connectionDetails = connectionDetailsEunomia,
      cdmDatabaseSchema = cdmDatabaseSchemaEunomia,
      resultsDatabaseSchema = resultsDatabaseSchemaEunomia,
      cdmSourceName = "Eunomia",
      checkNames = "measurePersonCompleteness",
      outputFolder = outputFolder,
      writeToTable = TRUE,
      writeTableName = writeTableName
    ),
    warning = function(w) {
      msg <- conditionMessage(w)
      # Suppress warnings about converting logical columns and missing check names
      if (grepl("Converting to numeric", msg) || grepl("Missing check names", msg)) {
        invokeRestart("muffleWarning")
      }
    }
  )

  connection <- DatabaseConnector::connect(connectionDetailsEunomia)
  on.exit(DatabaseConnector::disconnect(connection), add = TRUE)

  testExportFile <- "dq-result-test.json"

  DataQualityDashboard::writeDBResultsToJson(
    connection,
    resultsDatabaseSchemaEunomia,
    cdmDatabaseSchemaEunomia,
    writeTableName,
    outputFolder,
    testExportFile
  )

  # Check that file was exported properly
  expect_true(file.exists(file.path(outputFolder, testExportFile)))

  # Check that export length matches length of db table
  results <- jsonlite::fromJSON(file.path(outputFolder, testExportFile))
  tableRows <- DatabaseConnector::renderTranslateQuerySql(
    connection,
    sql = "select count(*) from @resultsDatabaseSchema.@writeTableName;",
    resultsDatabaseSchema = resultsDatabaseSchemaEunomia,
    writeTableName = writeTableName,
    snakeCaseToCamelCase = TRUE
  )
  expect_true(length(results$CheckResults) == tableRows)
})

library(testthat)

test_that("Camel correctly converted to snake and back", {
  testthat::skip_if_not_installed("Eunomia")
  outputFolder <- tempfile("dqd_")
  on.exit(unlink(outputFolder, recursive = TRUE))

  withCallingHandlers(
    results <- executeDqChecks(
      connectionDetails = connectionDetailsEunomia,
      cdmDatabaseSchema = cdmDatabaseSchemaEunomia,
      resultsDatabaseSchema = resultsDatabaseSchemaEunomia,
      cdmSourceName = "Eunomia",
      checkNames = "measurePersonCompleteness",
      outputFolder = outputFolder,
      outputFile = "foo.json",
      writeToTable = FALSE
    ),
    warning = function(w) {
      if (grepl("^Missing check names", w$message)) {
        invokeRestart("muffleWarning")
      }
    }
  )

  jsonFilePath <- file.path(outputFolder, "foo.json")
  expect_warning(
    convertJsonResultsFileCase(jsonFilePath, writeToFile = FALSE, targetCase = "camel"),
    regexp = "^File is already in camelcase!"
  )
  snakeResults <- convertJsonResultsFileCase(jsonFilePath, writeToFile = TRUE, outputFolder, outputFile = "snake.json", targetCase = "snake")
  snakeNames <- c("NUM_VIOLATED_ROWS", "PCT_VIOLATED_ROWS", "NUM_DENOMINATOR_ROWS", "EXECUTION_TIME", "QUERY_TEXT", "CHECK_NAME", "CHECK_LEVEL", "CHECK_DESCRIPTION", "CDM_TABLE_NAME", "SQL_FILE", "CATEGORY", "CONTEXT", "checkId", "FAILED", "PASSED", "IS_ERROR", "NOT_APPLICABLE", "THRESHOLD_VALUE")


  expect_equal(length(snakeResults), 7)
  expect_true(setequal(names(snakeResults$CheckResults), snakeNames))

  snakeFilePath <- file.path(outputFolder, "snake.json")
  expect_warning(
    convertJsonResultsFileCase(snakeFilePath, writeToFile = FALSE, targetCase = "snake"),
    regexp = "^File is already in snakecase!"
  )
  camelResults <- convertJsonResultsFileCase(snakeFilePath, writeToFile = TRUE, outputFolder, targetCase = "camel")
  camelNames <- c("numViolatedRows", "pctViolatedRows", "numDenominatorRows", "executionTime", "queryText", "checkName", "checkLevel", "checkDescription", "cdmTableName", "sqlFile", "category", "context", "checkId", "failed", "passed", "isError", "notApplicable", "thresholdValue")
  camelFilePath <- file.path(outputFolder, "snake_camel.json")



  expect_equal(length(camelResults), 7)
  expect_true(setequal(names(camelResults$CheckResults), camelNames))
  expect_true(file.exists(camelFilePath))

  origJson <- jsonlite::toJSON(results)
  reconvertedJson <- jsonlite::toJSON(camelResults)
  expect_equal(origJson, reconvertedJson)
})

test_that("Invalid case throws error", {
  expect_error(
    convertJsonResultsFileCase("bar.json", writeToFile = FALSE, targetCase = "foo"),
    regexp = "^targetCase must be either 'camel' or 'snake'."
  )
})

test_that("Output folder required when writing to file", {
  expect_error(
    convertJsonResultsFileCase("bar.json", writeToFile = TRUE, targetCase = "camel"),
    regexp = "^You must specify an output folder if writing to file."
  )
})

library(testthat)

test_that("Execute reEvaluateThresholds on Synthea/Eunomia", {
  testthat::skip_if_not_installed("Eunomia")
  outputFolder <- tempfile("dqd_")
  on.exit(unlink(outputFolder, recursive = TRUE))

  withCallingHandlers(
    results <- executeDqChecks(
      connectionDetails = connectionDetailsEunomia,
      cdmDatabaseSchema = cdmDatabaseSchemaEunomia,
      resultsDatabaseSchema = resultsDatabaseSchemaEunomia,
      cdmSourceName = "Eunomia",
      checkNames = "measurePersonCompleteness",
      outputFolder = outputFolder,
      writeToTable = FALSE
    ),
    warning = function(w) {
      if (grepl("^Missing check names", w$message)) {
        invokeRestart("muffleWarning")
      }
    }
  )

  jsonPath <- list.files(outputFolder, ".json", full.names = TRUE)

  results2 <- reEvaluateThresholds(
    jsonFilePath = jsonPath,
    outputFolder = outputFolder,
    outputFile = "reEvaluated.txt"
  )

  expect_type(results2, "list")
})

# Copyright 2026 Observational Health Data Sciences and Informatics
#
# This file is part of DataQualityDashboard
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Write DQD results database table to json
#'
#' @param connection                A connection object
#' @param resultsDatabaseSchema     The fully qualified database name of the results schema
#' @param cdmDatabaseSchema         The fully qualified database name of the CDM schema
#' @param writeTableName            Name of DQD results table in the database to read from
#' @param outputFolder              The folder to output the json results file to
#' @param outputFile                The output filename of the json results file
#'
#' @return NULL (writes results to JSON file)
#'
#' @export
#'

writeDBResultsToJson <- function(connection,
                                 resultsDatabaseSchema,
                                 cdmDatabaseSchema,
                                 writeTableName,
                                 outputFolder,
                                 outputFile) {
  metadata <- DatabaseConnector::renderTranslateQuerySql(
    connection,
    sql = "select * from @cdmDatabaseSchema.cdm_source;",
    snakeCaseToCamelCase = TRUE,
    cdmDatabaseSchema = cdmDatabaseSchema
  )

  checkResults <- DatabaseConnector::renderTranslateQuerySql(
    connection,
    sql = "select * from @resultsDatabaseSchema.@writeTableName;",
    snakeCaseToCamelCase = TRUE,
    resultsDatabaseSchema = resultsDatabaseSchema,
    writeTableName = writeTableName
  )

  # Quick patch for missing value issues related to SQL Only Implementation
  checkResults["error"][checkResults["error"] == ""] <- NA
  checkResults["warning"][checkResults["warning"] == ""] <- NA
  checkResults["executionTime"][checkResults["executionTime"] == ""] <- "0 secs"
  checkResults["queryText"][checkResults["queryText"] == ""] <- "[Generated via SQL Only]"

  overview <- .summarizeResults(
    checkResults = checkResults
  )

  # Quick patch for non-camel-case column name
  names(checkResults)[names(checkResults) == "checkid"] <- "checkId"

  allResults <- list(
    startTimestamp = Sys.time(),
    endTimestamp = Sys.time(),
    executionTime = "0 secs",
    CheckResults = checkResults,
    Metadata = metadata,
    Overview = overview
  )

  .writeResultsToJson(
    allResults,
    outputFolder,
    outputFile
  )
}

# Copyright 2026 Observational Health Data Sciences and Informatics
#
# This file is part of DataQualityDashboard
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Write JSON Results to SQL Table
#'
#' @param connectionDetails         A connectionDetails object for connecting to the CDM database
#' @param resultsDatabaseSchema     The fully qualified database name of the results schema
#' @param jsonFilePath              Path to the JSON results file generated using the execute function
#' @param writeTableName            Name of table in the database to write results to
#' @param cohortDefinitionId        If writing results for a single cohort this is the ID that will be appended to the table name
#' @param singleTable               If TRUE, writes all results to a single table. If FALSE (default), writes to 3 separate tables by check level (table, field, concept) (NOTE this default behavior will be deprecated in the future)
#'
#' @return NULL (writes results to database table)
#'
#' @export

writeJsonResultsToTable <- function(connectionDetails,
                                    resultsDatabaseSchema,
                                    jsonFilePath,
                                    writeTableName = "dqdashboard_results",
                                    cohortDefinitionId = c(),
                                    singleTable = FALSE) {
  jsonData <- jsonlite::read_json(jsonFilePath)
  checkResults <- lapply(jsonData$CheckResults, function(cr) {
    cr[sapply(cr, is.null)] <- NA
    as.data.frame(cr)
  })
  df <- do.call(plyr::rbind.fill, checkResults)

  if (singleTable) {
    # Write all results to a single table
    .writeResultsToTable(
      connectionDetails = connectionDetails,
      resultsDatabaseSchema = resultsDatabaseSchema,
      checkResults = df,
      writeTableName = writeTableName,
      cohortDefinitionId = cohortDefinitionId
    )
  } else {
    # Write to 3 separate tables by check level (backward compatibility)
    warning("Writing to 3 separate tables by check level is deprecated and will be removed in a future version. Use singleTable = TRUE to write DQD results to a single table.")

    # Split results by check level
    tableLevelResults <- df[df$checkLevel == "TABLE", ]
    fieldLevelResults <- df[df$checkLevel == "FIELD", ]
    conceptLevelResults <- df[df$checkLevel == "CONCEPT", ]

    # Write table-level results
    if (nrow(tableLevelResults) > 0) {
      tableTableName <- paste0(writeTableName, "_table")
      .writeResultsToTable(
        connectionDetails = connectionDetails,
        resultsDatabaseSchema = resultsDatabaseSchema,
        checkResults = tableLevelResults,
        writeTableName = tableTableName,
        cohortDefinitionId = cohortDefinitionId
      )
    }

    # Write field-level results
    if (nrow(fieldLevelResults) > 0) {
      fieldTableName <- paste0(writeTableName, "_field")
      .writeResultsToTable(
        connectionDetails = connectionDetails,
        resultsDatabaseSchema = resultsDatabaseSchema,
        checkResults = fieldLevelResults,
        writeTableName = fieldTableName,
        cohortDefinitionId = cohortDefinitionId
      )
    }

    # Write concept-level results
    if (nrow(conceptLevelResults) > 0) {
      conceptTableName <- paste0(writeTableName, "_concept")
      .writeResultsToTable(
        connectionDetails = connectionDetails,
        resultsDatabaseSchema = resultsDatabaseSchema,
        checkResults = conceptLevelResults,
        writeTableName = conceptTableName,
        cohortDefinitionId = cohortDefinitionId
      )
    }
  }
}


#' Write JSON Results to CSV file
#'
#' @param jsonPath    Path to the JSON results file generated using the execute function
#' @param csvPath     Path to the CSV output file
#' @param columns     (OPTIONAL) List of desired columns
#' @param delimiter   (OPTIONAL) CSV delimiter
#'
#' @return NULL (writes results to CSV file)
#'
#' @export

writeJsonResultsToCsv <- function(jsonPath,
                                  csvPath,
                                  columns = c(
                                    "checkId", "failed", "passed",
                                    "isError", "notApplicable",
                                    "checkName", "checkDescription",
                                    "thresholdValue", "notesValue",
                                    "checkLevel", "category",
                                    "subcategory", "context",
                                    "checkLevel", "cdmTableName",
                                    "cdmFieldName", "conceptId",
                                    "unitConceptId", "numViolatedRows",
                                    "pctViolatedRows", "numDenominatorRows",
                                    "executionTime", "notApplicableReason",
                                    "error", "queryText"
                                  ),
                                  delimiter = ",") {
  tryCatch(
    expr = {
      ParallelLogger::logInfo(sprintf("Loading results from %s", jsonPath))
      jsonData <- jsonlite::read_json(jsonPath)
      checkResults <- lapply(jsonData$CheckResults, function(cr) {
        cr[sapply(cr, is.null)] <- NA
        as.data.frame(cr)
      })
      .writeResultsToCsv(
        checkResults = do.call(plyr::rbind.fill, checkResults),
        csvPath = csvPath,
        columns = columns,
        delimiter = delimiter
      )
    },
    error = function(e) {
      ParallelLogger::logError(sprintf("Writing to CSV file failed: %s", e$message))
    }
  )
}

library(testthat)

test_that("Write JSON results", {
  testthat::skip_if_not_installed("Eunomia")
  outputFolder <- tempfile("dqd_")
  on.exit(unlink(outputFolder, recursive = TRUE))

  withCallingHandlers(
    results <- executeDqChecks(
      connectionDetails = connectionDetailsEunomia,
      cdmDatabaseSchema = cdmDatabaseSchemaEunomia,
      resultsDatabaseSchema = resultsDatabaseSchemaEunomia,
      cdmSourceName = "Eunomia",
      checkNames = "measurePersonCompleteness",
      outputFolder = outputFolder,
      writeToTable = FALSE
    ),
    warning = function(w) {
      if (grepl("^Missing check names", w$message)) {
        invokeRestart("muffleWarning")
      }
    }
  )

  jsonPath <- list.files(outputFolder, ".json", full.names = TRUE)
  csvPath <- file.path(outputFolder, "results.csv")
  writeJsonResultsToCsv(
    jsonPath = jsonPath,
    csvPath = csvPath
  )
  expect_true(file.exists(csvPath))

  # Test default behavior (3 separate tables with deprecation warning)
  expect_warning(
    DataQualityDashboard::writeJsonResultsToTable(
      connectionDetails = connectionDetailsEunomia,
      resultsDatabaseSchema = resultsDatabaseSchemaEunomia,
      jsonFilePath = jsonPath,
      writeTableName = "dqd_json_results"
    ),
    "Writing to 3 separate tables by check level is deprecated"
  )
  connection <- DatabaseConnector::connect(connectionDetailsEunomia)
  on.exit(DatabaseConnector::disconnect(connection), add = TRUE)
  tableNames <- DatabaseConnector::getTableNames(connection = connection, databaseSchema = resultsDatabaseSchemaEunomia)
  expect_true("dqd_json_results_table" %in% tolower(tableNames))
  DatabaseConnector::renderTranslateExecuteSql(connection, "DROP TABLE @database_schema.dqd_json_results_table;", database_schema = resultsDatabaseSchemaEunomia)
})

test_that("Write JSON results with singleTable parameter", {
  testthat::skip_if_not_installed("Eunomia")
  outputFolder <- tempfile("dqd_")
  on.exit(unlink(outputFolder, recursive = TRUE))

  withCallingHandlers(
    results <- executeDqChecks(
      connectionDetails = connectionDetailsEunomia,
      cdmDatabaseSchema = cdmDatabaseSchemaEunomia,
      resultsDatabaseSchema = resultsDatabaseSchemaEunomia,
      cdmSourceName = "Eunomia",
      checkNames = "measurePersonCompleteness",
      outputFolder = outputFolder,
      writeToTable = FALSE
    ),
    warning = function(w) {
      if (grepl("^Missing check names", w$message)) {
        invokeRestart("muffleWarning")
      }
    }
  )

  jsonPath <- list.files(outputFolder, ".json", full.names = TRUE)

  # Test singleTable = TRUE (new behavior)
  expect_warning(
    DataQualityDashboard::writeJsonResultsToTable(
      connectionDetails = connectionDetailsEunomia,
      resultsDatabaseSchema = resultsDatabaseSchemaEunomia,
      jsonFilePath = jsonPath,
      writeTableName = "dqd_single_table",
      singleTable = TRUE
    ),
    NA
  )

  connection <- DatabaseConnector::connect(connectionDetailsEunomia)
  on.exit(DatabaseConnector::disconnect(connection), add = TRUE)
  tableNames <- DatabaseConnector::getTableNames(connection = connection, databaseSchema = resultsDatabaseSchemaEunomia)
  expect_true("dqd_single_table" %in% tolower(tableNames))
  DatabaseConnector::renderTranslateExecuteSql(connection, "DROP TABLE @database_schema.dqd_single_table;", database_schema = resultsDatabaseSchemaEunomia)

  # Test singleTable = FALSE (old behavior with deprecation warning)
  # Since we only have table-level checks, only the table-level table will be created
  expect_warning(
    DataQualityDashboard::writeJsonResultsToTable(
      connectionDetails = connectionDetailsEunomia,
      resultsDatabaseSchema = resultsDatabaseSchemaEunomia,
      jsonFilePath = jsonPath,
      writeTableName = "dqd_separate_tables",
      singleTable = FALSE
    ),
    "Writing to 3 separate tables by check level is deprecated"
  )

  tableNames <- DatabaseConnector::getTableNames(connection = connection, databaseSchema = resultsDatabaseSchemaEunomia)
  # Check that table-level table was created (only one that should exist for this test)
  expect_true("dqd_separate_tables_table" %in% tolower(tableNames))

  # Clean up
  DatabaseConnector::renderTranslateExecuteSql(connection, "DROP TABLE @database_schema.dqd_separate_tables_table;", database_schema = resultsDatabaseSchemaEunomia)
})

library(testthat)

# When devtools::load_all is run, create symbolic link for sql directory
# Allows testing with devtools::test
# Only run in local development, not in CI/covr
if (Sys.getenv("DEVTOOLS_LOAD") == "true" &&
  !identical(Sys.getenv("CI"), "true") &&
  !identical(Sys.getenv("COVR"), "true")) {
  print("setting sql folder symbolic link")
  packageRoot <- normalizePath(system.file("..", package = "DataQualityDashboard"))
  # Create symbolic link so code can be used in devtools::test()
  tryCatch(
    {
      if (requireNamespace("R.utils", quietly = TRUE)) {
        R.utils::createLink(link = file.path(packageRoot, "sql"), system.file("sql", package = "DataQualityDashboard"))
        options("use.devtools.sql_shim" = TRUE)
      } else {
        # Fallback: create a simple file.copy if R.utils is not available
        sqlLinkPath <- file.path(packageRoot, "sql")
        sqlPackagePath <- system.file("sql", package = "DataQualityDashboard")
        if (!dir.exists(sqlLinkPath)) {
          dir.create(sqlLinkPath, recursive = TRUE)
        }
        file.copy(from = sqlPackagePath, to = dirname(sqlLinkPath), recursive = TRUE, overwrite = TRUE)
        options("use.devtools.sql_shim" = TRUE)
      }
    },
    error = function(e) {
      warning("Failed to create symbolic link for SQL directory: ", e$message)
      # Continue without the symbolic link - the package should still work
    }
  )
}
options(connectionObserver = NULL)

# Copyright 2026 Observational Health Data Sciences and Informatics
#
# This file is part of DataQualityDashboard
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Internal function to run and process each data quality check.
#'
#' @param checkDescription          The description of the data quality check
#' @param tableChecks               A dataframe containing the table checks
#' @param fieldChecks               A dataframe containing the field checks
#' @param conceptChecks             A dataframe containing the concept checks
#' @param connectionDetails         A connectionDetails object for connecting to the CDM database
#' @param connection                A connection for connecting to the CDM database using the DatabaseConnector::connect(connectionDetails) function.
#' @param cdmDatabaseSchema         The fully qualified database name of the CDM schema
#' @param vocabDatabaseSchema       The fully qualified database name of the vocabulary schema (default is to set it as the cdmDatabaseSchema)
#' @param resultsDatabaseSchema     The fully qualified database name of the results schema
#' @param writeTableName            The table tor write DQD results to. Used when sqlOnly or writeToTable is True.
#' @param cohortDatabaseSchema      The schema where the cohort table is located.
#' @param cohortTableName           The name of the cohort table.
#' @param cohortDefinitionId        The cohort definition id for the cohort you wish to run the DQD on. The package assumes a standard OHDSI cohort table called 'Cohort'
#' @param outputFolder              The folder to output logs and SQL files to
#' @param sqlOnlyUnionCount         (OPTIONAL) How many SQL commands to union before inserting them into output table (speeds processing when queries done in parallel). Default is 1.
#' @param sqlOnlyIncrementalInsert  (OPTIONAL) Boolean to determine whether insert check results and associated metadata into output table.  Default is FALSE (for backwards compatability to <= v2.2.0)
#' @param sqlOnly                   Should the SQLs be executed (FALSE) or just returned (TRUE)?
#'
#' @return A dataframe containing the check results or SQL queries (NULL if sqlOnlyIncrementalInsert is TRUE)
#'
#' @import magrittr
#'
#' @keywords internal
#'
.runCheck <- function(checkDescription,
                      tableChecks,
                      fieldChecks,
                      conceptChecks,
                      connectionDetails,
                      connection,
                      cdmDatabaseSchema,
                      vocabDatabaseSchema,
                      resultsDatabaseSchema,
                      writeTableName,
                      cohortDatabaseSchema,
                      cohortTableName,
                      cohortDefinitionId,
                      outputFolder,
                      sqlOnlyUnionCount,
                      sqlOnlyIncrementalInsert,
                      sqlOnly) {
  ParallelLogger::logInfo(sprintf("Processing check description: %s", checkDescription$checkName))

  filterExpression <- sprintf(
    "%sChecks %%>%% dplyr::filter(%s)",
    tolower(checkDescription$checkLevel),
    checkDescription$evaluationFilter
  )
  checks <- eval(parse(text = filterExpression))

  if (length(cohortDefinitionId > 0)) {
    cohort <- TRUE
  } else {
    cohort <- FALSE
  }

  if (nrow(checks) > 0) {
    dfs <- apply(X = checks, MARGIN = 1, function(check) {
      columns <- lapply(names(check), function(c) {
        setNames(check[c], c)
      })

      params <- c(
        list(dbms = connectionDetails$dbms),
        list(sqlFilename = checkDescription$sqlFile),
        list(packageName = "DataQualityDashboard"),
        list(warnOnMissingParameters = FALSE),
        list(cdmDatabaseSchema = cdmDatabaseSchema),
        list(cohortDatabaseSchema = cohortDatabaseSchema),
        list(cohortTableName = cohortTableName),
        list(cohortDefinitionId = cohortDefinitionId),
        list(vocabDatabaseSchema = vocabDatabaseSchema),
        list(cohort = cohort),
        unlist(columns, recursive = FALSE)
      )

      sql <- do.call(SqlRender::loadRenderTranslateSql, params)

      if (sqlOnly && sqlOnlyIncrementalInsert) {
        checkQuery <- .createSqlOnlyQueries(
          params,
          check,
          tableChecks,
          fieldChecks,
          conceptChecks,
          sql,
          connectionDetails,
          checkDescription
        )
        data.frame(query = checkQuery)
      } else if (sqlOnly) {
        write(x = sql, file = file.path(
          outputFolder,
          sprintf("%s.sql", checkDescription$checkName)
        ), append = TRUE)
        data.frame()
      } else {
        .processCheck(
          connection = connection,
          connectionDetails = connectionDetails,
          check = check,
          checkDescription = checkDescription,
          sql = sql,
          outputFolder = outputFolder
        )
      }
    })

    dfs <- do.call(rbind, dfs)

    if (sqlOnly && sqlOnlyIncrementalInsert) {
      sqlToUnion <- dfs$query
      if (length(sqlToUnion) > 0) {
        .writeSqlOnlyQueries(sqlToUnion, sqlOnlyUnionCount, resultsDatabaseSchema, writeTableName, connectionDetails$dbms, outputFolder, checkDescription)
        return(NULL)
      }
    } else {
      return(dfs)
    }
  } else {
    ParallelLogger::logWarn(paste0("Warning: Evaluation resulted in no checks: ", filterExpression))
    return(data.frame())
  }
}

# Copyright 2026 Observational Health Data Sciences and Informatics
#
# This file is part of DataQualityDashboard
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Internal function to send the fully qualified sql to the database and return the numerical result.
#'
#' @param connection                A connection for connecting to the CDM database using the DatabaseConnector::connect(connectionDetails) function.
#' @param connectionDetails         A connectionDetails object for connecting to the CDM database.
#' @param check                     The data quality check
#' @param checkDescription          The description of the data quality check
#' @param sql                       The fully qualified sql for the data quality check
#' @param outputFolder              The folder to output logs and SQL files to.
#'
#' @return A dataframe containing the check results
#'
#' @keywords internal
#'

.processCheck <- function(connection,
                          connectionDetails,
                          check,
                          checkDescription,
                          sql,
                          outputFolder) {
  singleThreaded <- TRUE
  start <- Sys.time()
  if (is.null(connection)) {
    singleThreaded <- FALSE
    connection <- DatabaseConnector::connect(connectionDetails = connectionDetails)
    on.exit(DatabaseConnector::disconnect(connection = connection))
  }

  errorReportFile <- file.path(
    outputFolder, "errors",
    sprintf(
      "%s_%s_%s_%s.txt",
      checkDescription$checkLevel,
      checkDescription$checkName,
      check["cdmTableName"],
      check["cdmFieldName"]
    )
  )
  tryCatch(
    expr = {
      if (singleThreaded) {
        if (.needsAutoCommit(connectionDetails = connectionDetails, connection = connection)) {
          rJava::.jcall(connection@jConnection, "V", "setAutoCommit", TRUE)
        }
      }
      result <- DatabaseConnector::querySql(
        connection = connection, sql = sql,
        errorReportFile = errorReportFile,
        snakeCaseToCamelCase = TRUE
      )

      delta <- difftime(Sys.time(), start, units = "secs")
      return(.recordResult(
        result = result, check = check, checkDescription = checkDescription, sql = sql,
        executionTime = sprintf("%f %s", delta, attr(delta, "units"))
      ))
    },
    warning = function(w) {
      ParallelLogger::logWarn(sprintf(
        "[Level: %s] [Check: %s] [CDM Table: %s] [CDM Field: %s] %s",
        checkDescription$checkLevel,
        checkDescription$checkName,
        check["cdmTableName"],
        check["cdmFieldName"], w$message
      ))
      return(.recordResult(check = check, checkDescription = checkDescription, sql = sql, warning = w$message))
    },
    error = function(e) {
      ParallelLogger::logError(sprintf(
        "[Level: %s] [Check: %s] [CDM Table: %s] [CDM Field: %s] %s",
        checkDescription$checkLevel,
        checkDescription$checkName,
        check["cdmTableName"],
        check["cdmFieldName"], e$message
      ))
      return(.recordResult(check = check, checkDescription = checkDescription, sql = sql, error = e$message))
    }
  )
}

# Copyright 2026 Observational Health Data Sciences and Informatics
#
# This file is part of DataQualityDashboard
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' @title Re-evaluate Thresholds
#'
#' @description Re-evaluate an existing DQD result against an updated thresholds file.
#'
#' @param jsonFilePath              Path to the JSON results file generated using the execute function
#' @param outputFolder              The folder to output new JSON result file to
#' @param outputFile                File to write results JSON object to
#' @param tableCheckThresholdLoc    The location of the threshold file for evaluating the table checks. If not specified the default thresholds will be applied.
#' @param fieldCheckThresholdLoc    The location of the threshold file for evaluating the field checks. If not specified the default thresholds will be applied.
#' @param conceptCheckThresholdLoc  The location of the threshold file for evaluating the concept checks. If not specified the default thresholds will be applied.
#' @param cdmVersion                The CDM version to target for the data source. By default, 5.3 is used.
#'
#' @return A list containing the re-evaluated DQD results
#'
#' @export

reEvaluateThresholds <- function(jsonFilePath,
                                 outputFolder,
                                 outputFile,
                                 tableCheckThresholdLoc = "default",
                                 fieldCheckThresholdLoc = "default",
                                 conceptCheckThresholdLoc = "default",
                                 cdmVersion = "5.3") {
  # Read in results to data frame --------------------------------------
  dqdResults <- jsonlite::read_json(path = jsonFilePath)

  df <- lapply(dqdResults$CheckResults, function(cr) {
    cr[sapply(cr, is.null)] <- NA
    as.data.frame(cr)
  })
  df <- do.call(plyr::rbind.fill, df)

  # Add required fields that might be missing due to writing to json
  if (!("cdmFieldName" %in% colnames(df))) {
    df$cdmFieldName <- NA
  }
  if (!("error" %in% colnames(df))) {
    df$error <- NA
  }
  if (!("conceptId" %in% colnames(df))) {
    df$conceptId <- NA
  }
  if (!("unitConceptId" %in% colnames(df))) {
    df$unitConceptId <- NA
  }

  # Read in  new thresholds ----------------------------------------------
  tableChecks <- .readThresholdFile(tableCheckThresholdLoc, defaultLoc = sprintf("OMOP_CDMv%s_Table_Level.csv", cdmVersion))
  fieldChecks <- .readThresholdFile(fieldCheckThresholdLoc, defaultLoc = sprintf("OMOP_CDMv%s_Field_Level.csv", cdmVersion))
  fieldChecks$cdmFieldName <- toupper(fieldChecks$cdmFieldName) # Uppercase in results, lowercase in threshold files
  conceptChecks <- .readThresholdFile(conceptCheckThresholdLoc, defaultLoc = sprintf("OMOP_CDMv%s_Concept_Level.csv", cdmVersion))
  conceptChecks$cdmFieldName <- toupper(conceptChecks$cdmFieldName)

  newCheckResults <- .evaluateThresholds(
    checkResults = df,
    tableChecks = tableChecks,
    fieldChecks = fieldChecks,
    conceptChecks = conceptChecks
  )

  newOverview <- .summarizeResults(checkResults = newCheckResults)

  newDqdResults <- dqdResults
  newDqdResults$CheckResults <- newCheckResults
  newDqdResults$Overview <- newOverview

  .writeResultsToJson(
    result = newDqdResults,
    outputFolder = outputFolder,
    outputFile = outputFile
  )

  return(newDqdResults)
}

# Copyright 2026 Observational Health Data Sciences and Informatics
#
# This file is part of DataQualityDashboard
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Internal function to create queries when running in "incremental insert" sqlOnly mode
#'
#' @param params                    Collection of parameters from .runCheck
#' @param check                     Create SQL for this specific check type
#' @param tablechecks               A dataframe containing the table checks
#' @param fieldChecks               A dataframe containing the field checks
#' @param conceptChecks             A dataframe containing the concept checks
#' @param sql                       The rendered SQL for this check
#' @param connectionDetails         A connectionDetails object for connecting to the CDM database
#' @param checkDescription          The description of the data quality check
#'
#' @return A rendered SQL query to add into the incremental insert sqlOnly query

#' @noRd
#' @keywords internal
#'
.createSqlOnlyQueries <- function(
    params,
    check,
    tableChecks,
    fieldChecks,
    conceptChecks,
    sql,
    connectionDetails,
    checkDescription) {
  resultShell <- .recordResult(check = check, checkDescription = checkDescription, sql = sql)

  resultShell$queryText <- gsub(";", "", resultShell$queryText)
  resultShell$checkDescription <- gsub("\t", " ", gsub("\r", " ", gsub("\n", " ", gsub("'", "''", resultShell$checkDescription))))

  # Retrieve the numeric threshold value for the specific check.
  thresholdValue <- .getThreshold(
    checkName = resultShell$checkName,
    checkLevel = resultShell$checkLevel,
    cdmTableName = resultShell$cdmTableName,
    cdmFieldName = resultShell$cdmFieldName,
    conceptId = resultShell$conceptId,
    unitConceptId = resultShell$unitConceptId,
    tableChecks = tableChecks,
    fieldChecks = fieldChecks,
    conceptChecks = conceptChecks
  )

  # Generate the wrapping query for the desired check. This creates a final row for insertion that includes nearly all the metadata for the query (in addition to calling the check query itself)
  # The only metadata that are not included in this wrapping query include:
  # 1. execution_time -- since this query is not being executed (only the SQL is generated), execution_time is not available
  # 2. queryText -- although this could be included, it seemed redundant since it is part of the generated SQL file
  # 3. warning -- not available since the SQL is not executed
  # 4. error -- not available since the SQL is not executed
  # 5. not_applicable_reason -- this currently requires post-processing
  # 6. notes_value -- although this could be included, it seemed redundant
  checkQuery <- SqlRender::loadRenderTranslateSql(
    sqlFilename = file.path("sqlOnly", "cte_sql_for_results_table.sql"),
    packageName = "DataQualityDashboard",
    dbms = connectionDetails$dbms,
    queryText = resultShell$queryText,
    checkName = resultShell$checkName,
    checkLevel = resultShell$checkLevel,
    renderedCheckDescription = resultShell$checkDescription,
    cdmTableName = resultShell$cdmTableName,
    cdmFieldName = resultShell$cdmFieldName,
    conceptId = resultShell$conceptId,
    unitConceptId = resultShell$unitConceptId,
    sqlFile = checkDescription$sqlFile,
    category = resultShell$category,
    subcategory = resultShell$subcategory,
    context = resultShell$context,
    checkId = resultShell$checkId,
    thresholdValue = thresholdValue
  )

  return(checkQuery)
}


#' Internal function to write queries when running in sqlOnly mode
#'
#' @param sqlToUnion                List of one or more SQL queries to union
#' @param sqlOnlyUnionCount         Value of @sqlOnlyUnionCount - determines max # of sql queries to union in a single cte
#' @param resultsDatabaseSchema     The fully qualified database name of the results schema
#' @param writeTableName            The table tor write DQD results to. Used when sqlOnly or writeToTable is True.
#' @param dbms                      The database type (e.g. spark, sql server) - needed for proper query rendering
#' @param outputFolder              Location to write the generated SQL files
#' @param checkDescription          The description of the data quality check
#'
#' @return NULL (writes SQL queries to file)
#'
#' @noRd
#' @keywords internal
#'
.writeSqlOnlyQueries <- function(
    sqlToUnion,
    sqlOnlyUnionCount,
    resultsDatabaseSchema,
    writeTableName,
    dbms,
    outputFolder,
    checkDescription) {
  outFile <- file.path(
    outputFolder,
    sprintf("%s_%s.sql", checkDescription$checkLevel, checkDescription$checkName)
  )

  # Delete existing file
  unlink(outFile)

  ustart <- 1
  while (ustart <= length(sqlToUnion)) {
    uend <- min(ustart + sqlOnlyUnionCount - 1, length(sqlToUnion))

    sqlUnioned <- paste(sqlToUnion[ustart:uend], collapse = " UNION ALL ")

    # Generate INSERT commands to insert results + metadata into results table
    sql <- SqlRender::loadRenderTranslateSql(
      sqlFilename = file.path("sqlOnly", "insert_ctes_into_result_table.sql"),
      packageName = "DataQualityDashboard",
      dbms = dbms,
      resultsDatabaseSchema = resultsDatabaseSchema,
      tableName = writeTableName,
      queryText = sqlUnioned
    )

    write(
      x = sql,
      file = outFile,
      append = TRUE
    )

    ustart <- ustart + sqlOnlyUnionCount
  }
}


#' Internal function to write the DDL to outputFolder
#'
#' @param resultsDatabaseSchema     The fully qualified database name of the results schema
#' @param writeTableName            The table tor write DQD results to. Used when sqlOnly or writeToTable is True.
#' @param dbms                      The database type (e.g. spark, sql server) - needed for proper query rendering
#' @param outputFolder              Location to write the generated SQL files
#'
#' @return NULL (writes DDL to file)
#'
#' @noRd
#' @keywords internal
.writeDDL <- function(
    resultsDatabaseSchema,
    writeTableName,
    dbms,
    outputFolder) {
  tableName <- sprintf("%s.%s", resultsDatabaseSchema, writeTableName)

  sql <- SqlRender::loadRenderTranslateSql(
    sqlFilename = "result_dataframe_ddl.sql",
    packageName = "DataQualityDashboard",
    dbms = dbms,
    tableName = tableName
  )

  write(
    x = sql,
    file = file.path(
      outputFolder,
      "ddlDqdResults.sql"
    )
  )
}


#' Internal function to get one threshold
#' Note: this does not evaluate is_error or not_applicable status
#'
#' @param checkName                 The name of the check - such as measurePersonCompleteness
#' @param checkLevel                The check level - such as TABLE
#' @param cdmTableName              The name of the CDM table - such as MEASUREMENT
#' @param cdmFieldName              Then name of the CDM field - such as MEASUREMENT_CONCEPT_ID
#' @param conceptId                 The specific concept_id being checked - a valid concept_id number
#' @param unitConceptId             The specific unit concept id being checked - a valid concept_id number
#' @param tableChecks               A dataframe containing the table checks
#' @param fieldChecks               A dataframe containing the field checks
#' @param conceptChecks             A dataframe containing the concept checks
#'
#' @return A numeric value indicating the threshold value
#'
#' @noRd
#' @keywords internal
.getThreshold <- function(
    checkName,
    checkLevel,
    cdmTableName,
    cdmFieldName,
    conceptId,
    unitConceptId,
    tableChecks,
    fieldChecks,
    conceptChecks) {
  thresholdField <- sprintf("%sThreshold", checkName)

  # find if field exists -----------------------------------------------
  thresholdFieldExists <- eval(parse(
    text = sprintf(
      "'%s' %%in%% colnames(%sChecks)",
      thresholdField,
      tolower(checkLevel)
    )
  ))

  if (!thresholdFieldExists) {
    thresholdValue <- NA
  } else {
    if (checkLevel == "TABLE") {
      thresholdFilter <- sprintf(
        "tableChecks$%s[tableChecks$cdmTableName == '%s']",
        thresholdField, cdmTableName
      )
    } else if (checkLevel == "FIELD") {
      thresholdFilter <- sprintf(
        "fieldChecks$%s[fieldChecks$cdmTableName == '%s' &
                                fieldChecks$cdmFieldName == '%s']",
        thresholdField,
        cdmTableName,
        cdmFieldName
      )
    } else if (checkLevel == "CONCEPT") {
      if (is.na(unitConceptId) &&
        grepl(",", conceptId)) {
        thresholdFilter <- sprintf(
          "conceptChecks$%s[conceptChecks$cdmTableName == '%s' &
                                  conceptChecks$cdmFieldName == '%s' &
                                  conceptChecks$conceptId == '%s']",
          thresholdField,
          cdmTableName,
          cdmFieldName,
          conceptId
        )
      } else if (is.na(unitConceptId) &&
        !grepl(",", conceptId)) {
        thresholdFilter <- sprintf(
          "conceptChecks$%s[conceptChecks$cdmTableName == '%s' &
                                  conceptChecks$cdmFieldName == '%s' &
                                  conceptChecks$conceptId == %s]",
          thresholdField,
          cdmTableName,
          cdmFieldName,
          conceptId
        )
      } else {
        thresholdFilter <- sprintf(
          "conceptChecks$%s[conceptChecks$cdmTableName == '%s' &
                                  conceptChecks$cdmFieldName == '%s' &
                                  conceptChecks$conceptId == %s &
                                  conceptChecks$unitConceptId == '%s']",
          thresholdField,
          cdmTableName,
          cdmFieldName,
          conceptId,
          as.integer(unitConceptId)
        )
      }
    }
    thresholdValue <- eval(parse(text = thresholdFilter))
  }

  # Need value of 0 for NA in generated SQL
  if (is.na(thresholdValue)) {
    thresholdValue <- 0
  }

  return(thresholdValue)
}

# Copyright 2026 Observational Health Data Sciences and Informatics
#
# This file is part of DataQualityDashboard
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Internal function to put the results of each quality check into a dataframe.
#'
#' @param result                    The result of the data quality check
#' @param check                     The data quality check
#' @param checkDescription          The description of the data quality check
#' @param sql                       The fully qualified sql for the data quality check
#' @param executionTime             The total time it took to execute the data quality check
#' @param warning                   Any warnings returned from the server
#' @param error                     Any errors returned from the server
#'
#' @return A dataframe containing the check results
#'
#' @keywords internal
#' @importFrom stats setNames
#'


.recordResult <- function(result = NULL,
                          check,
                          checkDescription,
                          sql,
                          executionTime = NA,
                          warning = NA,
                          error = NA) {
  columns <- lapply(names(check), function(c) {
    setNames(check[c], c)
  })

  params <- c(
    list(sql = checkDescription$checkDescription),
    list(warnOnMissingParameters = FALSE),
    lapply(unlist(columns, recursive = FALSE), toupper)
  )

  reportResult <- data.frame(
    numViolatedRows = NA,
    pctViolatedRows = NA,
    numDenominatorRows = NA,
    executionTime = executionTime,
    queryText = sql,
    checkName = checkDescription$checkName,
    checkLevel = checkDescription$checkLevel,
    checkDescription = do.call(SqlRender::render, params),
    cdmTableName = check["cdmTableName"],
    cdmFieldName = check["cdmFieldName"],
    conceptId = check["conceptId"],
    unitConceptId = check["unitConceptId"],
    sqlFile = checkDescription$sqlFile,
    category = checkDescription$kahnCategory,
    subcategory = checkDescription$kahnSubcategory,
    context = checkDescription$kahnContext,
    warning = warning,
    error = error,
    checkId = .getCheckId(checkDescription$checkLevel, checkDescription$checkName, check["cdmTableName"], check["cdmFieldName"], check["conceptId"], check["unitConceptId"]),
    row.names = NULL, stringsAsFactors = FALSE
  )

  if (!is.null(result)) {
    reportResult$numViolatedRows <- result$numViolatedRows
    reportResult$pctViolatedRows <- result$pctViolatedRows
    reportResult$numDenominatorRows <- result$numDenominatorRows
  }
  return(reportResult)
}

# Copyright 2026 Observational Health Data Sciences and Informatics
#
# This file is part of DataQualityDashboard
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Internal function to define the id of each check.
#'
#' @param checkLevel                The level of the check. Options are table, field, or concept
#' @param checkName                 The name of the data quality check
#' @param cdmTableName              The name of the CDM data table the quality check is applied to
#' @param cdmFieldName              The name of the field in the CDM data table the quality check is applied to
#' @param conceptId                 The concept id the quality check is applied to
#' @param unitConceptId             The unit concept id the quality check is applied to
#'
#' @return A character string representing the unique check ID
#'
#' @keywords internal
#' @importFrom stats na.omit
#'

.getCheckId <- function(checkLevel,
                        checkName,
                        cdmTableName,
                        cdmFieldName = NA,
                        conceptId = NA,
                        unitConceptId = NA) {
  tolower(
    paste(
      na.omit(c(
        dplyr::na_if(gsub(" ", "", checkLevel), ""),
        dplyr::na_if(gsub(" ", "", checkName), ""),
        dplyr::na_if(gsub(" ", "", cdmTableName), ""),
        dplyr::na_if(gsub(" ", "", cdmFieldName), ""),
        dplyr::na_if(gsub(" ", "", conceptId), ""),
        dplyr::na_if(gsub(" ", "", unitConceptId), "")
      )),
      collapse = "_"
    )
  )
}

# Only use devtools::load_all() when running tests directly with devtools
# R CMD check loads the package automatically, so we don't need this
# Check if we're in an R CMD check environment by looking at the current directory
isRcmdCheck <- grepl("\\.Rcheck", getwd())

# Only run devtools setup in local development, not in CI/covr
isLocalDev <- !isRcmdCheck &&
  !identical(Sys.getenv("CI"), "true") &&
  !identical(Sys.getenv("COVR"), "true") &&
  requireNamespace("devtools", quietly = TRUE)

if (isLocalDev) {
  devtools::load_all()

  # Create symbolic link for sql directory if it doesn't exist
  # This allows testing with devtools::test
  packageRoot <- normalizePath(system.file("..", package = "DataQualityDashboard"))
  sqlLinkPath <- file.path(packageRoot, "sql")
  sqlPackagePath <- system.file("sql", package = "DataQualityDashboard")

  if (!file.exists(sqlLinkPath) && sqlPackagePath != "") {
    print("setting sql folder symbolic link")
    # Create symbolic link so code can be used in devtools::test()
    tryCatch(
      {
        if (requireNamespace("R.utils", quietly = TRUE)) {
          R.utils::createLink(link = sqlLinkPath, sqlPackagePath)
          options("use.devtools.sql_shim" = TRUE)
        } else {
          # Fallback: create a simple file.copy if R.utils is not available
          if (!dir.exists(sqlLinkPath)) {
            dir.create(sqlLinkPath, recursive = TRUE)
          }
          file.copy(from = sqlPackagePath, to = dirname(sqlLinkPath), recursive = TRUE, overwrite = TRUE)
          options("use.devtools.sql_shim" = TRUE)
        }
      },
      error = function(e) {
        warning("Failed to create symbolic link for SQL directory: ", e$message)
        # Continue without the symbolic link - the package should still work
      }
    )
  }
}

# Ensure the package is properly loaded and SQL files are accessible
if (!requireNamespace("DataQualityDashboard", quietly = TRUE)) {
  # Try to load the package if it's not already loaded
  library(DataQualityDashboard)
}

# Verify SQL files are accessible
sqlFile <- system.file("sql", "sql_server", "field_plausible_after_birth.sql", package = "DataQualityDashboard")
if (sqlFile == "") {
  stop("Cannot find SQL files. Make sure the package is properly loaded.")
}

if (!identical(Sys.getenv("NOT_CRAN"), "true")) {
  jdbcDriverFolder <- ""
} else if (Sys.getenv("DONT_DOWNLOAD_JDBC_DRIVERS", "") == "TRUE") {
  jdbcDriverFolder <- Sys.getenv("DATABASECONNECTOR_JAR_FOLDER")
} else {
  jdbcDriverFolder <- tempfile("jdbcDrivers")
  dir.create(jdbcDriverFolder)
  DatabaseConnector::downloadJdbcDrivers("postgresql", jdbcDriverFolder)
  DatabaseConnector::downloadJdbcDrivers("sql server", jdbcDriverFolder)
  DatabaseConnector::downloadJdbcDrivers("oracle", jdbcDriverFolder)
  DatabaseConnector::downloadJdbcDrivers("redshift", jdbcDriverFolder)
}

if (requireNamespace("Eunomia", quietly = TRUE)) {
  connectionDetailsEunomia <- Eunomia::getEunomiaConnectionDetails()
  cdmDatabaseSchemaEunomia <- "main"
  resultsDatabaseSchemaEunomia <- "main"

  # Separate connection details for NA tests, as this requires removing records
  connectionDetailsEunomiaNaChecks <- Eunomia::getEunomiaConnectionDetails()

  # Separate connection details for plausibleAfterBirth test
  connectionDetailsPlausibleAfterBirth <- Eunomia::getEunomiaConnectionDetails()

  # Separate connection details for observation period overlap test
  connectionDetailsEunomiaOverlap <- Eunomia::getEunomiaConnectionDetails()
}

# Helper function to verify database connection
verifyConnection <- function(connectionDetails) {
  tryCatch(
    {
      connection <- DatabaseConnector::connect(connectionDetails = connectionDetails)
      on.exit(DatabaseConnector::disconnect(connection), add = TRUE)
      # Test the connection with a simple query that works across all databases
      # For Oracle, we need FROM DUAL, which SqlRender handles when we use a proper template
      if (tolower(connectionDetails$dbms) == "oracle") {
        testSql <- "SELECT 1 FROM DUAL"
      } else {
        testSql <- "SELECT 1"
      }
      testSql <- SqlRender::translate(sql = testSql, targetDialect = connectionDetails$dbms)
      DatabaseConnector::querySql(connection = connection, sql = testSql)
      return(TRUE)
    },
    error = function(e) {
      stop(e$message)
    }
  )
}

# Copyright 2026 Observational Health Data Sciences and Informatics
#
# This file is part of DataQualityDashboard
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


#' @title List DQ checks
#'
#' @description Details on all checks defined by the DataQualityDashboard Package.
#'
#' @param cdmVersion                The CDM version to target for the data source. By default, 5.3 is used.
#' @param tableCheckThresholdLoc    The location of the threshold file for evaluating the table checks. If not specified the default thresholds will be applied.
#' @param fieldCheckThresholdLoc    The location of the threshold file for evaluating the field checks. If not specified the default thresholds will be applied.
#' @param conceptCheckThresholdLoc  The location of the threshold file for evaluating the concept checks. If not specified the default thresholds will be applied.
#'
#' @return A list containing check descriptions, table checks, field checks, and concept checks
#'
#' @importFrom readr read_csv local_edition
#'
#' @export
listDqChecks <- function(cdmVersion = "5.3", tableCheckThresholdLoc = "default", fieldCheckThresholdLoc = "default", conceptCheckThresholdLoc = "default") {
  # temporary patch to work around vroom 1.6.4 bug
  readr::local_edition(1)

  dqChecks <- {}
  dqChecks$checkDescriptions <-
    read_csv(system.file(
      "csv",
      sprintf("OMOP_CDMv%s_Check_Descriptions.csv", cdmVersion),
      package = "DataQualityDashboard"
    ))

  dqChecks$tableChecks <- .readThresholdFile(
    checkThresholdLoc = tableCheckThresholdLoc,
    defaultLoc = sprintf("OMOP_CDMv%s_Table_Level.csv", cdmVersion)
  )

  dqChecks$fieldChecks <- .readThresholdFile(
    checkThresholdLoc = fieldCheckThresholdLoc,
    defaultLoc = sprintf("OMOP_CDMv%s_Field_Level.csv", cdmVersion)
  )

  dqChecks$conceptChecks <- .readThresholdFile(
    checkThresholdLoc = conceptCheckThresholdLoc,
    defaultLoc = sprintf("OMOP_CDMv%s_Concept_Level.csv", cdmVersion)
  )

  return(dqChecks)
}

# Copyright 2026 Observational Health Data Sciences and Informatics
#
# This file is part of DataQualityDashboard
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' @title Convert JSON results file case
#'
#' @description Convert a DQD JSON results file between camelcase and (all-caps) snakecase. Enables viewing of pre-v.2.1.0 results files in later DQD versions, and vice versa
#'
#' @param jsonFilePath  Path to the JSON results file to be converted
#' @param writeToFile   Whether or not to write the converted results back to a file (must be either TRUE or FALSE)
#' @param outputFolder  The folder to output the converted JSON results file to
#' @param outputFile    (OPTIONAL) File to write converted results JSON object to. Default is name of input file with a "_camel" or "_snake" postfix
#' @param targetCase    Case into which the results file parameters should be converted (must be either "camel" or "snake")
#'
#' @returns DQD results object (a named list)
#'
#' @importFrom jsonlite fromJSON
#' @importFrom SqlRender snakeCaseToCamelCase camelCaseToSnakeCase
#' @importFrom dplyr rename_with
#' @importFrom tools file_path_sans_ext
#' @importFrom readr local_edition
#'
#' @export

convertJsonResultsFileCase <- function(
    jsonFilePath,
    writeToFile,
    outputFolder = NA,
    outputFile = "",
    targetCase) {
  if (!any(targetCase %in% c("camel", "snake"))) {
    stop("targetCase must be either 'camel' or 'snake'.")
  }
  stopifnot(is.logical(writeToFile))
  if (writeToFile && is.na(outputFolder)) {
    stop("You must specify an output folder if writing to file.")
  }

  # temporary patch to work around vroom 1.6.4 bug
  readr::local_edition(1)

  results <- jsonlite::fromJSON(jsonFilePath)

  if ("numViolatedRows" %in% names(results$CheckResults) && targetCase == "camel") {
    warning("File is already in camelcase! No conversion will be performed.")
    return(results)
  }
  if ("NUM_VIOLATED_ROWS" %in% names(results$CheckResults) && targetCase == "snake") {
    warning("File is already in snakecase! No conversion will be performed.")
    return(results)
  }

  if (targetCase == "camel") {
    swapFunction <- SqlRender::snakeCaseToCamelCase
  } else {
    swapFunction <- function(x) {
      toupper(SqlRender::camelCaseToSnakeCase(x))
    }
  }

  results$Metadata <- dplyr::rename_with(results$Metadata, swapFunction)
  results$CheckResults <- dplyr::rename_with(results$CheckResults, swapFunction, -c("checkId"))

  if (writeToFile) {
    if (nchar(outputFile) == 0) {
      jsonFile <- tools::file_path_sans_ext(basename(jsonFilePath))
      outputFile <- paste(jsonFile, "_", targetCase, ".json", sep = "")
    }
    .writeResultsToJson(results, outputFolder, outputFile)
  }

  return(results)
}

---
title: "Add a New Data Quality Check"
author: "Don Torok"
date: "`r Sys.Date()`"
output:
  pdf_document:
    number_sections: true
    toc: true
  html_document:
    number_sections: true
    toc: true
vignette: >
  %\VignetteIndexEntry{Add a New Data Quality Check}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::knitr}
---

# Add a New Check to Data Quality Dashboard
## Steps
1. Write the SQL query for your check.
1. Format the Query for the Data Quality Dashboard
1. Add the Query to Check Descriptions File
1. Add Hooks to Field, Table or Concept Check CSV File


The following will show, by example, how to add a check that emergency visits in the CDM are less than or equal to two days.
## Write the SQL Query for Your Check
The query should return the number of rows that fail the check, i.e. the duration of the ER visit is greater than two days.

		SELECT count(*)
		FROM visit_occurrence
		WHERE visit_concept_id = 9203 /* ER visit */
		  AND dateDiff(days, visit_start_date, visit_end_date) > 2;

## Format the Query for the Data Quality Dashboard
In the DataQualityDashboard git directory DataQualityDashboard/inst/sql/sql_server are the SQL files that implement the various data quality checks. The files have the following input parameters:
@cdmDatabaseSchema
@vocabDatabaseSchema
@cdmTableName
@cdmFieldName

And the result of the query is expected to have the following fields:
num_violated_rows; pct_violated_rows; num_denominator_rows

Its best to find an existing query similar to the test you want to implement and use that as a starting point for your test query. 
 

		SELECT num_violated_rows
		    , CASE WHEN denominator.num_rows = 0 
		           THEN 0 
		           ELSE 1.0*dqd_check.num_violated_rows/denominator.num_rows 
		     END  AS pct_violated_rows, 
		  denominator.num_rows as num_denominator_rows
		FROM
		(
			<Your query that returns num_violated_rows>
			 
		) dqd_check
		CROSS JOIN
		( 
			SELECT COUNT_BIG(*) AS num_rows
			FROM @cdmDatabaseSchema.@cdmTableName cdmTable
		) denominator;

The initial query should then be set up to use the input parameters.  This test is only valid for Visit Occurrence  and the columns visit_start_date, visit_end_date, and visit_concept_id are fixed. As a result we only need to add the @cdmDatabaseSchema and @cdmTableName parameters and alias the count(*) field to **num_violated_rows**

		SELECT count(*) AS num_violated_rows
		FROM @cdmDatabaseSchema.@cdmTableName
		WHERE visit_concept_id = 9203 /* ER visit */
		 AND dateDiff(days, visit_start_date, visit_end_date) > 2

Paste the check for violating rows into the template from a similar query and test the complete query on a OMOP CDM. A good way to confirm that the input parameters are correct is to use launchSqlRenderDeveloper from the SQLRender package. The package is included in the Data Quality R project.

![Figure 1: launchSqlRenderDeveloper](./images/library_SQLRender.png)

This will bring up a SqlRender window in your browser. Paste in your complete DQ query. The program will see the input parameters and allow you to define them. The rendered translation, dialect specific, is output. You can run the "Rendered Translation" on your OMOP CDM.

![Figure 2: launchSqlRenderDeveloper](./images/SQLRenderDeveloper.png)

SQL Server SQL is the lingua franca for Data Quality Dashboard queries. SQLRender will modify the query to other SQL dialects. See [SQLRender on CRAN](https://CRAN.R-project.org/package=SqlRender) for additional information. When satisfied with the results of the query save the SQL in a file named ERVisitLength.sql in the directory DataQualityDashboard/inst/sql/sql_server.

## Add the Query to Check Descriptions File
The Check Description file is found in the DataQualityDashboard/inst/csv directory. There is a Check Description for each CDM release.  Within the Check Description file are the following columns:
- checkLevel -- Possible values are TABLE, FIELD or CONCEPT. This is the csv file that has the information telling the DQD code on what tables and fields to run the check. For our example, check of the length of an ER stay, this will be the csv file with field level checks. 
- checkName -- Name used in the DQD code to identify the check. For this example use **ERVisitLength**
- checkDescription -- Short user provided description of what the check does. This description is displayed in the DQD results. 
- kahnContext -- The following 3 columns are a means of organizing quality checks. For additional information see [Harmonized Data Quality Assessment Terminology and Framework](https://pmc.ncbi.nlm.nih.gov/articles/PMC5051581/). For this example use 'Verification'
- kahnCategory -- Use 'Plausibility'
- kahnSubcategory -- Use 'Temporal'
- sqlFile -- The name of the SQL file we created above. The name should include the values for checkLevel and checkName.  For example, field_ERVisitLength.sql
- evaluationFilter -- This is a column name in the Field Check csv file and a conditional statement, that when TRUE will result in the DQD code running the check.  This will become clearer after describing how to update the Field Checks csv file in the next section. For this example use **ERVisitLength=='Yes'**. This means run our test when the row value in the column headed ERVisitLength equals Yes.

## Add Hooks to Field/Table/Concept Check CSV File
The field level check file, OMOP_CDMv5.4_Field_Level.csv, is in the same directory as the Check Descriptions file.  The first two columns are 'cdmTableName' and 'cdmFieldName'. There is a row for each table/column in the OMOP CDM. After the first two columns are triplets of columns with the naming scheme **testName**, **testNameThreshold**, **testNameNotes**. We need to add these columns for our ER visit length test. The testName is the name used in defining the evaluationFilter column in the above step, **ERVisitLength**. The other two column names are ERVisitLengthThreshold and ERVisitLengthNotes. The evaluation filter we added, ERVisitLengh=='Yes' means that for every table/column in this file where the row value in the column ERVisitLength is 'Yes', run the SQL in the file we created, ERVisitLength.sql, passing in the schema, table and column names. (The schema name is defined when running the DQD code.) For this example the only row with a Yes will be for the visit_date_date in the visit_occurrence table.

The testNameThreshold column should have a value between 0 and 100 for all rows where there is a Yes in the testName column. Remember the SQL in ERVisitLength.sql returns the pct_violated_rows. If pct_violated_row is greater than the threshold the test is recorded as failed.  If our threshold is 0 then any ER visit with a length greater than two days will return a pct_violated_row greater than zero and the test will fail. If you want to allow for a few 'bad' visit lengths you can set the threshold to a low number. Or if you want the test to be run, but never flag an error, set the threshold to 100.

The testNameNotes column is usually left blank. The convention is that this column will be used by the person running the tests to document circumstances in the source data that might cause the test to fail.

Add the above three columns to the end of the csv file. In the row for Visit Occurrence/Visit start date put 'Yes' in the ERVisitLength column and 0 in the ERVisitLengthThreshold column. Our test will be run once whenever field level checks are run. If any ER visit length is greater than two days the check will be marked as failed.

You can run just the ERVisitLength test by setting the input parameter checkNames = c("ERVisitLength") in the R program DataQualityDashboard::executeDqChecks.

---
title: "Failure Thresholds and How to Change Them"
author: "Clair Blacketer"
date: "`r Sys.Date()`"
output:
  pdf_document:
    number_sections: true
    toc: true
  html_document:
    number_sections: true
    toc: true
vignette: >
  %\VignetteIndexEntry{Failure Thresholds and How to Change Them}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::knitr}
---

# DQD Failure Thresholds

As described in the [introduction to the tool](https://ohdsi.github.io/DataQualityDashboard/#introduction), the Data Quality Dashboard works by systematically applying 20 data quality check types to a database by leveraging the structure of the OMOP Common Data Model. This process results in over 3,300 potential data quality checks. These checks are then resolved against the database, each one producing a number and percent of records that fail. The percent failure is then compared against a number to determine if the check itself should be marked as a pass or a fail overall. Essentially, the threshold number is the percent of failed records for a particular check you are willing to accept. If the percent failures is lower than the threshold then it passes, otherwise the check fails.

A default set of failure thresholds are shipped with the package. Many of these are general thresholds determined by CDM experts and were designed to test the minimum quality measures necessary to run an analysis on a database. They do not take into account an apriori knowledge of the database or data that is available. With that in mind, it is possible to change the failure thresholds and even set different thresholds for different databases.

## DQD Control Files

There is a set of three csv files that underlie the DQD. These files indicate which checks should be run and what their failure thresholds should be. There is one file per check level: TABLE, FIELD, and CONCEPT. This vignette will walk through how to update the field level check thresholds but the process is the same for all three files. 

### Step 1: Find and copy the control files

The control files are located in the R package in the inst/csv folder and can also be downloaded [here](https://github.com/OHDSI/DataQualityDashboard/tree/master/inst/csv). In that location there should be four files per CDM version the DQD supports. These are:

- **OMOP_CDMvx.x.x_Check_Descriptions.csv**: This file contains the check types, their descriptions, and which sql file is associated with each. It does not contain any failure thresholds and should not need to be changed. 
- **OMOP_CDMvx.x.x_Table_Level.csv**: This file has a list of all tables in the CDM version and any checks to be performed at this level as well as the failure thresholds associated. Right now only the checktype [measurePersonCompleteness](https://ohdsi.github.io/DataQualityDashboard/articles/CheckTypeDescriptions.html#measurepersoncompleteness) is considered a table level check. If you would like to change the failure thresholds for the measurePersonCompleteness check copy this file to a location the R instance can access.
- **OMOP_CDMvx.x.x_Field_Level.csv**: This file has a list of all tables and fields in the CDM version and any checks to be performed at this level as well as the failure thresholds associated. The majority of the [check types](https://ohdsi.github.io/DataQualityDashboard/articles/CheckTypeDescriptions.html) run by the DQD are field level checks. If you would like to change the failure thresholds for any of these checks copy this file to a location the R instance can access. An example of this file is seen in figure 1 below.

![Figure 1: Field Level Control File Example](images/fieldLevelFile.png)

- **OMOP_CDMvx.x.x_Concept_Level.csv**: This file has a list of all concepts and any checks to be performed on that concept. For example, there are checks that evaluate biologically plausible ranges for measurement values, like [plausibleValueLow](https://ohdsi.github.io/DataQualityDashboard/articles/CheckTypeDescriptions.html#plausiblevaluelow-1). If you would like to change the failure thresholds for any of these checks copy this file to a location the R instance can access.

### Step 2: Turn On/Off Checks and Change Thresholds

Using the field level file as an example, when you open it there will be two columns listing the tables and fields in the OMOP CDM, as seen in figure 1. Moving horizontally through the file there will be a column named for each of the [check types](https://ohdsi.github.io/DataQualityDashboard/articles/CheckTypeDescriptions.html) available to run at the field level. At the intersection of each check type and CDM field will be a cell with either a 'Yes' or 'No' value. A 'Yes' indicates that the check type will be run for that specific table and field. For example in figure 1 there is a 'Yes' in the **isRequired** column for the field **person_id** in the **PERSON** table. This means that the [**isRequired**](https://ohdsi.github.io/DataQualityDashboard/articles/CheckTypeDescriptions.html#isRequired) check will be run, substituting **person_id** for cdmFieldName and **PERSON** for cdmTableName.

So instead of the generic check type: *The number and percent of records with a NULL value in the **cdmFieldName** of the **cdmTableName** that is considered not nullable*

The 'Yes' in cell C2 in figure 1 will resolve that check type to an individual data quality check: *The number and percent of records with a NULL value in the **person_id** of the **PERSON** table that is considered not nullable*

Using this method it is possible to turn checks on or off by changing the 'Yes'/'No' values in the control files. This will affect individual checks only. If you would like to filter checks based on check level (TABLE, FIELD, CONCEPT), check type (turn off all fkClass checks, for example), or CDM table (turn off all checks for the COST table), this can be done at the execution step and does not need to be specified in the control file. The vignette **Advanced DQD Execution Options** details how to do this.

For each check type column in the control file there will be one or more additional columns based on the inputs the check type needs to function. For instance the isForeignKey check has the fields fkTableName and fkFieldName so that the check knows where to find the primary key associated with the foreign key. Each check type will also have a column named using the convention **checktype**Threshold. This is the column that lists the threshold against which the check will be evaluated to determine a pass or fail. Similarly to how the 'Yes'/'No' indicators work, the intersection of each CDM field and check type threshold field will hold a value that will be compared to the outcome of the check to determine if the check passes or fails. In figure 1 for the check that is turned on in cell C2 the threshold is listed in cell D2. In this case the value is 0 indicating that if any records in the database violate the rule then this check will fail. 

To change the threshold for a check, input a value between 0 and 100. The threshold value is based on the percent of records that violate a check and indicates the percent of violating records you are willing to tolerate. A 0 thresholds means that any violating records will cause the check to fail and a 100 means that all records evaluated could violate the check and the check will still pass. After changing any thresholds in the control files, save the files in a location that the R instance can access. 

### Step 2a: Documenting Metadata on Updated Thresholds

In addition to thresholds, each check type will also have a column named using the convention **checktype**Notes. These columns allow entry of any information about why a particular check threshold was changed for a certain database, or perhaps record a link to an issue tracking system to denote that a failure has been identified and a fix is in the works. If any notes are recorded these will be exposed in the shiny application. 

### Step 3: Run the DQD Using the Updated Thresholds

Follow the instructions on the [Getting Started](https://ohdsi.github.io/DataQualityDashboard/articles/DataQualityDashboard.html) page to set the parameters to run the Data Quality Dashboard. When running the execute function, point the package to the updated threshold files as shown in the example below. To do this, The options **tableCheckThresholdLoc**, **fieldCheckThresholdLoc**, **conceptCheckThresholdLoc** should contain the fully qualified location of the files. If you would like to revert to the default thresholds these options can be removed from the execute function call or be set to "default".

```r
DataQualityDashboard::executeDqChecks(connectionDetails = connectionDetails, 
                                      cdmDatabaseSchema = cdmDatabaseSchema, 
                                      resultsDatabaseSchema = resultsDatabaseSchema,
                                      cdmSourceName = cdmSourceName, 
                                      numThreads = numThreads,
                                      sqlOnly = sqlOnly, 
                                      outputFolder = outputFolder, 
                                      verboseMode = verboseMode,
                                      writeToTable = writeToTable,
                                      checkLevels = checkLevels,
                                      tablesToExclude = tablesToExclude,
                                      checkNames = checkNames,
                                      tableCheckThresholdLoc = location of the table check file,
                                      fieldCheckThresholdLoc = location of the field check file,
                                      conceptCheckThresholdLoc = location of the concept check file)

```

---
title: "Check Status Definitions"
author: "Dmitry Ilyn, Maxim Moinat"
date: "`r Sys.Date()`"
output:
    pdf_document:
        number_sections: true
        toc: true
    html_document:
        number_sections: true
        toc: true
vignette: >
    %\VignetteIndexEntry{Check Status Definitions}
    %\VignetteEncoding{UTF-8}
    %\VignetteEngine{knitr::knitr}
---

## Introduction
In the DataQualityDashboard v2, new check statuses were introduced: `Error` and `Not Applicable`. These were introduced to more accurately reflect the quality of data contained in a CDM instance, addressing scenarios where pass/fail is not appropriate. The new set of mutually exclusive status states are listed below in priority order:

- **Is Error:** if a SQL error occurred during execution

- **Not Applicable:** if DQ check is not applicable for reasons explained in the section below

- **Failed:** if percent violating rows is greater than the threshold

- **Passed:** if percent violating rows is smaller than the threshold


## Not Applicable

The results of a DQ check may not be applicable to a given CDM instance depending on the implementation and content of the instance. For example, the DQ check for plausible values of HbA1c lab results would pass with no violations even if there were no results for that lab test in the database. It is not uncommon to have \> 1000 DQ checks that do not apply to a given CDM instance. The results from DQ checks that are not applicable skew to overall results. Listed below are the scenarios for which a DQ check result is flagged as notApplicable:

1.  If the cdmTable DQ check determines that a table does not exist in the database, then all DQ checks (except cdm_table) addressing that table are flagged as notApplicable

2.  If a table exists but is empty, then all field level and concept level checks for that table are flagged as notApplicable, except for cdmField checks, which evaluates if the field is defined or not. A cdmField check is marked as notApplicable if the CDM table it refers to does not exist (tested by cdmTable). An empty table is detected when the measureValueCompleteness DQ check for any of the fields in the table returns a denominator count = 0 (numDenominatorRows=0).

3.  If a field is not populated, then all field level and concept level checks except for measureValueCompleteness and isRequired are flagged as notApplicable

    a. A field is not populated if the measureValueCompleteness DQ check finds denominator count \> 0 and number of violated rows = denominator count (numDenominatorRows \> 0 AND numDenominatorRows = numViolatedRows).

    b. The measureValueCompleteness check is marked as not applicable if:

        a. The CDM table it refers to does not exist or is empty.

        b. The CDM field it refers to does not exist.

    c. The isRequired check is marked as not applicable if:

        a. The CDM table it refers to does not exist or is empty.

        b. The CDM field it refers to does not exist.

4.  Flagging a Concept_ID level DQ check as notApplicable depends on whether the DQ check logic includes a UNIT_CONCEPT_ID. There are two scenarios for DQ checks evaluating specific Concept_ids.

    a. The DQ check does not include a UNIT_CONCEPT_ID (value is null). A DQ check is flagged as notApplicable if there are no instances of the Concept_ID in the table/field. E.g. plausibility checks for specific conditions and gender. Both pregnancy and male do not have UNIT_CONCEPT_IDs.

    b. The DQ check includes a UNIT_CONCEPT_ID. A DQ check is flagged as notApplicable if there are no instances of both concept and unit concept IDs in the table/field. E.g. all DQ checks referencing the concept_ID for HbA1c lab results expressed in mg/dl units will be flagged as notApplicable if there are no instances of that concept_ID in the table/field addressed by the DQ check.

---
title: "Running the DQD in SqlOnly mode"
author: "Maxim Moinat"
date: "`r Sys.Date()`"
output:
  pdf_document:
    number_sections: true
    toc: true
  html_document:
    number_sections: true
    toc: true
vignette: >
  %\VignetteIndexEntry{Running the DQD in SqlOnly mode}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::knitr}
---

# Description

This article describes how to use DQD to generate only the SQL that executes all DataQualityDashboard checks, without actually executing them.
There are a few main advantages of running DQD in Sql-only mode:

* Create queries locally, before sending to server. This allows for generation of the SQL on one machine and execution on another (e.g. when R cannot connect directly to the database server, or you want to run the DQD SQL as part of your ETL).
* Since these are fully functional queries, this can help with debugging.
* **[NEW in v2.3.0!]** Performance. If you use `sqlOnlyIncrementalInsert = TRUE` and `sqlOnlyUnionCount > 1`, multiple checks are unioned within a cte in the output SQL query to speed performance. When testing on Spark, this resulted in a 10x or higher performance gain.
  - Performance for these queries has NOT been benchmarked on all database systems. In order to obtain optimal results in your database you may need to adjust the `sqlOnlyUnionCount` and/or tune database parameters such as indexing and parallelism

The new `sqlOnlyIncrementalInsert` mode generates SQL queries that will actually populate a DQD results table in your database with the results of the checks.  There are currently some differences in the result when running these queries, compared to a normal DQD run:

* If you set `sqlOnlyUnionCount` > 1, if one check results in an error, multiple checks might fail (since the queries are unioned in ctes).
* The status `not_applicable` is not evaluated. A check fails or passes.
* The query text is not shown in the results table.
* Notes from threshold file are not included in results.
* Execution metadata is not automatically added (total and query execution time; CDM_SOURCE metadata).

Running DQD with `sqlOnly = TRUE` and `sqlOnlyIncrementalInsert = FALSE` will generate SQL queries that can be run to generate the result of each DQ check, but which will not write the results back to the database.

# Generating the "Incremental Insert" DQD SQL
A few things to note:

* A dummy `connectionDetails` object is needed where only the `dbms` is used during SQL-only execution.
  - By setting the dbms to 'sql server' the output SQL can still be rendered to any other dialect using `SqlRender` (see example below). 
* `sqlOnlyUnionCount` determines the number of check sqls to union in a single query. A smaller number gives more control and progress information, a higher number typically gives a higher performance. Here, 100 is used.

```R
library(DataQualityDashboard)

# ConnectionDetails object needed for sql dialect
dbmsConnectionDetails <- DatabaseConnector::createConnectionDetails(
  dbms = "sql server",  # can be rendered to any dbms upon execution
  pathToDriver = "/"
)

# Database parameters that are pre-filled in the written queries
# Use @-syntax if creating a template-sql at execution-time (e.g. "@cdmDatabaseSchema")
cdmDatabaseSchema <- "@cdmDatabaseSchema"   # the fully qualified database schema name of the CDM
resultsDatabaseSchema <- "@resultsDatabaseSchema"   # the fully qualified database schema name of the results schema (that you can write to)
writeTableName <- "@writeTableName"

sqlFolder <- "./results_sql_only"
cdmSourceName <- "Synthea"

sqlOnly <- TRUE
sqlOnlyIncrementalInsert <- TRUE    # this will generate an insert SQL query for each check type that will compute check results and insert them into a database table
sqlOnlyUnionCount <- 100            # this unions up to 100 queries in each insert query

verboseMode <- TRUE

cdmVersion <- "5.4"
checkLevels <- c("TABLE", "FIELD", "CONCEPT")
tablesToExclude <- c()
checkNames <- c()

# Run DQD with sqlOnly=TRUE and sqlOnlyIncrementalInsert=TRUE. This will create a sql file for each check type in the output folder
DataQualityDashboard::executeDqChecks(
  connectionDetails = dbmsConnectionDetails,
  cdmDatabaseSchema = cdmDatabaseSchema,
  resultsDatabaseSchema = resultsDatabaseSchema,
  writeTableName = writeTableName,
  cdmSourceName = cdmSourceName,
  sqlOnly = sqlOnly,
  sqlOnlyUnionCount = sqlOnlyUnionCount,
  sqlOnlyIncrementalInsert = sqlOnlyIncrementalInsert,
  outputFolder = sqlFolder,
  checkLevels = checkLevels,
  verboseMode = verboseMode,
  cdmVersion = cdmVersion,
  tablesToExclude = tablesToExclude,
  checkNames = checkNames
)
```

After running above code, you will end up with a number of sql files in the specified output directory:

* One sql file per check type: `TABLE|FIELD|CONCEPT_<check_name>.sql`.
* `ddlDqdResults.sql` with the result table creation query.

The queries can then be run in any SQL client, making sure to run `ddlDqdResults.sql` first.
The order of the check queries is not important, and can even be run in parallel.
This will run the check, and store the result in the specified `writeTableName`.
In order to show this result in the DQD Dashboard Shiny app, this table has to be exported and converted to the .json format.
See below for example code of how this can be achieved.

# (OPTIONAL) Execute queries
Below code snippet shows how you can run the generated queries on an OMOP CDM database using OHDSI R packages, and display the results in the DQD Dashboard.
Note that this approach uses two non-exported DQD functions (`.summarizeResults`, `.writeResultsToJson`) that are not tested for this purpose. In the future we plan to expand support for incremental-insert mode with a more robust set of public functions.  Please reach out with feedback on our [GitHub page](https://github.com/OHDSI/DataQualityDashboard/issues) if you'd like to have input on the development of this new feature!

```R
library(DatabaseConnector)
cdmSourceName <- "<YourSourceName>"
sqlFolder <- "./results_sql_only"
jsonOutputFolder <- sqlFolder
jsonOutputFile <- "sql_only_results.json"

dbms <- Sys.getenv("DBMS")
server <- Sys.getenv("DB_SERVER")
port <- Sys.getenv("DB_PORT")
user <- Sys.getenv("DB_USER")
password <- Sys.getenv("DB_PASSWORD")
pathToDriver <- Sys.getenv("PATH_TO_DRIVER")
connectionDetails <- DatabaseConnector::createConnectionDetails(
  dbms = dbms,
  server = server,
  port = port,
  user = user,
  password = password,
  pathToDriver = pathToDriver
)
cdmDatabaseSchema <- '<YourCdmSchemaName>'
resultsDatabaseSchema <- '<YourResultsSchemaName>'
writeTableName <- 'dqd_results' # or whatever you want to name your results table

c <- DatabaseConnector::connect(connectionDetails)

# Create results table
ddlFile <- file.path(sqlFolder, "ddlDqdResults.sql")
DatabaseConnector::renderTranslateExecuteSql(
  connection = c,
  sql = readChar(ddlFile, file.info(ddlFile)$size),
  resultsDatabaseSchema = resultsDatabaseSchema,
  writeTableName = writeTableName
)

# Run checks
dqdSqlFiles <- Sys.glob(file.path(sqlFolder, "*.sql"))
for (dqdSqlFile in dqdSqlFiles) {
  if (dqdSqlFile == ddlFile) {
    next
  }
  print(dqdSqlFile)
  tryCatch(
    expr = {
      DatabaseConnector::renderTranslateExecuteSql(
        connection = c,
        sql = readChar(dqdSqlFile, file.info(dqdSqlFile)$size),
        cdmDatabaseSchema = cdmDatabaseSchema,
        resultsDatabaseSchema = resultsDatabaseSchema,
        writeTableName = writeTableName
      )
    },
    error = function(e) {
     print(sprintf("Writing table failed for check %s with error %s", dqdSqlFile, e$message))
    }
  )
}

# Extract results table to JSON file for viewing or secondary use

DataQualityDashboard::writeDBResultsToJson(
    c,
    connectionDetails,
    resultsDatabaseSchema,
    cdmDatabaseSchema,
    writeTableName,
    jsonOutputFolder,
    jsonOutputFile
  )


jsonFilePath <- R.utils::getAbsolutePath(file.path(jsonOutputFolder, jsonOutputFile))
DataQualityDashboard::viewDqDashboard(jsonFilePath)
```

---
title: "Getting Started"
author: "Clair Blacketer"
date: "`r Sys.Date()`"
output:
  pdf_document:
    number_sections: true
    toc: true
  html_document:
    number_sections: true
    toc: true
vignette: >
  %\VignetteIndexEntry{Getting Started}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::knitr}
---

R Installation
===============

Install from CRAN:

```r
install.packages("DataQualityDashboard")
```

To install from GitHub:

```r
install.packages("remotes")
remotes::install_github("OHDSI/DataQualityDashboard")
```

Note
=====
To view the JSON results in the shiny application the package requires that the CDM_SOURCE table has at least one row with some details about the database. This is to ensure that some metadata is delivered along with the JSON, should it be shared. As a best practice it is recommended to always fill in this table during ETL or at least prior to running the DQD. 


Executing Data Quality Checks
==============================
  ```r

# fill out the connection details -----------------------------------------------------------------------
connectionDetails <- DatabaseConnector::createConnectionDetails(
  dbms = "", 
  user = "", 
  password = "", 
  server = "", 
  port = "", 
  extraSettings = "",
  pathToDriver = ""
)

cdmDatabaseSchema <- "yourCdmSchema" # the fully qualified database schema name of the CDM
resultsDatabaseSchema <- "yourResultsSchema" # the fully qualified database schema name of the results schema (that you can write to)
cdmSourceName <- "Your CDM Source" # a human readable name for your CDM source
cdmVersion <- "5.4" # the CDM version you are targetting. Currently supports 5.2, 5.3, and 5.4

# determine how many threads (concurrent SQL sessions) to use ----------------------------------------
numThreads <- 1 # on Redshift, 3 seems to work well

# specify if you want to execute the queries or inspect them ------------------------------------------
sqlOnly <- FALSE # set to TRUE if you just want to get the SQL scripts and not actually run the queries
sqlOnlyIncrementalInsert <- FALSE # set to TRUE if you want the generated SQL queries to calculate DQD results and insert them into a database table (@resultsDatabaseSchema.@writeTableName)
sqlOnlyUnionCount <- 1  # in sqlOnlyIncrementalInsert mode, the number of check sqls to union in a single query; higher numbers can improve performance in some DBMS (e.g. a value of 25 may be 25x faster)

# NOTES specific to sqlOnly <- TRUE option ------------------------------------------------------------
# 1. You do not need a live database connection.  Instead, connectionDetails only needs these parameters:
#      connectionDetails <- DatabaseConnector::createConnectionDetails(
#        dbms = "", # specify your dbms
#        pathToDriver = "/"
#      )
# 2. Since these are fully functional queries, this can help with debugging.
# 3. In the results output by the sqlOnlyIncrementalInsert queries, placeholders are populated for execution_time, query_text, and warnings/errors; and the NOT_APPLICABLE rules are not applied.
# 4. In order to use the generated SQL to insert metadata and check results into output table, you must set sqlOnlyIncrementalInsert = TRUE.  Otherwise sqlOnly is backwards compatable with <= v2.2.0, generating queries which run the checks but don't store the results.


# where should the results and logs go? ----------------------------------------------------------------
outputFolder <- "output"
outputFile <- "results.json"


# logging type -------------------------------------------------------------------------------------
verboseMode <- TRUE # set to FALSE if you don't want the logs to be printed to the console

# write results to table? ------------------------------------------------------------------------------
writeToTable <- TRUE # set to FALSE if you want to skip writing to a SQL table in the results schema

# specify the name of the results table (used when writeToTable = TRUE and when sqlOnlyIncrementalInsert = TRUE)
writeTableName <- "dqdashboard_results"

# write results to a csv file? -----------------------------------------------------------------------
writeToCsv <- FALSE # set to FALSE if you want to skip writing to csv file
csvFile <- "" # only needed if writeToCsv is set to TRUE

# if writing to table and using Redshift, bulk loading can be initialized -------------------------------

# Sys.setenv("AWS_ACCESS_KEY_ID" = "",
#            "AWS_SECRET_ACCESS_KEY" = "",
#            "AWS_DEFAULT_REGION" = "",
#            "AWS_BUCKET_NAME" = "",
#            "AWS_OBJECT_KEY" = "",
#            "AWS_SSE_TYPE" = "AES256",
#            "USE_MPP_BULK_LOAD" = TRUE)

# which DQ check levels to run -------------------------------------------------------------------
checkLevels <- c("TABLE", "FIELD", "CONCEPT")

# which DQ checks to run? ------------------------------------
checkNames <- c() # Names can be found in inst/csv/OMOP_CDM_v5.3_Check_Descriptions.csv

# which DQ severity levels to run? ----------------------------
checkSeverity <- c("fatal", "convention", "characterization") 

# want to EXCLUDE a pre-specified list of checks? run the following code:
#
# checksToExclude <- c() # Names of check types to exclude from your DQD run
# allChecks <- DataQualityDashboard::listDqChecks()
# checkNames <- allChecks$checkDescriptions %>%
#   subset(!(checkName %in% checksToExclude)) %>%
#   select(checkName)

# which CDM tables to exclude? ------------------------------------
tablesToExclude <- c("CONCEPT", "VOCABULARY", "CONCEPT_ANCESTOR", "CONCEPT_RELATIONSHIP", "CONCEPT_CLASS", "CONCEPT_SYNONYM", "RELATIONSHIP", "DOMAIN") # list of CDM table names to skip evaluating checks against; by default DQD excludes the vocab tables

# run the job --------------------------------------------------------------------------------------
DataQualityDashboard::executeDqChecks(connectionDetails = connectionDetails, 
                              cdmDatabaseSchema = cdmDatabaseSchema, 
                              resultsDatabaseSchema = resultsDatabaseSchema,
                              cdmSourceName = cdmSourceName, 
                              cdmVersion = cdmVersion,
                              numThreads = numThreads,
                              sqlOnly = sqlOnly, 
                              sqlOnlyUnionCount = sqlOnlyUnionCount,
                              sqlOnlyIncrementalInsert = sqlOnlyIncrementalInsert,
                              outputFolder = outputFolder,
                              outputFile = outputFile,
                              verboseMode = verboseMode,
                              writeToTable = writeToTable,
                              writeToCsv = writeToCsv,
                              csvFile = csvFile,
                              checkLevels = checkLevels,
			      checkSeverity = checkSeverity,
                              tablesToExclude = tablesToExclude,
                              checkNames = checkNames)

# inspect logs ----------------------------------------------------------------------------
ParallelLogger::launchLogViewer(logFileName = file.path(outputFolder, cdmSourceName, 
                                                        sprintf("log_DqDashboard_%s.txt", cdmSourceName)))

# (OPTIONAL) if you want to write the JSON file to the results table separately -----------------------------
jsonFilePath <- ""
DataQualityDashboard::writeJsonResultsToTable(connectionDetails = connectionDetails, 
                                              resultsDatabaseSchema = resultsDatabaseSchema, 
                                              jsonFilePath = jsonFilePath)
                                              

```

Viewing Results
================

**Launching Dashboard as Shiny App**
```r
# Use the fully-qualified path to the JSON results file
DataQualityDashboard::viewDqDashboard(jsonFilePath)
```

**Launching on a web server**

If you have npm installed:

1. Install http-server:

```
npm install -g http-server
```

2. Name the output file *results.json* and place it in inst/shinyApps/www

3. Go to inst/shinyApps/www, then run:

```
http-server
```

View checks
===========
To see description of checks using R, execute the command below:
```
checks <- DataQualityDashboard::listDqChecks(cdmVersion = "5.3") # Put the version of the CDM you are using

```

# Execute DQ checks using sqlOnly=TRUE and sqlOnlyUnionCount=4 and sqlOnlyIncrementalInsert=TRUE

    Code
      cat(SqlRender::readSql(dqdSqlFilePath))
    Output
      /*********
      SQL to insert individual DQD results directly into output table, rather than waiting until collecting all results.
      Note that this  does not include information about SQL errors or performance
      **********/
      
      WITH cte_all AS (
        /*********
      SQL to create query for insertion into results table. These may be unioned together prior to insert.
      Note that this does not include information about SQL errors or performance.
      **********/
      
      SELECT 
        cte.num_violated_rows
        ,cte.pct_violated_rows
        ,cte.num_denominator_rows
        ,'' as execution_time
        ,'' as query_text
        ,'measurePersonCompleteness' as check_name
        ,'TABLE' as check_level
        ,'The number and percent of persons in the CDM that do not have at least one record in the OBSERVATION_PERIOD table' as check_description
        ,'OBSERVATION_PERIOD' as cdm_table_name
        ,'NA' as cdm_field_name
        ,'NA' as concept_id
        ,'NA' as unit_concept_id
        ,'table_person_completeness.sql' as sql_file
        ,'Completeness' as category
        ,'NA' as subcategory
        ,'Validation' as context
        ,'' as warning
        ,'' as error
        ,'table_measurepersoncompleteness_observation_period' as checkid
        ,0 as is_error
        ,0 as not_applicable
        ,CASE WHEN (cte.pct_violated_rows * 100) > 0 THEN 1 ELSE 0 END as failed
        ,CASE WHEN (cte.pct_violated_rows * 100) <= 0 THEN 1 ELSE 0 END as passed
        ,NULL as not_applicable_reason
        ,0 as threshold_value
        ,NULL as notes_value
      FROM (
        
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = OBSERVATION_PERIOD
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.OBSERVATION_PERIOD cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      
      
      ) cte
       UNION ALL /*********
      SQL to create query for insertion into results table. These may be unioned together prior to insert.
      Note that this does not include information about SQL errors or performance.
      **********/
      
      SELECT 
        cte.num_violated_rows
        ,cte.pct_violated_rows
        ,cte.num_denominator_rows
        ,'' as execution_time
        ,'' as query_text
        ,'measurePersonCompleteness' as check_name
        ,'TABLE' as check_level
        ,'The number and percent of persons in the CDM that do not have at least one record in the VISIT_OCCURRENCE table' as check_description
        ,'VISIT_OCCURRENCE' as cdm_table_name
        ,'NA' as cdm_field_name
        ,'NA' as concept_id
        ,'NA' as unit_concept_id
        ,'table_person_completeness.sql' as sql_file
        ,'Completeness' as category
        ,'NA' as subcategory
        ,'Validation' as context
        ,'' as warning
        ,'' as error
        ,'table_measurepersoncompleteness_visit_occurrence' as checkid
        ,0 as is_error
        ,0 as not_applicable
        ,CASE WHEN (cte.pct_violated_rows * 100) > 95 THEN 1 ELSE 0 END as failed
        ,CASE WHEN (cte.pct_violated_rows * 100) <= 95 THEN 1 ELSE 0 END as passed
        ,NULL as not_applicable_reason
        ,95 as threshold_value
        ,NULL as notes_value
      FROM (
        
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = VISIT_OCCURRENCE
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.VISIT_OCCURRENCE cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      
      
      ) cte
       UNION ALL /*********
      SQL to create query for insertion into results table. These may be unioned together prior to insert.
      Note that this does not include information about SQL errors or performance.
      **********/
      
      SELECT 
        cte.num_violated_rows
        ,cte.pct_violated_rows
        ,cte.num_denominator_rows
        ,'' as execution_time
        ,'' as query_text
        ,'measurePersonCompleteness' as check_name
        ,'TABLE' as check_level
        ,'The number and percent of persons in the CDM that do not have at least one record in the VISIT_DETAIL table' as check_description
        ,'VISIT_DETAIL' as cdm_table_name
        ,'NA' as cdm_field_name
        ,'NA' as concept_id
        ,'NA' as unit_concept_id
        ,'table_person_completeness.sql' as sql_file
        ,'Completeness' as category
        ,'NA' as subcategory
        ,'Validation' as context
        ,'' as warning
        ,'' as error
        ,'table_measurepersoncompleteness_visit_detail' as checkid
        ,0 as is_error
        ,0 as not_applicable
        ,CASE WHEN (cte.pct_violated_rows * 100) > 100 THEN 1 ELSE 0 END as failed
        ,CASE WHEN (cte.pct_violated_rows * 100) <= 100 THEN 1 ELSE 0 END as passed
        ,NULL as not_applicable_reason
        ,100 as threshold_value
        ,NULL as notes_value
      FROM (
        
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = VISIT_DETAIL
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.VISIT_DETAIL cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      
      
      ) cte
       UNION ALL /*********
      SQL to create query for insertion into results table. These may be unioned together prior to insert.
      Note that this does not include information about SQL errors or performance.
      **********/
      
      SELECT 
        cte.num_violated_rows
        ,cte.pct_violated_rows
        ,cte.num_denominator_rows
        ,'' as execution_time
        ,'' as query_text
        ,'measurePersonCompleteness' as check_name
        ,'TABLE' as check_level
        ,'The number and percent of persons in the CDM that do not have at least one record in the CONDITION_OCCURRENCE table' as check_description
        ,'CONDITION_OCCURRENCE' as cdm_table_name
        ,'NA' as cdm_field_name
        ,'NA' as concept_id
        ,'NA' as unit_concept_id
        ,'table_person_completeness.sql' as sql_file
        ,'Completeness' as category
        ,'NA' as subcategory
        ,'Validation' as context
        ,'' as warning
        ,'' as error
        ,'table_measurepersoncompleteness_condition_occurrence' as checkid
        ,0 as is_error
        ,0 as not_applicable
        ,CASE WHEN (cte.pct_violated_rows * 100) > 95 THEN 1 ELSE 0 END as failed
        ,CASE WHEN (cte.pct_violated_rows * 100) <= 95 THEN 1 ELSE 0 END as passed
        ,NULL as not_applicable_reason
        ,95 as threshold_value
        ,NULL as notes_value
      FROM (
        
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = CONDITION_OCCURRENCE
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.CONDITION_OCCURRENCE cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      
      
      ) cte
      
      )
      INSERT INTO @yourResultsSchema.dqdashboard_results
      SELECT *
      FROM cte_all
      ;
      
      /*********
      SQL to insert individual DQD results directly into output table, rather than waiting until collecting all results.
      Note that this  does not include information about SQL errors or performance
      **********/
      
      WITH cte_all AS (
        /*********
      SQL to create query for insertion into results table. These may be unioned together prior to insert.
      Note that this does not include information about SQL errors or performance.
      **********/
      
      SELECT 
        cte.num_violated_rows
        ,cte.pct_violated_rows
        ,cte.num_denominator_rows
        ,'' as execution_time
        ,'' as query_text
        ,'measurePersonCompleteness' as check_name
        ,'TABLE' as check_level
        ,'The number and percent of persons in the CDM that do not have at least one record in the DRUG_EXPOSURE table' as check_description
        ,'DRUG_EXPOSURE' as cdm_table_name
        ,'NA' as cdm_field_name
        ,'NA' as concept_id
        ,'NA' as unit_concept_id
        ,'table_person_completeness.sql' as sql_file
        ,'Completeness' as category
        ,'NA' as subcategory
        ,'Validation' as context
        ,'' as warning
        ,'' as error
        ,'table_measurepersoncompleteness_drug_exposure' as checkid
        ,0 as is_error
        ,0 as not_applicable
        ,CASE WHEN (cte.pct_violated_rows * 100) > 95 THEN 1 ELSE 0 END as failed
        ,CASE WHEN (cte.pct_violated_rows * 100) <= 95 THEN 1 ELSE 0 END as passed
        ,NULL as not_applicable_reason
        ,95 as threshold_value
        ,NULL as notes_value
      FROM (
        
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = DRUG_EXPOSURE
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.DRUG_EXPOSURE cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      
      
      ) cte
       UNION ALL /*********
      SQL to create query for insertion into results table. These may be unioned together prior to insert.
      Note that this does not include information about SQL errors or performance.
      **********/
      
      SELECT 
        cte.num_violated_rows
        ,cte.pct_violated_rows
        ,cte.num_denominator_rows
        ,'' as execution_time
        ,'' as query_text
        ,'measurePersonCompleteness' as check_name
        ,'TABLE' as check_level
        ,'The number and percent of persons in the CDM that do not have at least one record in the PROCEDURE_OCCURRENCE table' as check_description
        ,'PROCEDURE_OCCURRENCE' as cdm_table_name
        ,'NA' as cdm_field_name
        ,'NA' as concept_id
        ,'NA' as unit_concept_id
        ,'table_person_completeness.sql' as sql_file
        ,'Completeness' as category
        ,'NA' as subcategory
        ,'Validation' as context
        ,'' as warning
        ,'' as error
        ,'table_measurepersoncompleteness_procedure_occurrence' as checkid
        ,0 as is_error
        ,0 as not_applicable
        ,CASE WHEN (cte.pct_violated_rows * 100) > 95 THEN 1 ELSE 0 END as failed
        ,CASE WHEN (cte.pct_violated_rows * 100) <= 95 THEN 1 ELSE 0 END as passed
        ,NULL as not_applicable_reason
        ,95 as threshold_value
        ,NULL as notes_value
      FROM (
        
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = PROCEDURE_OCCURRENCE
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.PROCEDURE_OCCURRENCE cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      
      
      ) cte
       UNION ALL /*********
      SQL to create query for insertion into results table. These may be unioned together prior to insert.
      Note that this does not include information about SQL errors or performance.
      **********/
      
      SELECT 
        cte.num_violated_rows
        ,cte.pct_violated_rows
        ,cte.num_denominator_rows
        ,'' as execution_time
        ,'' as query_text
        ,'measurePersonCompleteness' as check_name
        ,'TABLE' as check_level
        ,'The number and percent of persons in the CDM that do not have at least one record in the DEVICE_EXPOSURE table' as check_description
        ,'DEVICE_EXPOSURE' as cdm_table_name
        ,'NA' as cdm_field_name
        ,'NA' as concept_id
        ,'NA' as unit_concept_id
        ,'table_person_completeness.sql' as sql_file
        ,'Completeness' as category
        ,'NA' as subcategory
        ,'Validation' as context
        ,'' as warning
        ,'' as error
        ,'table_measurepersoncompleteness_device_exposure' as checkid
        ,0 as is_error
        ,0 as not_applicable
        ,CASE WHEN (cte.pct_violated_rows * 100) > 100 THEN 1 ELSE 0 END as failed
        ,CASE WHEN (cte.pct_violated_rows * 100) <= 100 THEN 1 ELSE 0 END as passed
        ,NULL as not_applicable_reason
        ,100 as threshold_value
        ,NULL as notes_value
      FROM (
        
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = DEVICE_EXPOSURE
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.DEVICE_EXPOSURE cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      
      
      ) cte
       UNION ALL /*********
      SQL to create query for insertion into results table. These may be unioned together prior to insert.
      Note that this does not include information about SQL errors or performance.
      **********/
      
      SELECT 
        cte.num_violated_rows
        ,cte.pct_violated_rows
        ,cte.num_denominator_rows
        ,'' as execution_time
        ,'' as query_text
        ,'measurePersonCompleteness' as check_name
        ,'TABLE' as check_level
        ,'The number and percent of persons in the CDM that do not have at least one record in the MEASUREMENT table' as check_description
        ,'MEASUREMENT' as cdm_table_name
        ,'NA' as cdm_field_name
        ,'NA' as concept_id
        ,'NA' as unit_concept_id
        ,'table_person_completeness.sql' as sql_file
        ,'Completeness' as category
        ,'NA' as subcategory
        ,'Validation' as context
        ,'' as warning
        ,'' as error
        ,'table_measurepersoncompleteness_measurement' as checkid
        ,0 as is_error
        ,0 as not_applicable
        ,CASE WHEN (cte.pct_violated_rows * 100) > 95 THEN 1 ELSE 0 END as failed
        ,CASE WHEN (cte.pct_violated_rows * 100) <= 95 THEN 1 ELSE 0 END as passed
        ,NULL as not_applicable_reason
        ,95 as threshold_value
        ,NULL as notes_value
      FROM (
        
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = MEASUREMENT
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.MEASUREMENT cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      
      
      ) cte
      
      )
      INSERT INTO @yourResultsSchema.dqdashboard_results
      SELECT *
      FROM cte_all
      ;
      
      /*********
      SQL to insert individual DQD results directly into output table, rather than waiting until collecting all results.
      Note that this  does not include information about SQL errors or performance
      **********/
      
      WITH cte_all AS (
        /*********
      SQL to create query for insertion into results table. These may be unioned together prior to insert.
      Note that this does not include information about SQL errors or performance.
      **********/
      
      SELECT 
        cte.num_violated_rows
        ,cte.pct_violated_rows
        ,cte.num_denominator_rows
        ,'' as execution_time
        ,'' as query_text
        ,'measurePersonCompleteness' as check_name
        ,'TABLE' as check_level
        ,'The number and percent of persons in the CDM that do not have at least one record in the OBSERVATION table' as check_description
        ,'OBSERVATION' as cdm_table_name
        ,'NA' as cdm_field_name
        ,'NA' as concept_id
        ,'NA' as unit_concept_id
        ,'table_person_completeness.sql' as sql_file
        ,'Completeness' as category
        ,'NA' as subcategory
        ,'Validation' as context
        ,'' as warning
        ,'' as error
        ,'table_measurepersoncompleteness_observation' as checkid
        ,0 as is_error
        ,0 as not_applicable
        ,CASE WHEN (cte.pct_violated_rows * 100) > 95 THEN 1 ELSE 0 END as failed
        ,CASE WHEN (cte.pct_violated_rows * 100) <= 95 THEN 1 ELSE 0 END as passed
        ,NULL as not_applicable_reason
        ,95 as threshold_value
        ,NULL as notes_value
      FROM (
        
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = OBSERVATION
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.OBSERVATION cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      
      
      ) cte
       UNION ALL /*********
      SQL to create query for insertion into results table. These may be unioned together prior to insert.
      Note that this does not include information about SQL errors or performance.
      **********/
      
      SELECT 
        cte.num_violated_rows
        ,cte.pct_violated_rows
        ,cte.num_denominator_rows
        ,'' as execution_time
        ,'' as query_text
        ,'measurePersonCompleteness' as check_name
        ,'TABLE' as check_level
        ,'The number and percent of persons in the CDM that do not have at least one record in the DEATH table' as check_description
        ,'DEATH' as cdm_table_name
        ,'NA' as cdm_field_name
        ,'NA' as concept_id
        ,'NA' as unit_concept_id
        ,'table_person_completeness.sql' as sql_file
        ,'Completeness' as category
        ,'NA' as subcategory
        ,'Validation' as context
        ,'' as warning
        ,'' as error
        ,'table_measurepersoncompleteness_death' as checkid
        ,0 as is_error
        ,0 as not_applicable
        ,CASE WHEN (cte.pct_violated_rows * 100) > 100 THEN 1 ELSE 0 END as failed
        ,CASE WHEN (cte.pct_violated_rows * 100) <= 100 THEN 1 ELSE 0 END as passed
        ,NULL as not_applicable_reason
        ,100 as threshold_value
        ,NULL as notes_value
      FROM (
        
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = DEATH
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.DEATH cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      
      
      ) cte
       UNION ALL /*********
      SQL to create query for insertion into results table. These may be unioned together prior to insert.
      Note that this does not include information about SQL errors or performance.
      **********/
      
      SELECT 
        cte.num_violated_rows
        ,cte.pct_violated_rows
        ,cte.num_denominator_rows
        ,'' as execution_time
        ,'' as query_text
        ,'measurePersonCompleteness' as check_name
        ,'TABLE' as check_level
        ,'The number and percent of persons in the CDM that do not have at least one record in the NOTE table' as check_description
        ,'NOTE' as cdm_table_name
        ,'NA' as cdm_field_name
        ,'NA' as concept_id
        ,'NA' as unit_concept_id
        ,'table_person_completeness.sql' as sql_file
        ,'Completeness' as category
        ,'NA' as subcategory
        ,'Validation' as context
        ,'' as warning
        ,'' as error
        ,'table_measurepersoncompleteness_note' as checkid
        ,0 as is_error
        ,0 as not_applicable
        ,CASE WHEN (cte.pct_violated_rows * 100) > 100 THEN 1 ELSE 0 END as failed
        ,CASE WHEN (cte.pct_violated_rows * 100) <= 100 THEN 1 ELSE 0 END as passed
        ,NULL as not_applicable_reason
        ,100 as threshold_value
        ,NULL as notes_value
      FROM (
        
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = NOTE
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.NOTE cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      
      
      ) cte
       UNION ALL /*********
      SQL to create query for insertion into results table. These may be unioned together prior to insert.
      Note that this does not include information about SQL errors or performance.
      **********/
      
      SELECT 
        cte.num_violated_rows
        ,cte.pct_violated_rows
        ,cte.num_denominator_rows
        ,'' as execution_time
        ,'' as query_text
        ,'measurePersonCompleteness' as check_name
        ,'TABLE' as check_level
        ,'The number and percent of persons in the CDM that do not have at least one record in the SPECIMEN table' as check_description
        ,'SPECIMEN' as cdm_table_name
        ,'NA' as cdm_field_name
        ,'NA' as concept_id
        ,'NA' as unit_concept_id
        ,'table_person_completeness.sql' as sql_file
        ,'Completeness' as category
        ,'NA' as subcategory
        ,'Validation' as context
        ,'' as warning
        ,'' as error
        ,'table_measurepersoncompleteness_specimen' as checkid
        ,0 as is_error
        ,0 as not_applicable
        ,CASE WHEN (cte.pct_violated_rows * 100) > 100 THEN 1 ELSE 0 END as failed
        ,CASE WHEN (cte.pct_violated_rows * 100) <= 100 THEN 1 ELSE 0 END as passed
        ,NULL as not_applicable_reason
        ,100 as threshold_value
        ,NULL as notes_value
      FROM (
        
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = SPECIMEN
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.SPECIMEN cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      
      
      ) cte
      
      )
      INSERT INTO @yourResultsSchema.dqdashboard_results
      SELECT *
      FROM cte_all
      ;
      
      /*********
      SQL to insert individual DQD results directly into output table, rather than waiting until collecting all results.
      Note that this  does not include information about SQL errors or performance
      **********/
      
      WITH cte_all AS (
        /*********
      SQL to create query for insertion into results table. These may be unioned together prior to insert.
      Note that this does not include information about SQL errors or performance.
      **********/
      
      SELECT 
        cte.num_violated_rows
        ,cte.pct_violated_rows
        ,cte.num_denominator_rows
        ,'' as execution_time
        ,'' as query_text
        ,'measurePersonCompleteness' as check_name
        ,'TABLE' as check_level
        ,'The number and percent of persons in the CDM that do not have at least one record in the PAYER_PLAN_PERIOD table' as check_description
        ,'PAYER_PLAN_PERIOD' as cdm_table_name
        ,'NA' as cdm_field_name
        ,'NA' as concept_id
        ,'NA' as unit_concept_id
        ,'table_person_completeness.sql' as sql_file
        ,'Completeness' as category
        ,'NA' as subcategory
        ,'Validation' as context
        ,'' as warning
        ,'' as error
        ,'table_measurepersoncompleteness_payer_plan_period' as checkid
        ,0 as is_error
        ,0 as not_applicable
        ,CASE WHEN (cte.pct_violated_rows * 100) > 100 THEN 1 ELSE 0 END as failed
        ,CASE WHEN (cte.pct_violated_rows * 100) <= 100 THEN 1 ELSE 0 END as passed
        ,NULL as not_applicable_reason
        ,100 as threshold_value
        ,NULL as notes_value
      FROM (
        
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = PAYER_PLAN_PERIOD
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.PAYER_PLAN_PERIOD cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      
      
      ) cte
       UNION ALL /*********
      SQL to create query for insertion into results table. These may be unioned together prior to insert.
      Note that this does not include information about SQL errors or performance.
      **********/
      
      SELECT 
        cte.num_violated_rows
        ,cte.pct_violated_rows
        ,cte.num_denominator_rows
        ,'' as execution_time
        ,'' as query_text
        ,'measurePersonCompleteness' as check_name
        ,'TABLE' as check_level
        ,'The number and percent of persons in the CDM that do not have at least one record in the DRUG_ERA table' as check_description
        ,'DRUG_ERA' as cdm_table_name
        ,'NA' as cdm_field_name
        ,'NA' as concept_id
        ,'NA' as unit_concept_id
        ,'table_person_completeness.sql' as sql_file
        ,'Completeness' as category
        ,'NA' as subcategory
        ,'Validation' as context
        ,'' as warning
        ,'' as error
        ,'table_measurepersoncompleteness_drug_era' as checkid
        ,0 as is_error
        ,0 as not_applicable
        ,CASE WHEN (cte.pct_violated_rows * 100) > 95 THEN 1 ELSE 0 END as failed
        ,CASE WHEN (cte.pct_violated_rows * 100) <= 95 THEN 1 ELSE 0 END as passed
        ,NULL as not_applicable_reason
        ,95 as threshold_value
        ,NULL as notes_value
      FROM (
        
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = DRUG_ERA
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.DRUG_ERA cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      
      
      ) cte
       UNION ALL /*********
      SQL to create query for insertion into results table. These may be unioned together prior to insert.
      Note that this does not include information about SQL errors or performance.
      **********/
      
      SELECT 
        cte.num_violated_rows
        ,cte.pct_violated_rows
        ,cte.num_denominator_rows
        ,'' as execution_time
        ,'' as query_text
        ,'measurePersonCompleteness' as check_name
        ,'TABLE' as check_level
        ,'The number and percent of persons in the CDM that do not have at least one record in the DOSE_ERA table' as check_description
        ,'DOSE_ERA' as cdm_table_name
        ,'NA' as cdm_field_name
        ,'NA' as concept_id
        ,'NA' as unit_concept_id
        ,'table_person_completeness.sql' as sql_file
        ,'Completeness' as category
        ,'NA' as subcategory
        ,'Validation' as context
        ,'' as warning
        ,'' as error
        ,'table_measurepersoncompleteness_dose_era' as checkid
        ,0 as is_error
        ,0 as not_applicable
        ,CASE WHEN (cte.pct_violated_rows * 100) > 100 THEN 1 ELSE 0 END as failed
        ,CASE WHEN (cte.pct_violated_rows * 100) <= 100 THEN 1 ELSE 0 END as passed
        ,NULL as not_applicable_reason
        ,100 as threshold_value
        ,NULL as notes_value
      FROM (
        
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = DOSE_ERA
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.DOSE_ERA cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      
      
      ) cte
       UNION ALL /*********
      SQL to create query for insertion into results table. These may be unioned together prior to insert.
      Note that this does not include information about SQL errors or performance.
      **********/
      
      SELECT 
        cte.num_violated_rows
        ,cte.pct_violated_rows
        ,cte.num_denominator_rows
        ,'' as execution_time
        ,'' as query_text
        ,'measurePersonCompleteness' as check_name
        ,'TABLE' as check_level
        ,'The number and percent of persons in the CDM that do not have at least one record in the CONDITION_ERA table' as check_description
        ,'CONDITION_ERA' as cdm_table_name
        ,'NA' as cdm_field_name
        ,'NA' as concept_id
        ,'NA' as unit_concept_id
        ,'table_person_completeness.sql' as sql_file
        ,'Completeness' as category
        ,'NA' as subcategory
        ,'Validation' as context
        ,'' as warning
        ,'' as error
        ,'table_measurepersoncompleteness_condition_era' as checkid
        ,0 as is_error
        ,0 as not_applicable
        ,CASE WHEN (cte.pct_violated_rows * 100) > 95 THEN 1 ELSE 0 END as failed
        ,CASE WHEN (cte.pct_violated_rows * 100) <= 95 THEN 1 ELSE 0 END as passed
        ,NULL as not_applicable_reason
        ,95 as threshold_value
        ,NULL as notes_value
      FROM (
        
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = CONDITION_ERA
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.CONDITION_ERA cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      
      
      ) cte
      
      )
      INSERT INTO @yourResultsSchema.dqdashboard_results
      SELECT *
      FROM cte_all
      ;
      

# Execute DQ checks using sqlOnly=TRUE and sqlOnlyUnionCount=1 and sqlOnlyIncrementalInsert=TRUE

    Code
      cat(SqlRender::readSql(dqdSqlFilePath))
    Output
      /*********
      SQL to insert individual DQD results directly into output table, rather than waiting until collecting all results.
      Note that this  does not include information about SQL errors or performance
      **********/
      
      WITH cte_all AS (
        /*********
      SQL to create query for insertion into results table. These may be unioned together prior to insert.
      Note that this does not include information about SQL errors or performance.
      **********/
      
      SELECT 
        cte.num_violated_rows
        ,cte.pct_violated_rows
        ,cte.num_denominator_rows
        ,'' as execution_time
        ,'' as query_text
        ,'measurePersonCompleteness' as check_name
        ,'TABLE' as check_level
        ,'The number and percent of persons in the CDM that do not have at least one record in the OBSERVATION_PERIOD table' as check_description
        ,'OBSERVATION_PERIOD' as cdm_table_name
        ,'NA' as cdm_field_name
        ,'NA' as concept_id
        ,'NA' as unit_concept_id
        ,'table_person_completeness.sql' as sql_file
        ,'Completeness' as category
        ,'NA' as subcategory
        ,'Validation' as context
        ,'' as warning
        ,'' as error
        ,'table_measurepersoncompleteness_observation_period' as checkid
        ,0 as is_error
        ,0 as not_applicable
        ,CASE WHEN (cte.pct_violated_rows * 100) > 0 THEN 1 ELSE 0 END as failed
        ,CASE WHEN (cte.pct_violated_rows * 100) <= 0 THEN 1 ELSE 0 END as passed
        ,NULL as not_applicable_reason
        ,0 as threshold_value
        ,NULL as notes_value
      FROM (
        
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = OBSERVATION_PERIOD
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.OBSERVATION_PERIOD cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      
      
      ) cte
      
      )
      INSERT INTO @yourResultsSchema.dqdashboard_results
      SELECT *
      FROM cte_all
      ;
      
      /*********
      SQL to insert individual DQD results directly into output table, rather than waiting until collecting all results.
      Note that this  does not include information about SQL errors or performance
      **********/
      
      WITH cte_all AS (
        /*********
      SQL to create query for insertion into results table. These may be unioned together prior to insert.
      Note that this does not include information about SQL errors or performance.
      **********/
      
      SELECT 
        cte.num_violated_rows
        ,cte.pct_violated_rows
        ,cte.num_denominator_rows
        ,'' as execution_time
        ,'' as query_text
        ,'measurePersonCompleteness' as check_name
        ,'TABLE' as check_level
        ,'The number and percent of persons in the CDM that do not have at least one record in the VISIT_OCCURRENCE table' as check_description
        ,'VISIT_OCCURRENCE' as cdm_table_name
        ,'NA' as cdm_field_name
        ,'NA' as concept_id
        ,'NA' as unit_concept_id
        ,'table_person_completeness.sql' as sql_file
        ,'Completeness' as category
        ,'NA' as subcategory
        ,'Validation' as context
        ,'' as warning
        ,'' as error
        ,'table_measurepersoncompleteness_visit_occurrence' as checkid
        ,0 as is_error
        ,0 as not_applicable
        ,CASE WHEN (cte.pct_violated_rows * 100) > 95 THEN 1 ELSE 0 END as failed
        ,CASE WHEN (cte.pct_violated_rows * 100) <= 95 THEN 1 ELSE 0 END as passed
        ,NULL as not_applicable_reason
        ,95 as threshold_value
        ,NULL as notes_value
      FROM (
        
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = VISIT_OCCURRENCE
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.VISIT_OCCURRENCE cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      
      
      ) cte
      
      )
      INSERT INTO @yourResultsSchema.dqdashboard_results
      SELECT *
      FROM cte_all
      ;
      
      /*********
      SQL to insert individual DQD results directly into output table, rather than waiting until collecting all results.
      Note that this  does not include information about SQL errors or performance
      **********/
      
      WITH cte_all AS (
        /*********
      SQL to create query for insertion into results table. These may be unioned together prior to insert.
      Note that this does not include information about SQL errors or performance.
      **********/
      
      SELECT 
        cte.num_violated_rows
        ,cte.pct_violated_rows
        ,cte.num_denominator_rows
        ,'' as execution_time
        ,'' as query_text
        ,'measurePersonCompleteness' as check_name
        ,'TABLE' as check_level
        ,'The number and percent of persons in the CDM that do not have at least one record in the VISIT_DETAIL table' as check_description
        ,'VISIT_DETAIL' as cdm_table_name
        ,'NA' as cdm_field_name
        ,'NA' as concept_id
        ,'NA' as unit_concept_id
        ,'table_person_completeness.sql' as sql_file
        ,'Completeness' as category
        ,'NA' as subcategory
        ,'Validation' as context
        ,'' as warning
        ,'' as error
        ,'table_measurepersoncompleteness_visit_detail' as checkid
        ,0 as is_error
        ,0 as not_applicable
        ,CASE WHEN (cte.pct_violated_rows * 100) > 100 THEN 1 ELSE 0 END as failed
        ,CASE WHEN (cte.pct_violated_rows * 100) <= 100 THEN 1 ELSE 0 END as passed
        ,NULL as not_applicable_reason
        ,100 as threshold_value
        ,NULL as notes_value
      FROM (
        
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = VISIT_DETAIL
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.VISIT_DETAIL cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      
      
      ) cte
      
      )
      INSERT INTO @yourResultsSchema.dqdashboard_results
      SELECT *
      FROM cte_all
      ;
      
      /*********
      SQL to insert individual DQD results directly into output table, rather than waiting until collecting all results.
      Note that this  does not include information about SQL errors or performance
      **********/
      
      WITH cte_all AS (
        /*********
      SQL to create query for insertion into results table. These may be unioned together prior to insert.
      Note that this does not include information about SQL errors or performance.
      **********/
      
      SELECT 
        cte.num_violated_rows
        ,cte.pct_violated_rows
        ,cte.num_denominator_rows
        ,'' as execution_time
        ,'' as query_text
        ,'measurePersonCompleteness' as check_name
        ,'TABLE' as check_level
        ,'The number and percent of persons in the CDM that do not have at least one record in the CONDITION_OCCURRENCE table' as check_description
        ,'CONDITION_OCCURRENCE' as cdm_table_name
        ,'NA' as cdm_field_name
        ,'NA' as concept_id
        ,'NA' as unit_concept_id
        ,'table_person_completeness.sql' as sql_file
        ,'Completeness' as category
        ,'NA' as subcategory
        ,'Validation' as context
        ,'' as warning
        ,'' as error
        ,'table_measurepersoncompleteness_condition_occurrence' as checkid
        ,0 as is_error
        ,0 as not_applicable
        ,CASE WHEN (cte.pct_violated_rows * 100) > 95 THEN 1 ELSE 0 END as failed
        ,CASE WHEN (cte.pct_violated_rows * 100) <= 95 THEN 1 ELSE 0 END as passed
        ,NULL as not_applicable_reason
        ,95 as threshold_value
        ,NULL as notes_value
      FROM (
        
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = CONDITION_OCCURRENCE
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.CONDITION_OCCURRENCE cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      
      
      ) cte
      
      )
      INSERT INTO @yourResultsSchema.dqdashboard_results
      SELECT *
      FROM cte_all
      ;
      
      /*********
      SQL to insert individual DQD results directly into output table, rather than waiting until collecting all results.
      Note that this  does not include information about SQL errors or performance
      **********/
      
      WITH cte_all AS (
        /*********
      SQL to create query for insertion into results table. These may be unioned together prior to insert.
      Note that this does not include information about SQL errors or performance.
      **********/
      
      SELECT 
        cte.num_violated_rows
        ,cte.pct_violated_rows
        ,cte.num_denominator_rows
        ,'' as execution_time
        ,'' as query_text
        ,'measurePersonCompleteness' as check_name
        ,'TABLE' as check_level
        ,'The number and percent of persons in the CDM that do not have at least one record in the DRUG_EXPOSURE table' as check_description
        ,'DRUG_EXPOSURE' as cdm_table_name
        ,'NA' as cdm_field_name
        ,'NA' as concept_id
        ,'NA' as unit_concept_id
        ,'table_person_completeness.sql' as sql_file
        ,'Completeness' as category
        ,'NA' as subcategory
        ,'Validation' as context
        ,'' as warning
        ,'' as error
        ,'table_measurepersoncompleteness_drug_exposure' as checkid
        ,0 as is_error
        ,0 as not_applicable
        ,CASE WHEN (cte.pct_violated_rows * 100) > 95 THEN 1 ELSE 0 END as failed
        ,CASE WHEN (cte.pct_violated_rows * 100) <= 95 THEN 1 ELSE 0 END as passed
        ,NULL as not_applicable_reason
        ,95 as threshold_value
        ,NULL as notes_value
      FROM (
        
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = DRUG_EXPOSURE
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.DRUG_EXPOSURE cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      
      
      ) cte
      
      )
      INSERT INTO @yourResultsSchema.dqdashboard_results
      SELECT *
      FROM cte_all
      ;
      
      /*********
      SQL to insert individual DQD results directly into output table, rather than waiting until collecting all results.
      Note that this  does not include information about SQL errors or performance
      **********/
      
      WITH cte_all AS (
        /*********
      SQL to create query for insertion into results table. These may be unioned together prior to insert.
      Note that this does not include information about SQL errors or performance.
      **********/
      
      SELECT 
        cte.num_violated_rows
        ,cte.pct_violated_rows
        ,cte.num_denominator_rows
        ,'' as execution_time
        ,'' as query_text
        ,'measurePersonCompleteness' as check_name
        ,'TABLE' as check_level
        ,'The number and percent of persons in the CDM that do not have at least one record in the PROCEDURE_OCCURRENCE table' as check_description
        ,'PROCEDURE_OCCURRENCE' as cdm_table_name
        ,'NA' as cdm_field_name
        ,'NA' as concept_id
        ,'NA' as unit_concept_id
        ,'table_person_completeness.sql' as sql_file
        ,'Completeness' as category
        ,'NA' as subcategory
        ,'Validation' as context
        ,'' as warning
        ,'' as error
        ,'table_measurepersoncompleteness_procedure_occurrence' as checkid
        ,0 as is_error
        ,0 as not_applicable
        ,CASE WHEN (cte.pct_violated_rows * 100) > 95 THEN 1 ELSE 0 END as failed
        ,CASE WHEN (cte.pct_violated_rows * 100) <= 95 THEN 1 ELSE 0 END as passed
        ,NULL as not_applicable_reason
        ,95 as threshold_value
        ,NULL as notes_value
      FROM (
        
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = PROCEDURE_OCCURRENCE
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.PROCEDURE_OCCURRENCE cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      
      
      ) cte
      
      )
      INSERT INTO @yourResultsSchema.dqdashboard_results
      SELECT *
      FROM cte_all
      ;
      
      /*********
      SQL to insert individual DQD results directly into output table, rather than waiting until collecting all results.
      Note that this  does not include information about SQL errors or performance
      **********/
      
      WITH cte_all AS (
        /*********
      SQL to create query for insertion into results table. These may be unioned together prior to insert.
      Note that this does not include information about SQL errors or performance.
      **********/
      
      SELECT 
        cte.num_violated_rows
        ,cte.pct_violated_rows
        ,cte.num_denominator_rows
        ,'' as execution_time
        ,'' as query_text
        ,'measurePersonCompleteness' as check_name
        ,'TABLE' as check_level
        ,'The number and percent of persons in the CDM that do not have at least one record in the DEVICE_EXPOSURE table' as check_description
        ,'DEVICE_EXPOSURE' as cdm_table_name
        ,'NA' as cdm_field_name
        ,'NA' as concept_id
        ,'NA' as unit_concept_id
        ,'table_person_completeness.sql' as sql_file
        ,'Completeness' as category
        ,'NA' as subcategory
        ,'Validation' as context
        ,'' as warning
        ,'' as error
        ,'table_measurepersoncompleteness_device_exposure' as checkid
        ,0 as is_error
        ,0 as not_applicable
        ,CASE WHEN (cte.pct_violated_rows * 100) > 100 THEN 1 ELSE 0 END as failed
        ,CASE WHEN (cte.pct_violated_rows * 100) <= 100 THEN 1 ELSE 0 END as passed
        ,NULL as not_applicable_reason
        ,100 as threshold_value
        ,NULL as notes_value
      FROM (
        
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = DEVICE_EXPOSURE
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.DEVICE_EXPOSURE cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      
      
      ) cte
      
      )
      INSERT INTO @yourResultsSchema.dqdashboard_results
      SELECT *
      FROM cte_all
      ;
      
      /*********
      SQL to insert individual DQD results directly into output table, rather than waiting until collecting all results.
      Note that this  does not include information about SQL errors or performance
      **********/
      
      WITH cte_all AS (
        /*********
      SQL to create query for insertion into results table. These may be unioned together prior to insert.
      Note that this does not include information about SQL errors or performance.
      **********/
      
      SELECT 
        cte.num_violated_rows
        ,cte.pct_violated_rows
        ,cte.num_denominator_rows
        ,'' as execution_time
        ,'' as query_text
        ,'measurePersonCompleteness' as check_name
        ,'TABLE' as check_level
        ,'The number and percent of persons in the CDM that do not have at least one record in the MEASUREMENT table' as check_description
        ,'MEASUREMENT' as cdm_table_name
        ,'NA' as cdm_field_name
        ,'NA' as concept_id
        ,'NA' as unit_concept_id
        ,'table_person_completeness.sql' as sql_file
        ,'Completeness' as category
        ,'NA' as subcategory
        ,'Validation' as context
        ,'' as warning
        ,'' as error
        ,'table_measurepersoncompleteness_measurement' as checkid
        ,0 as is_error
        ,0 as not_applicable
        ,CASE WHEN (cte.pct_violated_rows * 100) > 95 THEN 1 ELSE 0 END as failed
        ,CASE WHEN (cte.pct_violated_rows * 100) <= 95 THEN 1 ELSE 0 END as passed
        ,NULL as not_applicable_reason
        ,95 as threshold_value
        ,NULL as notes_value
      FROM (
        
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = MEASUREMENT
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.MEASUREMENT cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      
      
      ) cte
      
      )
      INSERT INTO @yourResultsSchema.dqdashboard_results
      SELECT *
      FROM cte_all
      ;
      
      /*********
      SQL to insert individual DQD results directly into output table, rather than waiting until collecting all results.
      Note that this  does not include information about SQL errors or performance
      **********/
      
      WITH cte_all AS (
        /*********
      SQL to create query for insertion into results table. These may be unioned together prior to insert.
      Note that this does not include information about SQL errors or performance.
      **********/
      
      SELECT 
        cte.num_violated_rows
        ,cte.pct_violated_rows
        ,cte.num_denominator_rows
        ,'' as execution_time
        ,'' as query_text
        ,'measurePersonCompleteness' as check_name
        ,'TABLE' as check_level
        ,'The number and percent of persons in the CDM that do not have at least one record in the OBSERVATION table' as check_description
        ,'OBSERVATION' as cdm_table_name
        ,'NA' as cdm_field_name
        ,'NA' as concept_id
        ,'NA' as unit_concept_id
        ,'table_person_completeness.sql' as sql_file
        ,'Completeness' as category
        ,'NA' as subcategory
        ,'Validation' as context
        ,'' as warning
        ,'' as error
        ,'table_measurepersoncompleteness_observation' as checkid
        ,0 as is_error
        ,0 as not_applicable
        ,CASE WHEN (cte.pct_violated_rows * 100) > 95 THEN 1 ELSE 0 END as failed
        ,CASE WHEN (cte.pct_violated_rows * 100) <= 95 THEN 1 ELSE 0 END as passed
        ,NULL as not_applicable_reason
        ,95 as threshold_value
        ,NULL as notes_value
      FROM (
        
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = OBSERVATION
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.OBSERVATION cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      
      
      ) cte
      
      )
      INSERT INTO @yourResultsSchema.dqdashboard_results
      SELECT *
      FROM cte_all
      ;
      
      /*********
      SQL to insert individual DQD results directly into output table, rather than waiting until collecting all results.
      Note that this  does not include information about SQL errors or performance
      **********/
      
      WITH cte_all AS (
        /*********
      SQL to create query for insertion into results table. These may be unioned together prior to insert.
      Note that this does not include information about SQL errors or performance.
      **********/
      
      SELECT 
        cte.num_violated_rows
        ,cte.pct_violated_rows
        ,cte.num_denominator_rows
        ,'' as execution_time
        ,'' as query_text
        ,'measurePersonCompleteness' as check_name
        ,'TABLE' as check_level
        ,'The number and percent of persons in the CDM that do not have at least one record in the DEATH table' as check_description
        ,'DEATH' as cdm_table_name
        ,'NA' as cdm_field_name
        ,'NA' as concept_id
        ,'NA' as unit_concept_id
        ,'table_person_completeness.sql' as sql_file
        ,'Completeness' as category
        ,'NA' as subcategory
        ,'Validation' as context
        ,'' as warning
        ,'' as error
        ,'table_measurepersoncompleteness_death' as checkid
        ,0 as is_error
        ,0 as not_applicable
        ,CASE WHEN (cte.pct_violated_rows * 100) > 100 THEN 1 ELSE 0 END as failed
        ,CASE WHEN (cte.pct_violated_rows * 100) <= 100 THEN 1 ELSE 0 END as passed
        ,NULL as not_applicable_reason
        ,100 as threshold_value
        ,NULL as notes_value
      FROM (
        
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = DEATH
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.DEATH cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      
      
      ) cte
      
      )
      INSERT INTO @yourResultsSchema.dqdashboard_results
      SELECT *
      FROM cte_all
      ;
      
      /*********
      SQL to insert individual DQD results directly into output table, rather than waiting until collecting all results.
      Note that this  does not include information about SQL errors or performance
      **********/
      
      WITH cte_all AS (
        /*********
      SQL to create query for insertion into results table. These may be unioned together prior to insert.
      Note that this does not include information about SQL errors or performance.
      **********/
      
      SELECT 
        cte.num_violated_rows
        ,cte.pct_violated_rows
        ,cte.num_denominator_rows
        ,'' as execution_time
        ,'' as query_text
        ,'measurePersonCompleteness' as check_name
        ,'TABLE' as check_level
        ,'The number and percent of persons in the CDM that do not have at least one record in the NOTE table' as check_description
        ,'NOTE' as cdm_table_name
        ,'NA' as cdm_field_name
        ,'NA' as concept_id
        ,'NA' as unit_concept_id
        ,'table_person_completeness.sql' as sql_file
        ,'Completeness' as category
        ,'NA' as subcategory
        ,'Validation' as context
        ,'' as warning
        ,'' as error
        ,'table_measurepersoncompleteness_note' as checkid
        ,0 as is_error
        ,0 as not_applicable
        ,CASE WHEN (cte.pct_violated_rows * 100) > 100 THEN 1 ELSE 0 END as failed
        ,CASE WHEN (cte.pct_violated_rows * 100) <= 100 THEN 1 ELSE 0 END as passed
        ,NULL as not_applicable_reason
        ,100 as threshold_value
        ,NULL as notes_value
      FROM (
        
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = NOTE
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.NOTE cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      
      
      ) cte
      
      )
      INSERT INTO @yourResultsSchema.dqdashboard_results
      SELECT *
      FROM cte_all
      ;
      
      /*********
      SQL to insert individual DQD results directly into output table, rather than waiting until collecting all results.
      Note that this  does not include information about SQL errors or performance
      **********/
      
      WITH cte_all AS (
        /*********
      SQL to create query for insertion into results table. These may be unioned together prior to insert.
      Note that this does not include information about SQL errors or performance.
      **********/
      
      SELECT 
        cte.num_violated_rows
        ,cte.pct_violated_rows
        ,cte.num_denominator_rows
        ,'' as execution_time
        ,'' as query_text
        ,'measurePersonCompleteness' as check_name
        ,'TABLE' as check_level
        ,'The number and percent of persons in the CDM that do not have at least one record in the SPECIMEN table' as check_description
        ,'SPECIMEN' as cdm_table_name
        ,'NA' as cdm_field_name
        ,'NA' as concept_id
        ,'NA' as unit_concept_id
        ,'table_person_completeness.sql' as sql_file
        ,'Completeness' as category
        ,'NA' as subcategory
        ,'Validation' as context
        ,'' as warning
        ,'' as error
        ,'table_measurepersoncompleteness_specimen' as checkid
        ,0 as is_error
        ,0 as not_applicable
        ,CASE WHEN (cte.pct_violated_rows * 100) > 100 THEN 1 ELSE 0 END as failed
        ,CASE WHEN (cte.pct_violated_rows * 100) <= 100 THEN 1 ELSE 0 END as passed
        ,NULL as not_applicable_reason
        ,100 as threshold_value
        ,NULL as notes_value
      FROM (
        
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = SPECIMEN
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.SPECIMEN cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      
      
      ) cte
      
      )
      INSERT INTO @yourResultsSchema.dqdashboard_results
      SELECT *
      FROM cte_all
      ;
      
      /*********
      SQL to insert individual DQD results directly into output table, rather than waiting until collecting all results.
      Note that this  does not include information about SQL errors or performance
      **********/
      
      WITH cte_all AS (
        /*********
      SQL to create query for insertion into results table. These may be unioned together prior to insert.
      Note that this does not include information about SQL errors or performance.
      **********/
      
      SELECT 
        cte.num_violated_rows
        ,cte.pct_violated_rows
        ,cte.num_denominator_rows
        ,'' as execution_time
        ,'' as query_text
        ,'measurePersonCompleteness' as check_name
        ,'TABLE' as check_level
        ,'The number and percent of persons in the CDM that do not have at least one record in the PAYER_PLAN_PERIOD table' as check_description
        ,'PAYER_PLAN_PERIOD' as cdm_table_name
        ,'NA' as cdm_field_name
        ,'NA' as concept_id
        ,'NA' as unit_concept_id
        ,'table_person_completeness.sql' as sql_file
        ,'Completeness' as category
        ,'NA' as subcategory
        ,'Validation' as context
        ,'' as warning
        ,'' as error
        ,'table_measurepersoncompleteness_payer_plan_period' as checkid
        ,0 as is_error
        ,0 as not_applicable
        ,CASE WHEN (cte.pct_violated_rows * 100) > 100 THEN 1 ELSE 0 END as failed
        ,CASE WHEN (cte.pct_violated_rows * 100) <= 100 THEN 1 ELSE 0 END as passed
        ,NULL as not_applicable_reason
        ,100 as threshold_value
        ,NULL as notes_value
      FROM (
        
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = PAYER_PLAN_PERIOD
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.PAYER_PLAN_PERIOD cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      
      
      ) cte
      
      )
      INSERT INTO @yourResultsSchema.dqdashboard_results
      SELECT *
      FROM cte_all
      ;
      
      /*********
      SQL to insert individual DQD results directly into output table, rather than waiting until collecting all results.
      Note that this  does not include information about SQL errors or performance
      **********/
      
      WITH cte_all AS (
        /*********
      SQL to create query for insertion into results table. These may be unioned together prior to insert.
      Note that this does not include information about SQL errors or performance.
      **********/
      
      SELECT 
        cte.num_violated_rows
        ,cte.pct_violated_rows
        ,cte.num_denominator_rows
        ,'' as execution_time
        ,'' as query_text
        ,'measurePersonCompleteness' as check_name
        ,'TABLE' as check_level
        ,'The number and percent of persons in the CDM that do not have at least one record in the DRUG_ERA table' as check_description
        ,'DRUG_ERA' as cdm_table_name
        ,'NA' as cdm_field_name
        ,'NA' as concept_id
        ,'NA' as unit_concept_id
        ,'table_person_completeness.sql' as sql_file
        ,'Completeness' as category
        ,'NA' as subcategory
        ,'Validation' as context
        ,'' as warning
        ,'' as error
        ,'table_measurepersoncompleteness_drug_era' as checkid
        ,0 as is_error
        ,0 as not_applicable
        ,CASE WHEN (cte.pct_violated_rows * 100) > 95 THEN 1 ELSE 0 END as failed
        ,CASE WHEN (cte.pct_violated_rows * 100) <= 95 THEN 1 ELSE 0 END as passed
        ,NULL as not_applicable_reason
        ,95 as threshold_value
        ,NULL as notes_value
      FROM (
        
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = DRUG_ERA
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.DRUG_ERA cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      
      
      ) cte
      
      )
      INSERT INTO @yourResultsSchema.dqdashboard_results
      SELECT *
      FROM cte_all
      ;
      
      /*********
      SQL to insert individual DQD results directly into output table, rather than waiting until collecting all results.
      Note that this  does not include information about SQL errors or performance
      **********/
      
      WITH cte_all AS (
        /*********
      SQL to create query for insertion into results table. These may be unioned together prior to insert.
      Note that this does not include information about SQL errors or performance.
      **********/
      
      SELECT 
        cte.num_violated_rows
        ,cte.pct_violated_rows
        ,cte.num_denominator_rows
        ,'' as execution_time
        ,'' as query_text
        ,'measurePersonCompleteness' as check_name
        ,'TABLE' as check_level
        ,'The number and percent of persons in the CDM that do not have at least one record in the DOSE_ERA table' as check_description
        ,'DOSE_ERA' as cdm_table_name
        ,'NA' as cdm_field_name
        ,'NA' as concept_id
        ,'NA' as unit_concept_id
        ,'table_person_completeness.sql' as sql_file
        ,'Completeness' as category
        ,'NA' as subcategory
        ,'Validation' as context
        ,'' as warning
        ,'' as error
        ,'table_measurepersoncompleteness_dose_era' as checkid
        ,0 as is_error
        ,0 as not_applicable
        ,CASE WHEN (cte.pct_violated_rows * 100) > 100 THEN 1 ELSE 0 END as failed
        ,CASE WHEN (cte.pct_violated_rows * 100) <= 100 THEN 1 ELSE 0 END as passed
        ,NULL as not_applicable_reason
        ,100 as threshold_value
        ,NULL as notes_value
      FROM (
        
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = DOSE_ERA
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.DOSE_ERA cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      
      
      ) cte
      
      )
      INSERT INTO @yourResultsSchema.dqdashboard_results
      SELECT *
      FROM cte_all
      ;
      
      /*********
      SQL to insert individual DQD results directly into output table, rather than waiting until collecting all results.
      Note that this  does not include information about SQL errors or performance
      **********/
      
      WITH cte_all AS (
        /*********
      SQL to create query for insertion into results table. These may be unioned together prior to insert.
      Note that this does not include information about SQL errors or performance.
      **********/
      
      SELECT 
        cte.num_violated_rows
        ,cte.pct_violated_rows
        ,cte.num_denominator_rows
        ,'' as execution_time
        ,'' as query_text
        ,'measurePersonCompleteness' as check_name
        ,'TABLE' as check_level
        ,'The number and percent of persons in the CDM that do not have at least one record in the CONDITION_ERA table' as check_description
        ,'CONDITION_ERA' as cdm_table_name
        ,'NA' as cdm_field_name
        ,'NA' as concept_id
        ,'NA' as unit_concept_id
        ,'table_person_completeness.sql' as sql_file
        ,'Completeness' as category
        ,'NA' as subcategory
        ,'Validation' as context
        ,'' as warning
        ,'' as error
        ,'table_measurepersoncompleteness_condition_era' as checkid
        ,0 as is_error
        ,0 as not_applicable
        ,CASE WHEN (cte.pct_violated_rows * 100) > 95 THEN 1 ELSE 0 END as failed
        ,CASE WHEN (cte.pct_violated_rows * 100) <= 95 THEN 1 ELSE 0 END as passed
        ,NULL as not_applicable_reason
        ,95 as threshold_value
        ,NULL as notes_value
      FROM (
        
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = CONDITION_ERA
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.CONDITION_ERA cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      
      
      ) cte
      
      )
      INSERT INTO @yourResultsSchema.dqdashboard_results
      SELECT *
      FROM cte_all
      ;
      

# Execute DQ checks using sqlOnly=TRUE and sqlOnlyUnionCount=1 and sqlOnlyIncrementalInsert=FALSE (the behavior in version <= 2.2.0)

    Code
      cat(SqlRender::readSql(dqdSqlFilePath))
    Output
      
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = OBSERVATION_PERIOD
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.OBSERVATION_PERIOD cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      ;
      
      
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = VISIT_OCCURRENCE
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.VISIT_OCCURRENCE cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      ;
      
      
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = VISIT_DETAIL
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.VISIT_DETAIL cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      ;
      
      
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = CONDITION_OCCURRENCE
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.CONDITION_OCCURRENCE cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      ;
      
      
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = DRUG_EXPOSURE
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.DRUG_EXPOSURE cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      ;
      
      
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = PROCEDURE_OCCURRENCE
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.PROCEDURE_OCCURRENCE cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      ;
      
      
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = DEVICE_EXPOSURE
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.DEVICE_EXPOSURE cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      ;
      
      
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = MEASUREMENT
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.MEASUREMENT cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      ;
      
      
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = OBSERVATION
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.OBSERVATION cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      ;
      
      
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = DEATH
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.DEATH cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      ;
      
      
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = NOTE
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.NOTE cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      ;
      
      
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = SPECIMEN
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.SPECIMEN cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      ;
      
      
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = PAYER_PLAN_PERIOD
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.PAYER_PLAN_PERIOD cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      ;
      
      
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = DRUG_ERA
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.DRUG_ERA cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      ;
      
      
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = DOSE_ERA
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.DOSE_ERA cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      ;
      
      
      /*********
      Table Level:  
      MEASURE_PERSON_COMPLETENESS
      Determine what #/% of persons have at least one record in the cdmTable
      
      Parameters used in this template:
      schema = @yourCdmSchema
      cdmTableName = CONDITION_ERA
      
      **********/
      
      
      SELECT 
      	num_violated_rows, 
      	CASE 
      		WHEN denominator.num_rows = 0 THEN 0 
      		ELSE 1.0*num_violated_rows/denominator.num_rows 
      	END AS pct_violated_rows, 
          denominator.num_rows AS num_denominator_rows
      FROM
      (
      	SELECT 
      		COUNT_BIG(violated_rows.person_id) AS num_violated_rows
      	FROM
      	(
      		/*violatedRowsBegin*/
      		SELECT 
      			cdmTable.* 
      		FROM @yourCdmSchema.person cdmTable
      		
      			LEFT JOIN @yourCdmSchema.CONDITION_ERA cdmTable2 
      			    ON cdmTable.person_id = cdmTable2.person_id
      		WHERE cdmTable2.person_id IS NULL
      		/*violatedRowsEnd*/
      	) violated_rows
      ) violated_row_count,
      ( 
      	SELECT 
      		COUNT_BIG(*) AS num_rows
      	FROM @yourCdmSchema.person cdmTable
      	
      ) denominator
      ;
      

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/executeDqChecks.R
\name{.needsAutoCommit}
\alias{.needsAutoCommit}
\title{Internal function to determine if the connection needs auto commit}
\usage{
.needsAutoCommit(connectionDetails, connection)
}
\arguments{
\item{connectionDetails}{A connectionDetails object for connecting to the CDM database}

\item{connection}{A connection for connecting to the CDM database using the DatabaseConnector::connect(connectionDetails) function.}
}
\value{
A logical value indicating if the connection needs auto commit
}
\description{
Internal function to determine if the connection needs auto commit
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/writeResultsTo.R
\name{.writeResultsToTable}
\alias{.writeResultsToTable}
\title{Internal function to write the check results to a table in the database. Requires write access to the database}
\usage{
.writeResultsToTable(
  connectionDetails,
  resultsDatabaseSchema,
  checkResults,
  writeTableName,
  cohortDefinitionId
)
}
\arguments{
\item{connectionDetails}{A connectionDetails object for connecting to the CDM database}

\item{resultsDatabaseSchema}{The fully qualified database name of the results schema}

\item{checkResults}{A dataframe containing the fully summarized data quality check results}

\item{writeTableName}{The name of the table to be written to the database. Default is "dqdashboard_results".}

\item{cohortDefinitionId}{(OPTIONAL) The cohort definition id for the cohort you wish to run the DQD on. The package assumes a standard OHDSI cohort table called 'Cohort'
with the fields cohort_definition_id and subject_id.}
}
\value{
NULL (writes results to database table)
}
\description{
Internal function to write the check results to a table in the database. Requires write access to the database
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recordResult.R
\name{.recordResult}
\alias{.recordResult}
\title{Internal function to put the results of each quality check into a dataframe.}
\usage{
.recordResult(
  result = NULL,
  check,
  checkDescription,
  sql,
  executionTime = NA,
  warning = NA,
  error = NA
)
}
\arguments{
\item{result}{The result of the data quality check}

\item{check}{The data quality check}

\item{checkDescription}{The description of the data quality check}

\item{sql}{The fully qualified sql for the data quality check}

\item{executionTime}{The total time it took to execute the data quality check}

\item{warning}{Any warnings returned from the server}

\item{error}{Any errors returned from the server}
}
\value{
A dataframe containing the check results
}
\description{
Internal function to put the results of each quality check into a dataframe.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summarizeResults.R
\name{.summarizeResults}
\alias{.summarizeResults}
\title{Internal function to summarize the results of the DQD run.}
\usage{
.summarizeResults(checkResults)
}
\arguments{
\item{checkResults}{A dataframe containing the results of the checks after running against the database}
}
\value{
A list containing summary statistics of the check results
}
\description{
Internal function to summarize the results of the DQD run.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getCheckId.R
\name{.getCheckId}
\alias{.getCheckId}
\title{Internal function to define the id of each check.}
\usage{
.getCheckId(
  checkLevel,
  checkName,
  cdmTableName,
  cdmFieldName = NA,
  conceptId = NA,
  unitConceptId = NA
)
}
\arguments{
\item{checkLevel}{The level of the check. Options are table, field, or concept}

\item{checkName}{The name of the data quality check}

\item{cdmTableName}{The name of the CDM data table the quality check is applied to}

\item{cdmFieldName}{The name of the field in the CDM data table the quality check is applied to}

\item{conceptId}{The concept id the quality check is applied to}

\item{unitConceptId}{The unit concept id the quality check is applied to}
}
\value{
A character string representing the unique check ID
}
\description{
Internal function to define the id of each check.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/writeResultsTo.R
\name{.writeResultsToCsv}
\alias{.writeResultsToCsv}
\title{Internal function to write the check results to a csv file.}
\usage{
.writeResultsToCsv(
  checkResults,
  csvPath,
  columns = c("checkId", "failed", "passed", "isError", "notApplicable", "checkName",
    "checkDescription", "thresholdValue", "notesValue", "checkLevel", "category",
    "subcategory", "context", "checkLevel", "cdmTableName", "cdmFieldName", "conceptId",
    "unitConceptId", "numViolatedRows", "pctViolatedRows", "numDenominatorRows",
    "executionTime", "notApplicableReason", "error", "queryText"),
  delimiter = ","
)
}
\arguments{
\item{checkResults}{A dataframe containing the fully summarized data quality check results}

\item{csvPath}{The path where the csv file should be written}

\item{columns}{The columns to be included in the csv file. Default is all columns in the checkResults dataframe.}

\item{delimiter}{The delimiter for the file. Default is comma.}
}
\value{
NULL (writes results to CSV file)
}
\description{
Internal function to write the check results to a csv file.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/executeDqChecks.R
\name{executeDqChecks}
\alias{executeDqChecks}
\title{Execute DQ checks}
\usage{
executeDqChecks(
  connectionDetails,
  cdmDatabaseSchema,
  resultsDatabaseSchema,
  vocabDatabaseSchema = cdmDatabaseSchema,
  cdmSourceName,
  numThreads = 1,
  sqlOnly = FALSE,
  sqlOnlyUnionCount = 1,
  sqlOnlyIncrementalInsert = FALSE,
  outputFolder,
  outputFile = "",
  verboseMode = FALSE,
  writeToTable = TRUE,
  writeTableName = "dqdashboard_results",
  writeToCsv = FALSE,
  csvFile = "",
  checkLevels = c("TABLE", "FIELD", "CONCEPT"),
  checkNames = c(),
  checkSeverity = c("fatal", "convention", "characterization"),
  cohortDefinitionId = c(),
  cohortDatabaseSchema = resultsDatabaseSchema,
  cohortTableName = "cohort",
  tablesToExclude = c("CONCEPT", "VOCABULARY", "CONCEPT_ANCESTOR",
    "CONCEPT_RELATIONSHIP", "CONCEPT_CLASS", "CONCEPT_SYNONYM", "RELATIONSHIP", "DOMAIN"),
  cdmVersion = "5.3",
  tableCheckThresholdLoc = "default",
  fieldCheckThresholdLoc = "default",
  conceptCheckThresholdLoc = "default"
)
}
\arguments{
\item{connectionDetails}{A connectionDetails object for connecting to the CDM database}

\item{cdmDatabaseSchema}{The fully qualified database name of the CDM schema}

\item{resultsDatabaseSchema}{The fully qualified database name of the results schema}

\item{vocabDatabaseSchema}{The fully qualified database name of the vocabulary schema (default is to set it as the cdmDatabaseSchema)}

\item{cdmSourceName}{The name of the CDM data source}

\item{numThreads}{The number of concurrent threads to use to execute the queries}

\item{sqlOnly}{Should the SQLs be executed (FALSE) or just returned (TRUE)?}

\item{sqlOnlyUnionCount}{(OPTIONAL) In sqlOnlyIncrementalInsert mode, how many SQL commands to union in each query to insert check results into results table (can speed processing when queries done in parallel). Default is 1.}

\item{sqlOnlyIncrementalInsert}{(OPTIONAL) In sqlOnly mode, boolean to determine whether to generate SQL queries that insert check results and associated metadata into results table.  Default is FALSE (for backwards compatibility to <= v2.2.0)}

\item{outputFolder}{The folder to output logs, SQL files, and JSON results file to}

\item{outputFile}{(OPTIONAL) File to write results JSON object}

\item{verboseMode}{Boolean to determine if the console will show all execution steps. Default is FALSE}

\item{writeToTable}{Boolean to indicate if the check results will be written to the dqdashboard_results table in the resultsDatabaseSchema. Default is TRUE}

\item{writeTableName}{The name of the results table. Defaults to `dqdashboard_results`.  Used when sqlOnly or writeToTable is True.}

\item{writeToCsv}{Boolean to indicate if the check results will be written to a csv file. Default is FALSE}

\item{csvFile}{(OPTIONAL) CSV file to write results}

\item{checkLevels}{Choose which DQ check levels to execute. Default is all 3 (TABLE, FIELD, CONCEPT)}

\item{checkNames}{(OPTIONAL) Choose which check names to execute. Names can be found in inst/csv/OMOP_CDM_v[cdmVersion]_Check_Descriptions.csv. Note that "cdmTable", "cdmField" and "measureValueCompleteness" are always executed.}

\item{checkSeverity}{Choose which DQ check severity levels to execute. Default is all 3 (fatal, convention, characterization)}

\item{cohortDefinitionId}{The cohort definition id for the cohort you wish to run the DQD on. The package assumes a standard OHDSI cohort table
with the fields cohort_definition_id and subject_id.}

\item{cohortDatabaseSchema}{The schema where the cohort table is located.}

\item{cohortTableName}{The name of the cohort table. Defaults to `cohort`.}

\item{tablesToExclude}{(OPTIONAL) Choose which CDM tables to exclude from the execution.}

\item{cdmVersion}{The CDM version to target for the data source. Options are "5.2", "5.3", or "5.4". By default, "5.3" is used.}

\item{tableCheckThresholdLoc}{The location of the threshold file for evaluating the table checks. If not specified the default thresholds will be applied.}

\item{fieldCheckThresholdLoc}{The location of the threshold file for evaluating the field checks. If not specified the default thresholds will be applied.}

\item{conceptCheckThresholdLoc}{The location of the threshold file for evaluating the concept checks. If not specified the default thresholds will be applied.}
}
\value{
A list object of results
}
\description{
This function will connect to the database, generate the sql scripts, and run the data quality checks against the database. By default, results will be written to a json file as well as a database table.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateNotApplicableStatus.R
\name{.applyNotApplicable}
\alias{.applyNotApplicable}
\title{Applies the 'Not Applicable' status to a single check}
\usage{
.applyNotApplicable(x)
}
\arguments{
\item{x}{Results from a single check}
}
\value{
A numeric value (0 or 1) indicating whether the check is not applicable
}
\description{
Applies the 'Not Applicable' status to a single check
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/writeJsonResultsTo.R
\name{writeJsonResultsToCsv}
\alias{writeJsonResultsToCsv}
\title{Write JSON Results to CSV file}
\usage{
writeJsonResultsToCsv(
  jsonPath,
  csvPath,
  columns = c("checkId", "failed", "passed", "isError", "notApplicable", "checkName",
    "checkDescription", "thresholdValue", "notesValue", "checkLevel", "category",
    "subcategory", "context", "checkLevel", "cdmTableName", "cdmFieldName", "conceptId",
    "unitConceptId", "numViolatedRows", "pctViolatedRows", "numDenominatorRows",
    "executionTime", "notApplicableReason", "error", "queryText"),
  delimiter = ","
)
}
\arguments{
\item{jsonPath}{Path to the JSON results file generated using the execute function}

\item{csvPath}{Path to the CSV output file}

\item{columns}{(OPTIONAL) List of desired columns}

\item{delimiter}{(OPTIONAL) CSV delimiter}
}
\value{
NULL (writes results to CSV file)
}
\description{
Write JSON Results to CSV file
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateNotApplicableStatus.R
\name{.containsNAchecks}
\alias{.containsNAchecks}
\title{Determines if all checks required for 'Not Applicable' status are in the checkNames}
\usage{
.containsNAchecks(checkNames)
}
\arguments{
\item{checkNames}{A character vector of check names}
}
\value{
A logical value indicating whether all required checks are present
}
\description{
Determines if all checks required for 'Not Applicable' status are in the checkNames
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/writeDBResultsTo.R
\name{writeDBResultsToJson}
\alias{writeDBResultsToJson}
\title{Write DQD results database table to json}
\usage{
writeDBResultsToJson(
  connection,
  resultsDatabaseSchema,
  cdmDatabaseSchema,
  writeTableName,
  outputFolder,
  outputFile
)
}
\arguments{
\item{connection}{A connection object}

\item{resultsDatabaseSchema}{The fully qualified database name of the results schema}

\item{cdmDatabaseSchema}{The fully qualified database name of the CDM schema}

\item{writeTableName}{Name of DQD results table in the database to read from}

\item{outputFolder}{The folder to output the json results file to}

\item{outputFile}{The output filename of the json results file}
}
\value{
NULL (writes results to JSON file)
}
\description{
Write DQD results database table to json
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateNotApplicableStatus.R
\name{.hasNAchecks}
\alias{.hasNAchecks}
\title{Determines if all checks are present expected to calculate the 'Not Applicable' status}
\usage{
.hasNAchecks(checkResults)
}
\arguments{
\item{checkResults}{A dataframe containing the results of the data quality checks}
}
\value{
A logical value indicating whether all required checks are present
}
\description{
Determines if all checks are present expected to calculate the 'Not Applicable' status
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/view.R
\name{viewDqDashboard}
\alias{viewDqDashboard}
\title{View DQ Dashboard}
\usage{
viewDqDashboard(jsonPath, launch.browser = NULL, display.mode = NULL, ...)
}
\arguments{
\item{jsonPath}{The fully-qualified path to the JSON file produced by  \code{\link{executeDqChecks}}}

\item{launch.browser}{Passed on to \code{shiny::runApp}}

\item{display.mode}{Passed on to \code{shiny::runApp}}

\item{...}{Extra parameters for shiny::runApp() like "port" or "host"}
}
\value{
NULL (launches Shiny application)
}
\description{
View DQ Dashboard
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/listChecks.R
\name{listDqChecks}
\alias{listDqChecks}
\title{List DQ checks}
\usage{
listDqChecks(
  cdmVersion = "5.3",
  tableCheckThresholdLoc = "default",
  fieldCheckThresholdLoc = "default",
  conceptCheckThresholdLoc = "default"
)
}
\arguments{
\item{cdmVersion}{The CDM version to target for the data source. By default, 5.3 is used.}

\item{tableCheckThresholdLoc}{The location of the threshold file for evaluating the table checks. If not specified the default thresholds will be applied.}

\item{fieldCheckThresholdLoc}{The location of the threshold file for evaluating the field checks. If not specified the default thresholds will be applied.}

\item{conceptCheckThresholdLoc}{The location of the threshold file for evaluating the concept checks. If not specified the default thresholds will be applied.}
}
\value{
A list containing check descriptions, table checks, field checks, and concept checks
}
\description{
Details on all checks defined by the DataQualityDashboard Package.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convertResultsCase.R
\name{convertJsonResultsFileCase}
\alias{convertJsonResultsFileCase}
\title{Convert JSON results file case}
\usage{
convertJsonResultsFileCase(
  jsonFilePath,
  writeToFile,
  outputFolder = NA,
  outputFile = "",
  targetCase
)
}
\arguments{
\item{jsonFilePath}{Path to the JSON results file to be converted}

\item{writeToFile}{Whether or not to write the converted results back to a file (must be either TRUE or FALSE)}

\item{outputFolder}{The folder to output the converted JSON results file to}

\item{outputFile}{(OPTIONAL) File to write converted results JSON object to. Default is name of input file with a "_camel" or "_snake" postfix}

\item{targetCase}{Case into which the results file parameters should be converted (must be either "camel" or "snake")}
}
\value{
DQD results object (a named list)
}
\description{
Convert a DQD JSON results file between camelcase and (all-caps) snakecase. Enables viewing of pre-v.2.1.0 results files in later DQD versions, and vice versa
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/evaluateThresholds.R
\name{.evaluateThresholds}
\alias{.evaluateThresholds}
\title{Internal function to evaluate the data quality checks against given thresholds.}
\usage{
.evaluateThresholds(checkResults, tableChecks, fieldChecks, conceptChecks)
}
\arguments{
\item{checkResults}{A dataframe containing the results of the data quality checks}

\item{tableChecks}{A dataframe containing the table checks}

\item{fieldChecks}{A dataframe containing the field checks}

\item{conceptChecks}{A dataframe containing the concept checks}
}
\value{
A dataframe with updated check results including pass/fail status and threshold values
}
\description{
Internal function to evaluate the data quality checks against given thresholds.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateNotApplicableStatus.R
\name{.calculateNotApplicableStatus}
\alias{.calculateNotApplicableStatus}
\title{Determines if check should be notApplicable and the notApplicableReason}
\usage{
.calculateNotApplicableStatus(checkResults)
}
\arguments{
\item{checkResults}{A dataframe containing the results of the data quality checks}
}
\value{
A dataframe with updated check results including notApplicable status and reasons
}
\description{
Determines if check should be notApplicable and the notApplicableReason
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readThresholdFile.R
\name{.readThresholdFile}
\alias{.readThresholdFile}
\title{Internal function to read threshold files}
\usage{
.readThresholdFile(checkThresholdLoc, defaultLoc)
}
\arguments{
\item{checkThresholdLoc}{The location of the threshold file}

\item{defaultLoc}{The default location of the threshold file}
}
\value{
A dataframe containing the threshold data
}
\description{
Internal function to read threshold files
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/writeResultsTo.R
\name{.writeResultsToJson}
\alias{.writeResultsToJson}
\title{Write DQD results to json}
\usage{
.writeResultsToJson(result, outputFolder, outputFile)
}
\arguments{
\item{result}{A DQD results object (list)}

\item{outputFolder}{The output folder}

\item{outputFile}{The output filename}
}
\value{
NULL (writes results to JSON file)
}
\description{
Write DQD results to json
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/processCheck.R
\name{.processCheck}
\alias{.processCheck}
\title{Internal function to send the fully qualified sql to the database and return the numerical result.}
\usage{
.processCheck(
  connection,
  connectionDetails,
  check,
  checkDescription,
  sql,
  outputFolder
)
}
\arguments{
\item{connection}{A connection for connecting to the CDM database using the DatabaseConnector::connect(connectionDetails) function.}

\item{connectionDetails}{A connectionDetails object for connecting to the CDM database.}

\item{check}{The data quality check}

\item{checkDescription}{The description of the data quality check}

\item{sql}{The fully qualified sql for the data quality check}

\item{outputFolder}{The folder to output logs and SQL files to.}
}
\value{
A dataframe containing the check results
}
\description{
Internal function to send the fully qualified sql to the database and return the numerical result.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reEvaluateThresholds.R
\name{reEvaluateThresholds}
\alias{reEvaluateThresholds}
\title{Re-evaluate Thresholds}
\usage{
reEvaluateThresholds(
  jsonFilePath,
  outputFolder,
  outputFile,
  tableCheckThresholdLoc = "default",
  fieldCheckThresholdLoc = "default",
  conceptCheckThresholdLoc = "default",
  cdmVersion = "5.3"
)
}
\arguments{
\item{jsonFilePath}{Path to the JSON results file generated using the execute function}

\item{outputFolder}{The folder to output new JSON result file to}

\item{outputFile}{File to write results JSON object to}

\item{tableCheckThresholdLoc}{The location of the threshold file for evaluating the table checks. If not specified the default thresholds will be applied.}

\item{fieldCheckThresholdLoc}{The location of the threshold file for evaluating the field checks. If not specified the default thresholds will be applied.}

\item{conceptCheckThresholdLoc}{The location of the threshold file for evaluating the concept checks. If not specified the default thresholds will be applied.}

\item{cdmVersion}{The CDM version to target for the data source. By default, 5.3 is used.}
}
\value{
A list containing the re-evaluated DQD results
}
\description{
Re-evaluate an existing DQD result against an updated thresholds file.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runCheck.R
\name{.runCheck}
\alias{.runCheck}
\title{Internal function to run and process each data quality check.}
\usage{
.runCheck(
  checkDescription,
  tableChecks,
  fieldChecks,
  conceptChecks,
  connectionDetails,
  connection,
  cdmDatabaseSchema,
  vocabDatabaseSchema,
  resultsDatabaseSchema,
  writeTableName,
  cohortDatabaseSchema,
  cohortTableName,
  cohortDefinitionId,
  outputFolder,
  sqlOnlyUnionCount,
  sqlOnlyIncrementalInsert,
  sqlOnly
)
}
\arguments{
\item{checkDescription}{The description of the data quality check}

\item{tableChecks}{A dataframe containing the table checks}

\item{fieldChecks}{A dataframe containing the field checks}

\item{conceptChecks}{A dataframe containing the concept checks}

\item{connectionDetails}{A connectionDetails object for connecting to the CDM database}

\item{connection}{A connection for connecting to the CDM database using the DatabaseConnector::connect(connectionDetails) function.}

\item{cdmDatabaseSchema}{The fully qualified database name of the CDM schema}

\item{vocabDatabaseSchema}{The fully qualified database name of the vocabulary schema (default is to set it as the cdmDatabaseSchema)}

\item{resultsDatabaseSchema}{The fully qualified database name of the results schema}

\item{writeTableName}{The table tor write DQD results to. Used when sqlOnly or writeToTable is True.}

\item{cohortDatabaseSchema}{The schema where the cohort table is located.}

\item{cohortTableName}{The name of the cohort table.}

\item{cohortDefinitionId}{The cohort definition id for the cohort you wish to run the DQD on. The package assumes a standard OHDSI cohort table called 'Cohort'}

\item{outputFolder}{The folder to output logs and SQL files to}

\item{sqlOnlyUnionCount}{(OPTIONAL) How many SQL commands to union before inserting them into output table (speeds processing when queries done in parallel). Default is 1.}

\item{sqlOnlyIncrementalInsert}{(OPTIONAL) Boolean to determine whether insert check results and associated metadata into output table.  Default is FALSE (for backwards compatability to <= v2.2.0)}

\item{sqlOnly}{Should the SQLs be executed (FALSE) or just returned (TRUE)?}
}
\value{
A dataframe containing the check results or SQL queries (NULL if sqlOnlyIncrementalInsert is TRUE)
}
\description{
Internal function to run and process each data quality check.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/writeJsonResultsTo.R
\name{writeJsonResultsToTable}
\alias{writeJsonResultsToTable}
\title{Write JSON Results to SQL Table}
\usage{
writeJsonResultsToTable(
  connectionDetails,
  resultsDatabaseSchema,
  jsonFilePath,
  writeTableName = "dqdashboard_results",
  cohortDefinitionId = c(),
  singleTable = FALSE
)
}
\arguments{
\item{connectionDetails}{A connectionDetails object for connecting to the CDM database}

\item{resultsDatabaseSchema}{The fully qualified database name of the results schema}

\item{jsonFilePath}{Path to the JSON results file generated using the execute function}

\item{writeTableName}{Name of table in the database to write results to}

\item{cohortDefinitionId}{If writing results for a single cohort this is the ID that will be appended to the table name}

\item{singleTable}{If TRUE, writes all results to a single table. If FALSE (default), writes to 3 separate tables by check level (table, field, concept) (NOTE this default behavior will be deprecated in the future)}
}
\value{
NULL (writes results to database table)
}
\description{
Write JSON Results to SQL Table
}
